/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.adapter;

import java.lang.ref.WeakReference;
import java.util.concurrent.Callable;

import android.content.Context;
import android.database.Cursor;
import android.graphics.Typeface;
import android.os.AsyncTask;
import android.support.v4.widget.CursorAdapter;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.ubuntuone.android.music.R;
import com.ubuntuone.android.music.adapter.holder.PlaylistViewHolder;
import com.ubuntuone.android.music.provider.MusicContract.Playlists;
import com.ubuntuone.android.music.provider.MusicProviderUtils;

public class U1PlaylistAdapter extends CursorAdapter
{
	private Context mContext;
	private LayoutInflater mInflater;
	
	private boolean mIncludeCustom = false;
	
	private CountSongsTask mCountFavoriteSongsTask;
	private CountSongsTask mCountCachedSongsTask;
	private CountSongsTask mCountPlayQueueSongsTask;
	
	public interface Position {
		int STARRED = 0;
		int CACHED = 1;
		int QUEUED = 2;
		int COUNT = 3;
	}
	
	public U1PlaylistAdapter(Context context, Cursor c, boolean includeCustom) {
		super(context, c, true);
		mContext = context;
		mInflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		mIncludeCustom = includeCustom;
	}
	
	@Override
	public int getCount() {
		if (mIncludeCustom) {
			return Position.COUNT + super.getCount(); // Favs, Cached, Queued
		} else {
			return super.getCount();
		}
	}
	
	@Override
	public View getView(int position, View convertView, ViewGroup parent) {
		View view;
		PlaylistViewHolder holder;
		if (convertView == null) {
			view = mInflater.inflate(R.layout.list_item_playlist, null);
			holder = new PlaylistViewHolder(view);
			view.setTag(holder);
		} else {
			view = convertView;
			holder = (PlaylistViewHolder) convertView.getTag();
		}
		
		if (mIncludeCustom && position < Position.COUNT) {
			holder.playlistTitle.setTypeface(null, Typeface.BOLD);
			
			switch (position) {
			case Position.STARRED:
				bindFavoritedView(holder);
				break;
			case Position.CACHED:
				bindCachedView(holder);
				break;
			case Position.QUEUED:
				bindQueuedView(holder);
				break;
			}
		} else {
			holder.playlistTitle.setTypeface(null, Typeface.NORMAL);
			
			Cursor cursor = getCursor();
			if (mIncludeCustom) position -= Position.COUNT;
			cursor.moveToPosition(position);
			bindView(view, mContext, cursor);
		}
		
		return view;
	}
	
	@SuppressWarnings("unchecked")
	private void bindFavoritedView(PlaylistViewHolder holder) {
		holder.playlistTitle.setText("Favorites");
		Callable<Integer> count = new Callable<Integer>() {
			@Override
			public Integer call() throws Exception {
				return MusicProviderUtils.getStarredSongCount(mContext);
			}
		};
		mCountFavoriteSongsTask = new CountSongsTask(holder.playlistSongCount);
		mCountFavoriteSongsTask.execute(count);
	}
	
	@SuppressWarnings("unchecked")
	private void bindCachedView(PlaylistViewHolder holder) {
		holder.playlistTitle.setText("Available off-line");
		Callable<Integer> count = new Callable<Integer>() {
			@Override
			public Integer call() throws Exception {
				return MusicProviderUtils.getCachedSongCount(mContext);
			}
		};
		mCountCachedSongsTask = new CountSongsTask(holder.playlistSongCount);
		mCountCachedSongsTask.execute(count);
	}
	
	@SuppressWarnings("unchecked")
	private void bindQueuedView(PlaylistViewHolder holder) {
		holder.playlistTitle.setText("Play queue");
		Callable<Integer> count = new Callable<Integer>() {
			@Override
			public Integer call() throws Exception {
				return MusicProviderUtils.getPlayQueueSongCount(mContext);
			}
		};
		mCountPlayQueueSongsTask = new CountSongsTask(holder.playlistSongCount);
		mCountPlayQueueSongsTask.execute(count);
	}
	
	@Override
	public void bindView(View view, Context context, Cursor cursor) {
		PlaylistViewHolder holder = (PlaylistViewHolder) view.getTag();
		
		String name = cursor.getString(cursor.getColumnIndex(Playlists.PLAYLIST_NAME));
		int length = cursor.getInt(cursor.getColumnIndex(Playlists.PLAYLIST_SONG_COUNT));
		int dirty = cursor.getInt(cursor.getColumnIndex(Playlists.DIRTY));
		if (dirty != 0) {
			holder.playlistTitle.setText(String.format("%s [%s]",
					name, context.getString(R.string.sync_pending)));
		} else {
			holder.playlistTitle.setText(name);
		}
		// TODO Get plural string resource for item(s).
		if (length == 0) {
			holder.playlistSongCount.setText(String.format("No songs", length));
		} else if (length == 1) {
			holder.playlistSongCount.setText(String.format("%s song", length));
		} else {
			holder.playlistSongCount.setText(String.format("%s songs", length));			
		}
	}
	
	@Override
	public View newView(Context context, Cursor cursor, ViewGroup parent) {
		// Nothing to do.
		return null;
	}
	
	private class CountSongsTask extends AsyncTask<Callable<Integer>, Void, Integer> {
		private WeakReference<TextView> mTextViewRef;
		
		public CountSongsTask(TextView textView) {
			mTextViewRef = new WeakReference<TextView>(textView);
		}
		
		@Override
		protected Integer doInBackground(Callable<Integer>... params) {
			if (params.length != 1) {
				throw new IllegalArgumentException("One callable has to be passed.");
			}
			Callable<Integer> countTask = params[0];
			try {
				return countTask.call();
			} catch (Exception e) {
				// In this case, -1 is returned as the result could not be computed.
			}
			return -1;
		}

		@Override
		protected void onPostExecute(Integer result) {
			super.onPostExecute(result);
			TextView view = mTextViewRef.get();
			if (view != null) {
				view.setText(mContext.getString(R.string.playlist_stats, result));
			}
		}
	}
}
