/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.model;

import static com.ubuntuone.android.music.util.LogUtils.makeLogTag;

import java.io.File;

import android.content.ContentResolver;
import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Environment;
import android.os.Handler;
import android.util.Log;

import com.ubuntuone.android.music.provider.MusicContract.Playlists;
import com.ubuntuone.android.music.provider.MusicContract.Songs;
import com.ubuntuone.android.music.provider.MusicProviderUtils;
import com.ubuntuone.android.music.service.MusicService;
import com.ubuntuone.android.music.util.FileUtilities;
import com.ubuntuone.api.music.U1MusicAPI;
import com.ubuntuone.api.music.client.Failure;
import com.ubuntuone.api.music.request.U1DownloadListener;
import com.ubuntuone.api.music.util.CancelTrigger;

public class Song
{
	private static final String TAG = makeLogTag(Song.class);
	
	private static final String PARTIAL_SUFFIX = ".part";
	
	private Uri mSongUri;
	private String mSongId;
	private String mArtist;
	private String mAlbum;
	private String mTitle;
	private String mSuffix;
	private int mBitRate = 0;
	private int mDuration = 0;
	private long mSize = 0;
	
	private File mPartialFile;
	private File mCompleteFile;
	
	private CancelTrigger mCancelTrigger;
	private DownloadSongTask mWorker;
	
	public Song(Context context, Uri uri) {
		ContentResolver resolver = context.getContentResolver();
		String[] projection = Songs.getDefaultProjection();
		Cursor cursor = null;
		try {
			cursor = resolver.query(uri, projection, null, null, null);
			if (cursor != null && cursor.moveToFirst()) {
				mSongUri = uri;
				
				mSongId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
				mArtist = cursor.getString(cursor.getColumnIndex(Songs.SONG_ARTIST));
				mAlbum = cursor.getString(cursor.getColumnIndex(Songs.SONG_ALBUM));
				mTitle = cursor.getString(cursor.getColumnIndex(Songs.SONG_TITLE));
				mSuffix = cursor.getString(cursor.getColumnIndex(Songs.SONG_SUFFIX));
				mBitRate = cursor.getInt(cursor.getColumnIndex(Songs.SONG_BIT_RATE));
				mDuration = cursor.getInt(cursor.getColumnIndex(Songs.SONG_DURATION));
				mSize = cursor.getLong(cursor.getColumnIndex(Songs.SONG_SIZE));
				
				String path = FileUtilities.getSongPath(mArtist, mAlbum, mTitle, mSuffix);
				String partialPath = path.concat(PARTIAL_SUFFIX);
				mPartialFile = new File(Environment.getExternalStorageDirectory(), partialPath);
				mCompleteFile = new File(Environment.getExternalStorageDirectory(), path);
			}
		} finally {
			if (cursor != null) cursor.close();
		}
	}
	
	public Uri getSongUri() {
		return mSongUri;
	}
	
	public String getSongId() {
		return mSongId;
	}
	
	public String getSuffix() {
		return mSuffix;
	}
	
	public int getBitRate() {
		return mBitRate;
	}
	
	public int getDuration() {
		return mDuration;
	}
	
	public long getSize() {
		return mSize;
	}
	
	public File getPartialFile() {
		return mPartialFile;
	}
	
	public boolean isCompleteFileAvailable() {
		return mCompleteFile.exists();
	}
	
	public File getCompleteFile() {
		return mCompleteFile;
	}
	
	public void saveAsComplete(Context context) {
		ContentResolver resolver = context.getContentResolver();
		ContentValues cv = new ContentValues();
		cv.put(Songs.SONG_PATH, mCompleteFile.getAbsolutePath());
		resolver.update(mSongUri, cv, null, null);
		// Show check mark.
		Uri queueSongUri = Playlists.buildPlaylistSongUri(
				MusicProviderUtils.QUEUE_ID, mSongId);
		resolver.notifyChange(queueSongUri, null);
	}
	
	public void download(final MusicService service, final Handler handler, 
			final int position, final U1MusicAPI api, final boolean autoPlay) {
		final U1DownloadListener listener = new U1DownloadListener() {
			private long lastUpdatedProgress = System.currentTimeMillis();
			private boolean isAlreadyPlaying = false;

			@Override
			public void onStart() {
				mCompleteFile.getParentFile().mkdirs();
				Log.i(TAG, "Started downloading: " + mCompleteFile.getAbsolutePath());
			}
			
			@Override
			public void onProgress(final long bytes, final long total) {
				if (total == 0) return;
				
				long now = System.currentTimeMillis();
				if (now - lastUpdatedProgress > 500) {
					lastUpdatedProgress = System.currentTimeMillis();
					handler.post(new Runnable() {
						@Override
						public void run() {
							int percent = (int) (100 * bytes / (double) total);
							service.callbackOnSongDownload(mSongUri, percent);
							
							// TODO Convert 15% to contentType dependent byte size.
							if (autoPlay && percent > 15 && !isAlreadyPlaying) {
								isAlreadyPlaying = true;
								service.doPlay(Song.this, position);
							}
						}
					});
				}
			}

			@Override
			public void onSuccess() {
				service.callbackOnSongDownload(mSongUri, 100);
				mPartialFile.renameTo(mCompleteFile);
				
				ContentValues cv = new ContentValues();
				cv.put(Songs.SONG_PATH, mCompleteFile.getAbsolutePath());
				service.getContentResolver().update(mSongUri, cv, null, null);
				// Show check mark.
				Uri queueSongUri = Playlists.buildPlaylistSongUri(
						MusicProviderUtils.QUEUE_ID, mSongId);
				service.getContentResolver().notifyChange(queueSongUri, null);
				
				
				// TODO Save song metadata to MediaStore.Audio.
				
				handler.post(new Runnable() {
					@Override
					public void run() {
						if (autoPlay && !isAlreadyPlaying) {
							Song currentPlaying = service.getCurrentPlaying();
							if (currentPlaying == null ||
								!currentPlaying.getSongUri().equals(mSongUri)) {
								service.doPlay(Song.this, position);
							}
						}
					}
				});
			}
			
			@Override
			public void onFailure(Failure failure) {
				if (mCancelTrigger != null && !mCancelTrigger.isCancelled()) {
					Log.e(TAG, String.format("Failed to download %s, reason: ",
							mSongId, failure.getMessage()));
				}
			}

			@Override
			public void onFinish() {
				Log.i(TAG, "Finished downloading: " + mCompleteFile.getAbsolutePath());
				mCancelTrigger = null;
			}
		};
		
		mWorker = new DownloadSongTask() {
			@Override
			protected Void doInBackground(Void... params) {
				api.downloadSong(mSongId, mPartialFile.getAbsolutePath(),
						listener, mCancelTrigger);
				return null;
			}
		};
		mWorker.execute();
	}
	
	public void cancelDownload() {
		if (mCancelTrigger != null) {
			mCancelTrigger.onCancel();
		}
	}
	
	private abstract class DownloadSongTask extends AsyncTask<Void, Void, Void> {
	}
}
