/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.provider;

import static com.ubuntuone.android.music.util.LogUtils.makeLogTag;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map.Entry;
import java.util.Random;
import java.util.Set;

import android.content.ContentValues;
import android.text.TextUtils;
import android.util.Log;

import com.ubuntuone.android.music.provider.MusicContract.Albums;
import com.ubuntuone.android.music.provider.MusicContract.ArtistAlbums;
import com.ubuntuone.android.music.provider.MusicContract.Artists;
import com.ubuntuone.android.music.provider.MusicContract.Genres;
import com.ubuntuone.android.music.provider.MusicContract.PlaylistSongs;
import com.ubuntuone.android.music.provider.MusicContract.Playlists;
import com.ubuntuone.android.music.provider.MusicContract.Songs;
import com.ubuntuone.android.music.util.HashUtils;
import com.ubuntuone.android.music.util.StringUtils;
import com.ubuntuone.api.music.model.U1Album;
import com.ubuntuone.api.music.model.U1Artist;
import com.ubuntuone.api.music.model.U1Playlist;
import com.ubuntuone.api.music.model.U1Song;

public final class MusicContentValues
{
	private static final String TAG = makeLogTag(MusicContentValues.class);
	
	private static final boolean PRINT_CONTENT_VALUES = false;
	
	public static String getAlbumList(U1Artist artist) {
		return StringUtils.join(artist.getAlbumIds(), ",");
	}
	
	public static ContentValues forArtist(U1Artist artist) {
		final ContentValues cv = new ContentValues();
		cv.put(Artists.UPDATED, System.currentTimeMillis());
		cv.put(Artists.ARTIST_ID, artist.getId());
		cv.put(Artists.ARTIST_NAME, artist.getArtist());
		cv.put(Artists.ALBUM_LIST, getAlbumList(artist));
		cv.put(Artists.ALBUM_COUNT, artist.getAlbumIds().size());
		cv.put(Artists.SONG_COUNT, artist.getSongCount());
		cv.put(Artists.ARTIST_URL, artist.getArtistUrl());
		cv.put(Artists.ARTIST_ART_URL, artist.getArtistArtUrl());
		printContentValues("artist", cv);
		return cv;
	}
	
	public static ArrayList<ContentValues> forArtistAlbums(U1Artist artist) {
		ArrayList<ContentValues> values = new ArrayList<ContentValues>();
		ArrayList<String> albumIds = artist.getAlbumIds();
		for (String albumId : albumIds) {
			ContentValues cv = new ContentValues();
			cv.put(ArtistAlbums.UPDATED, System.currentTimeMillis());
			cv.put(ArtistAlbums.ARTIST_ID, artist.getId());
			cv.put(ArtistAlbums.ALBUM_ID, albumId);
			values.add(cv);
			printContentValues("artist album", cv);
		}
		return values;
	}

	public static ContentValues forAlbum(U1Album album) {
		final ContentValues cv = new ContentValues();
		cv.put(Albums.UPDATED, System.currentTimeMillis());
		cv.put(Albums.ALBUM_ID, album.getId());
		cv.put(Albums.ALBUM_TITLE, album.getTitle());
		cv.put(Albums.ALBUM_ARTIST, album.getArtist());
		cv.put(Albums.ALBUM_ARTIST_ID, album.getArtistId());
		cv.put(Albums.ALBUM_YEAR, album.getYear());
		cv.put(Albums.ALBUM_DATE_ADDED, album.getParsedDate());
		cv.put(Albums.ALBUM_URL, album.getAlbumUrl());
		cv.put(Albums.ALBUM_ART_URL, album.getAlbumArtUrl());
		printContentValues("album", cv);
		return cv;
	}
	
	public static ContentValues forSong(U1Song song) {
		final ContentValues cv = new ContentValues();
		cv.put(Songs.UPDATED, System.currentTimeMillis());
		cv.put(Songs.SONG_ID, song.getId());
		cv.put(Songs.SONG_TITLE, song.getTitle());
		cv.put(Songs.SONG_ALBUM, song.getAlbum());
		cv.put(Songs.SONG_ALBUM_ID, song.getAlbumId());
		cv.put(Songs.SONG_ALBUM_ARTIST, song.getAlbumArtist());
		cv.put(Songs.SONG_ARTIST, song.getArtist());
		cv.put(Songs.SONG_ARTIST_ID, song.getArtistId());
		cv.put(Songs.SONG_GENRE, song.getGenre());
		// TODO Calculate /0 separated genre md5 sums maybe?
		cv.put(Songs.SONG_GENRE_ID, getGenreId(song.getGenre()));
		cv.put(Songs.SONG_YEAR, song.getYear());
		cv.put(Songs.SONG_DISC_NUMBER, song.getDiscNumber());
		cv.put(Songs.SONG_TRACK, song.getTrack());
		cv.put(Songs.SONG_DURATION, song.getDuration());
		cv.put(Songs.SONG_SIZE, song.getSize());
		cv.put(Songs.SONG_BIT_RATE, song.getBitRate());
		cv.put(Songs.SONG_SUFFIX, song.getSuffix());
		cv.put(Songs.SONG_CONTENT_TYPE, song.getContentType());
		// Insert path only when song has been cached.
		// cv.put(Songs.SONG_PATH, FileUtilities.getSongPath(song));
		cv.put(Songs.SONG_URL, song.getSongUrl());
		cv.put(Songs.SONG_ART_URL, song.getSongArtUrl());
		cv.put(Songs.SONG_STREAM_URL, song.getSongStreamUrl());
		printContentValues("song", cv);
		return cv;
	}
	
	public static ContentValues forPlaylist(U1Playlist playlist) {
		final ContentValues cv = new ContentValues();
		cv.put(Playlists.UPDATED, System.currentTimeMillis());
		cv.put(Playlists.PLAYLIST_ID, playlist.getId());
		cv.put(Playlists.PLAYLIST_NAME, playlist.getName());
		cv.put(Playlists.PLAYLIST_SONG_COUNT, playlist.getSongCount());
		cv.put(Playlists.PLAYLIST_URL, playlist.getPlaylistUrl());
		printContentValues("playlist", cv);
		return cv;
	}
	
	public static ContentValues forPlaylistSong(String playlistId, String songId) {
		final ContentValues cv = new ContentValues();
		cv.put(PlaylistSongs.UPDATED, System.currentTimeMillis());
		cv.put(PlaylistSongs.PLAYLIST_ID, playlistId);
		cv.put(PlaylistSongs.SONG_ID, songId);
		printContentValues("playlist song", cv);
		return cv;
	}
	
	public static ContentValues forShuffleOrder(Random random) {
		final ContentValues cv = new ContentValues();
		cv.put(PlaylistSongs.UPDATED, System.currentTimeMillis());
		cv.put(PlaylistSongs.SHUFFLE_ORDER, random.nextLong());
		printContentValues("playlist song update", cv);
		return cv;
	}
	
	public static String getGenreId(String genre) {
		String genreId = null;
		String lcGenre = genre != null ? genre.trim().toLowerCase() : null;
		if (TextUtils.isEmpty(lcGenre) ||
				lcGenre.equals("null") || lcGenre.equals("genre")) {
			// Ignore generic crap. Assume genre was inserted, we don't care.
			genreId = null;
		} else {
			genreId = HashUtils.md5(genre.toLowerCase(Locale.US));
		}
		return genreId;
	}
	
	public static ContentValues forGenre(String genre) {
		final ContentValues cv = new ContentValues();
		cv.put(Genres.UPDATED, System.currentTimeMillis());
		cv.put(Genres.GENRE_ID, getGenreId(genre));
		cv.put(Genres.GENRE_TITLE, genre);
		return cv;
	}
	
	public static void printContentValues(String what, ContentValues cv) {
		if (PRINT_CONTENT_VALUES) {
			Set<Entry<String, Object>> keySet = cv.valueSet();
			Iterator<Entry<String, Object>> iterator = keySet.iterator();
			Log.w(TAG, String.format("--- %s values ---", what));
			while (iterator.hasNext()) {
				Entry<String, Object> v = iterator.next();
				Log.w(TAG, String.format("%s: %s", v.getKey(), v.getValue().toString()));
			}
		}
	}
}
