/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.provider;

import java.util.List;

import android.app.SearchManager;
import android.net.Uri;
import android.provider.BaseColumns;
import android.provider.ContactsContract;
import android.text.TextUtils;

import com.ubuntuone.android.music.provider.MusicDatabase.Tables;

public class MusicContract
{
	/**
	 * Special value for {@link SyncColumns#UPDATED} indicating that an entry
	 * has never been updated, or doesn't exist yet.
	 */
	public static final long UPDATED_NEVER = -1;
	
	public interface StarredColumn {
		String STARRED = "starred";
	}

	public interface SyncColumns
	{
		/** Last time this entry was updated or synchronized. */
		String UPDATED = "updated";
	}

	interface ArtistsColumns
	{
		/** Unique string identifying this artist. */
		String ARTIST_ID = "artist_id";
		/** Name of this artist. */
		String ARTIST_NAME = "artist_name";
		/** List of album ids of this artist. */
		String ALBUM_LIST = "album_list";
		/** Total album count for this artist. */
		String ALBUM_COUNT = "album_count";
		/** Total song count for this artist. */
		String SONG_COUNT = "song_count";
		/** Full URL to this artist online. */
		String ARTIST_URL = "artist_url";
		/** Full URL of the art of this artist. */
		String ARTIST_ART_URL = "artist_art_url";
	}
	
	interface ArtistAlbumsColumns extends AlbumsColumns
	{
		/** String identifying referenced artist. */
		String ARTIST_ID = "artist_id";
	}

	interface AlbumsColumns
	{
		/** Unique string identifying this album. */
		String ALBUM_ID = "album_id";
		/** Artist name of this album. */
		String ALBUM_TITLE = "album_title";
		/** Year this album was released in. */
		String ALBUM_ARTIST = "album_artist";
		/** md5sum of ALBUM_ARTIST. */
		String ALBUM_ARTIST_ID = "album_artist_id";
		/** Title of this album. */
		String ALBUM_YEAR = "album_year";
		/** Date this album has been added to U1 in milliseconds. */
		String ALBUM_DATE_ADDED = "album_date_added";
		/** FUll URL to this album online. */
		String ALBUM_URL = "album_url";
		/** Full URL to this album's art online. */
		String ALBUM_ART_URL = "album_art_url";
	}

	interface SongsColumns
	{
		/** Unique string identifying this song, same as file storage node key. */
		String SONG_ID = "song_id";
		/** Title of this song. */
		String SONG_TITLE = "song_title";
		/** Title of the album containing this song. */
		String SONG_ALBUM = "song_album";
		/** Album id of this song. */
		String SONG_ALBUM_ID = "song_album_id";
		/** Artist of the album containing this song. */
		String SONG_ALBUM_ARTIST = "song_album_artist";
		/** Artist of this song. */
		String SONG_ARTIST = "song_artist";
		/** md5sum of SONG_ARTIST. */
		String SONG_ARTIST_ID = "song_artist_id";
		/** Genre of this song. */
		String SONG_GENRE = "song_genre";
		/** md5 sum's of genres contained in SONG_GENRE. */
		String SONG_GENRE_ID = "song_genre_ids";
		/** Year this song was released. */
		String SONG_YEAR = "song_year";
		/** Disc number of this song. */
		String SONG_DISC_NUMBER = "song_disc_number";
		/** Track number on the album of this song. */
		String SONG_TRACK = "song_track";
		/** Duration of this song in seconds. */
		String SONG_DURATION = "song_duration";
		/** File size of this song in bytes. */
		String SONG_SIZE = "song_size";
		/** Bit rate of this song in bits per second. */
		String SONG_BIT_RATE = "song_bit_rate";
		/** File extension of this song. */
		String SONG_SUFFIX = "song_suffix";
		/** MIME content type of this song. */
		String SONG_CONTENT_TYPE = "song_content_type";
		/** Preferred path to store the song. */
		String SONG_PATH = "song_path";
		/** Time stamp of when the song was last played. */
		String SONG_LAST_PLAYED_AT = "last_played_at";
		/** Full URL to this song online. */
		String SONG_URL = "song_url";
		/** Full URL to this song's art online. */
		String SONG_ART_URL = "song_art_url";
		/** Full URL to this song content online. */
		String SONG_STREAM_URL = "song_stream_url";
		/** Flag indicating this song is currently playing. */
		String SONG_IS_PLAYING = "song_is_playing";
	}

	interface PlaylistsColumns
	{
		/** Unique string identifying this playlist. */
		String PLAYLIST_ID = "playlist_id";
		/** Name of this playlist. */
		String PLAYLIST_NAME = "playlist_name";
		/** Song count of this playlist. */
		String PLAYLIST_SONG_COUNT = "playlist_song_count";
		/** Full URL to this playlist online. */
		String PLAYLIST_URL = "playlist_url";
		/** Flag indicating playlist should be sync'ed up. */
		String DIRTY = "dirty";
	}
	
	interface PlaylistSongsColumns extends SongsColumns
	{
		/** String identifying referenced playlist. */
		String PLAYLIST_ID = "playlist_id";
		/** Integer value indicating shuffle play order. */
		String SHUFFLE_ORDER = "shuffle_order";
		/** Integer value indicating if song was already played. */
		String WAS_PLAYED = "was_played";
	}
	
	interface GenresColumns
	{
		/** Unique string identifying this genre. */
		String GENRE_ID = "genre_id";
		/** Genre title. */
		String GENRE_TITLE = "genre_title";
	}

	public static final String CONTENT_AUTHORITY = "com.ubuntuone.android.music";

	public static final Uri BASE_CONTENT_URI = Uri.parse("content://"
			+ CONTENT_AUTHORITY);

	private static final String PATH_ARTISTS = "artists";
	private static final String PATH_ARTISTS_ALBUMS = "artists_albums";
	private static final String PATH_ALBUMS = "albums";
	private static final String PATH_SONGS = "songs";
	private static final String PATH_STARRED = "starred";
	private static final String PATH_CACHED = "cached";
	private static final String PATH_PLAYLISTS = "playlists";
	private static final String PATH_PLAYLISTS_SONGS = "playlists_songs";
	private static final String PATH_GENRES = "genres";
	private static final String PATH_SEARCH = "search";
	private static final String PATH_SEARCH_SUGGEST = "search_suggest_query";

	public static class Artists implements ArtistsColumns, SyncColumns,
			BaseColumns, StarredColumn
	{
		public static final Uri CONTENT_URI = BASE_CONTENT_URI.buildUpon()
				.appendPath(PATH_ARTISTS).build();

		public static final String CONTENT_TYPE =
				"vnd.android.cursor.dir/vnd.ubuntuone.music.artist";
		public static final String CONTENT_ITEM_TYPE =
				"vnd.android.cursor.item/vnd.ubuntuone.music.artist";
		
		/** Default query projection. */
		public static final String[] DEFAULT_PROJECTION = new String[] {
				Artists._ID, Artists.ARTIST_ID, Artists.ARTIST_NAME, Artists.ALBUM_LIST,
				Artists.ALBUM_COUNT, Artists.SONG_COUNT, Artists.STARRED
		};

		/** Default "ORDER BY" clause. */
		public static final String DEFAULT_SORT = ArtistsColumns.ARTIST_NAME
				+ " COLLATE NOCASE ASC";

		/** Build {@link Uri} for requested {@link #ARTIST_ID}. */
		public static Uri buildArtistUri(String artistId) {
			return CONTENT_URI.buildUpon().appendPath(artistId).build();
		}

		/**
		 * Build {@link Uri} that references any {@link Albums} associated with
		 * the requested {@link #ARTIST_ID}.
		 */
		public static Uri buildAlbumsUri(String artistId) {
			return CONTENT_URI.buildUpon().appendPath(artistId)
					.appendPath(PATH_ALBUMS).build();
		}
		
		/**
		 * Build {@link Uri} that references an {@link Album} associated with
		 * the requested {@link #ARTIST_ID} and {@link #ALBUM_ID}.
		 */
		public static Uri buildAlbumUri(String artistId, String albumId) {
			return CONTENT_URI.buildUpon().appendPath(artistId)
					.appendPath(PATH_ALBUMS).appendPath(albumId).build();
		}
		
		/**
		 * Build {@link Uri} that references any {@link Songs} associated with
		 * the requested {@link #ALBUM_ID}.
		 */
		public static Uri buildSongsUri(String artistId) {
			return CONTENT_URI.buildUpon().appendPath(artistId)
					.appendPath(PATH_SONGS).build();
		}

		public static Uri buildSearchUri(String query) {
			return CONTENT_URI.buildUpon().appendPath(PATH_SEARCH)
					.appendPath(query).build();
		}

		public static boolean isSearchUri(Uri uri) {
			List<String> pathSegments = uri.getPathSegments();
			return pathSegments.size() >= 2
					&& PATH_SEARCH.equals(pathSegments.get(1));
		}

		public static String getSearchQuery(Uri uri) {
			return uri.getPathSegments().get(2);
		}

		/** Read {@link #ARTIST_ID} from {@link Artists} {@link Uri}. */
		public static String getArtistId(Uri uri) {
			return uri.getPathSegments().get(1);
		}
		
		/** Read {@link #ALBUM_ID} from {@link Artists} {@link Uri}. */
		public static String getAlbumId(Uri uri) {
			return uri.getPathSegments().get(3);
		}
		
		public static Uri getArtistsAlbumsUri() {
			return ArtistAlbums.CONTENT_URI;
		}
	}
	
	public static class ArtistAlbums implements ArtistAlbumsColumns, SyncColumns,
			SongsColumns, BaseColumns
	{
		public static final Uri CONTENT_URI = BASE_CONTENT_URI.buildUpon()
				.appendPath(PATH_ARTISTS_ALBUMS).build();
		
		public static final String CONTENT_TYPE =
				"vnd.android.cursor.dir/vnd.ubuntuone.music.artists_albums";
		public static final String CONTENT_ITEM_TYPE =
				"vnd.android.cursor.item/vnd.ubuntuone.music.artists_albums";

		/** Artist albums default "ORDER BY" clause. */
		public static final String DEFAULT_SORT = Albums.ALBUM_TITLE
				+ " COLLATE NOCASE ASC";

		public static final String[] DEFAULT_PROJECTION = new String[] {
				Albums._ID, Albums.ALBUM_ID, Albums.ALBUM_TITLE, Albums.ALBUM_ARTIST,
				Albums.ALBUM_ARTIST, Albums.ALBUM_ARTIST_ID, Albums.ALBUM_YEAR,
				Albums.ALBUM_DATE_ADDED, Albums.STARRED, ArtistAlbums.ALBUM_ID
		};
	}

	public static class Albums implements AlbumsColumns, SyncColumns,
			BaseColumns, StarredColumn
	{
		public static final Uri CONTENT_URI = BASE_CONTENT_URI.buildUpon()
				.appendPath(PATH_ALBUMS).build();

		public static final String CONTENT_TYPE =
				"vnd.android.cursor.dir/vnd.ubuntuone.music.album";
		public static final String CONTENT_ITEM_TYPE =
				"vnd.android.cursor.item/vnd.ubuntuone.music.album";

		/** Default query projection. */
		public static final String[] DEFAULT_PROJECTION = new String[] {
				Albums._ID, Albums.ALBUM_ID, Albums.ALBUM_TITLE, Albums.ALBUM_ARTIST,
				Albums.ALBUM_ARTIST_ID, Albums.ALBUM_YEAR, Albums.ALBUM_DATE_ADDED,
				Albums.STARRED
		};

		/** Default "ORDER BY" clause. */
		public static final String DEFAULT_SORT = AlbumsColumns.ALBUM_TITLE
				+ " COLLATE NOCASE ASC, " + AlbumsColumns.ALBUM_ARTIST
				+ " COLLATE NOCASE ASC";
		
		/** Default "ORDER BY" clause for album songs. */
		public static final String DEFAULT_SONG_SORT = Songs.SONG_ALBUM
				+ " COLLATE NOCASE ASC, " + Songs.SONG_TRACK
				+ " COLLATE NOCASE ASC";

		/** Build {@link Uri} for requested {@link #ALBUM_ID}. */
		public static Uri buildAlbumUri(String albumId) {
			return CONTENT_URI.buildUpon().appendPath(albumId).build();
		}

		/**
		 * Build {@link Uri} that references any {@link Songs} associated with
		 * the requested {@link #ALBUM_ID}.
		 */
		public static Uri buildSongsUri(String albumId) {
			return CONTENT_URI.buildUpon().appendPath(albumId)
					.appendPath(PATH_SONGS).build();
		}

		public static Uri buildSearchUri(String query) {
			return CONTENT_URI.buildUpon().appendPath(PATH_SEARCH)
					.appendPath(query).build();
		}

		public static boolean isSearchUri(Uri uri) {
			List<String> pathSegments = uri.getPathSegments();
			return pathSegments.size() >= 2
					&& PATH_SEARCH.equals(pathSegments.get(1));
		}

		public static String getSearchQuery(Uri uri) {
			return uri.getPathSegments().get(2);
		}

		/** Read {@link #ALBUM_ID} from {@link Albums} {@link Uri}. */
		public static String getAlbumId(Uri uri) {
			return uri.getPathSegments().get(1);
		}
	}

	public static class Songs implements SongsColumns, SyncColumns,
			BaseColumns, StarredColumn
	{
		public static final Uri CONTENT_URI = BASE_CONTENT_URI.buildUpon()
				.appendPath(PATH_SONGS).build();

		public static final String CONTENT_TYPE =
				"vnd.android.cursor.dir/vnd.ubuntuone.music.song";
		public static final String CONTENT_ITEM_TYPE =
				"vnd.android.cursor.item/vnd.ubuntuone.music.song";
		
		public static final Uri STARRED_SONGS_URI = CONTENT_URI
				.buildUpon().appendPath(PATH_STARRED).build();

		public static final Uri CACHED_SONGS_URI = CONTENT_URI
				.buildUpon().appendPath(PATH_CACHED).build();
		
		/** Default query projection. */
		private static final String[] DEFAULT_PROJECTION = new String[] {
			Songs._ID, Songs.SONG_ID, Songs.SONG_TITLE, Songs.SONG_ALBUM,
			Songs.SONG_ALBUM_ID, Songs.SONG_ALBUM_ARTIST,
			Songs.SONG_ARTIST, Songs.SONG_ARTIST_ID, Songs.SONG_GENRE,
			Songs.SONG_GENRE_ID, Songs.SONG_YEAR, Songs.SONG_DISC_NUMBER,
			Songs.SONG_TRACK, Songs.SONG_DURATION, Songs.SONG_SIZE,
			Songs.SONG_BIT_RATE, Songs.SONG_SUFFIX, Songs.SONG_CONTENT_TYPE,
			Songs.SONG_PATH, Songs.SONG_LAST_PLAYED_AT, Songs.STARRED, Songs.SONG_IS_PLAYING,
		};
		
		public static String[] getDefaultProjection() {
			// karni: Earn $10 and tell me why you have to return a COPY of a *static final* for it
			// not to change here, in MusicContract#DEFAULT_PROJECTION. Platform bug?
			return DEFAULT_PROJECTION.clone();
		}

		/** Default "ORDER BY" clause. */
		public static final String DEFAULT_SORT = SongsColumns.SONG_TITLE
				+ " COLLATE NOCASE ASC, " + SongsColumns.SONG_ALBUM_ARTIST
				+ " COLLATE NOCASE ASC";

		/** Build {@link Uri} for requested {@link #SONG_ID}. */
		public static Uri buildSongUri(String songId) {
			return CONTENT_URI.buildUpon().appendPath(songId).build();
		}

		public static Uri buildSearchUri(String query) {
			return CONTENT_URI.buildUpon().appendPath(PATH_SEARCH)
					.appendPath(query).build();
		}

		public static boolean isSearchUri(Uri uri) {
			List<String> pathSegments = uri.getPathSegments();
			return pathSegments.size() >= 2
					&& PATH_SEARCH.equals(pathSegments.get(1));
		}

		public static String getSearchQuery(Uri uri) {
			return uri.getPathSegments().get(2);
		}

		/** Read {@link #SONG_ID} from {@link Songs} {@link Uri}. */
		public static String getSongId(Uri uri) {
			return uri.getPathSegments().get(1);
		}
	}

	public static class Playlists implements PlaylistsColumns, SyncColumns,
			BaseColumns, StarredColumn
	{
		public static final Uri CONTENT_URI = BASE_CONTENT_URI.buildUpon()
				.appendPath(PATH_PLAYLISTS).build();

		public static final String CONTENT_TYPE =
				"vnd.android.cursor.dir/vnd.ubuntuone.music.playlist";
		public static final String CONTENT_ITEM_TYPE =
				"vnd.android.cursor.item/vnd.ubuntuone.music.playlist";

		/** Playlists default "ORDER BY" clause. */
		public static final String DEFAULT_SORT =
				Playlists.PLAYLIST_NAME + " COLLATE NOCASE ASC";
		
		/** Default query projection. */
		public static final String[] DEFAULT_PROJECTION = new String[] {
			Playlists._ID, Playlists.PLAYLIST_ID, Playlists.PLAYLIST_NAME,
			Playlists.PLAYLIST_SONG_COUNT, Playlists.STARRED, Playlists.DIRTY
		};
		
		public static final String[] SONGS_DEFAULT_PROJECTION = new String[] {
			Songs._ID, Songs.SONG_ID, Songs.SONG_TITLE, Songs.SONG_ALBUM,
			Songs.SONG_ALBUM_ID, Songs.SONG_ALBUM_ARTIST,
			Songs.SONG_ARTIST, Songs.SONG_ARTIST_ID, Songs.SONG_GENRE,
			Songs.SONG_GENRE_ID, Songs.SONG_YEAR, Songs.SONG_DISC_NUMBER,
			Songs.SONG_TRACK, Songs.SONG_DURATION, Songs.SONG_SIZE,
			Songs.SONG_BIT_RATE, Songs.SONG_SUFFIX, Songs.SONG_CONTENT_TYPE,
			Songs.SONG_PATH, Songs.SONG_LAST_PLAYED_AT, Songs.STARRED, Songs.SONG_IS_PLAYING,
			PlaylistSongsColumns.SHUFFLE_ORDER
		};

		/** Build {@link Uri} for requested {@link #PLAYLIST_ID}. */
		public static Uri buildPlaylistUri(String playlistId) {
			return CONTENT_URI.buildUpon().appendPath(playlistId).build();
		}

		/**
		 * Build {@link Uri} that references any {@link Songs} associated with
		 * the requested {@link #PLAYLIST_ID}.
		 */
		public static Uri buildPlaylistSongsUri(String playlistId) {
			return CONTENT_URI.buildUpon().appendPath(playlistId)
					.appendPath(PATH_SONGS).build();
		}
		
		/**
		 * Build {@link Uri} that references a {@link Songs} associated with
		 * the requested {@link #SONG_ID}.
		 */
		public static Uri buildPlaylistSongUri(String playlistId, String songId) {
			return CONTENT_URI.buildUpon().appendPath(playlistId)
					.appendPath(PATH_SONGS).appendPath(songId).build();
		}

		public static Uri buildSearchUri(String query) {
			return CONTENT_URI.buildUpon().appendPath(PATH_SEARCH)
					.appendPath(query).build();
		}

		public static boolean isSearchUri(Uri uri) {
			List<String> pathSegments = uri.getPathSegments();
			return pathSegments.size() >= 2
					&& PATH_SEARCH.equals(pathSegments.get(1));
		}

		public static String getSearchQuery(Uri uri) {
			return uri.getPathSegments().get(2);
		}

		/** Read {@link #PLAYLIST_ID} from {@link Playlists} {@link Uri}. */
		public static String getPlaylistId(Uri uri) {
			return uri.getPathSegments().get(1);
		}
		
		/** Read {@link #SONG_ID} from {@link Playlists} {@link Uri}. */
		public static String getSongId(Uri uri) {
			return uri.getPathSegments().get(3);
		}
		
		public static Uri getPlaylistsSongsUri() {
			return PlaylistSongs.CONTENT_URI;
		}
	}
	
	public static class PlaylistSongs implements PlaylistSongsColumns, SyncColumns,
			SongsColumns, StarredColumn, BaseColumns
	{
		public static final Uri CONTENT_URI = BASE_CONTENT_URI.buildUpon()
				.appendPath(PATH_PLAYLISTS_SONGS).build();
		
		public static final String CONTENT_TYPE =
				"vnd.android.cursor.dir/vnd.ubuntuone.music.playlists_songs";
		public static final String CONTENT_ITEM_TYPE =
				"vnd.android.cursor.item/vnd.ubuntuone.music.playlists_songs";
		
		/** Playlist songs default "ORDER BY" clause. */
		public static final String DEFAULT_SORT = Qualified.PLAYLIST_SONGS_ID + " ASC";

		/** Playlist songs shuffle "ORDER BY" clause. */
		public static final String SHUFFLE_SORT = PlaylistSongsColumns.SHUFFLE_ORDER
				+ " ASC";

		public static final String[] DEFAULT_PROJECTION = new String[] {
				Songs._ID, Songs.SONG_ID, Songs.SONG_TITLE, Songs.SONG_ALBUM,
				Songs.SONG_ALBUM_ID, Songs.SONG_ALBUM_ARTIST,
				Songs.SONG_ARTIST, Songs.SONG_ARTIST_ID, Songs.SONG_GENRE,
				Songs.SONG_GENRE_ID, Songs.SONG_YEAR, Songs.SONG_DISC_NUMBER,
				Songs.SONG_TRACK, Songs.SONG_DURATION, Songs.SONG_SIZE,
				Songs.SONG_BIT_RATE, Songs.SONG_SUFFIX,
				Songs.SONG_CONTENT_TYPE, Songs.SONG_PATH,
				Songs.SONG_LAST_PLAYED_AT, Songs.STARRED, Songs.SONG_IS_PLAYING,
				PlaylistSongs.PLAYLIST_ID,
				PlaylistSongs.SHUFFLE_ORDER
		};
	}
	
	public static class Genres implements GenresColumns, SyncColumns,
			BaseColumns
	{
		public static final Uri CONTENT_URI = BASE_CONTENT_URI.buildUpon()
				.appendPath(PATH_GENRES).build();

		public static final String CONTENT_TYPE =
				"vnd.android.cursor.dir/vnd.ubuntuone.music.genre";
		public static final String CONTENT_ITEM_TYPE =
				"vnd.android.cursor.item/vnd.ubuntuone.music.genre";

		/** Default "ORDER BY" clause. */
		public static final String DEFAULT_SORT = GenresColumns.GENRE_TITLE
				+ " COLLATE NOCASE ASC";
		
		/** Default query projection. */
		public static final String[] DEFAULT_PROJECTION =
				new String[] { Genres._ID, Genres.GENRE_ID, Genres.GENRE_TITLE };

		/** Build {@link Uri} for requested {@link #GENRE_ID}. */
		public static Uri buildGenreUri(String genreId) {
			return CONTENT_URI.buildUpon().appendPath(genreId).build();
		}

		/**
		 * Build {@link Uri} that references any {@link Songs} associated with
		 * the requested {@link #GENRE_ID}.
		 */
		public static Uri buildGenreSongsUri(String genreId) {
			return CONTENT_URI.buildUpon().appendPath(genreId)
					.appendPath(PATH_SONGS).build();
		}

		public static Uri buildSearchUri(String query) {
			return CONTENT_URI.buildUpon().appendPath(PATH_SEARCH)
					.appendPath(query).build();
		}

		public static boolean isSearchUri(Uri uri) {
			List<String> pathSegments = uri.getPathSegments();
			return pathSegments.size() >= 2
					&& PATH_SEARCH.equals(pathSegments.get(1));
		}

		public static String getSearchQuery(Uri uri) {
			return uri.getPathSegments().get(2);
		}

		/** Read {@link #GENRE_ID} from {@link Genres} {@link Uri}. */
		public static String getGenreId(Uri uri) {
			return uri.getPathSegments().get(1);
		}
	}

	public static class SearchSuggest
	{
		public static final Uri CONTENT_URI = BASE_CONTENT_URI.buildUpon()
				.appendPath(PATH_SEARCH_SUGGEST).build();

		public static final String DEFAULT_SORT = SearchManager.SUGGEST_COLUMN_TEXT_1
				+ " COLLATE NOCASE ASC";
	}

	public static Uri addCallerIsSyncAdapterParameter(Uri uri) {
		return uri
				.buildUpon()
				.appendQueryParameter(ContactsContract.CALLER_IS_SYNCADAPTER,
						"true").build();
	}

	public static boolean hasCallerIsSyncAdapterParameter(Uri uri) {
		return TextUtils.equals("true",
				uri.getQueryParameter(ContactsContract.CALLER_IS_SYNCADAPTER));
	}
	
	/**
	 * {@link MusicContract} fields that are fully qualified with a specific
	 * parent {@link Tables}. Used when needed to work around SQL ambiguity.
	 */
	private interface Qualified
	{
		String PLAYLIST_SONGS_ID = Tables.PLAYLISTS_SONGS + "." + BaseColumns._ID;
	}

	private MusicContract() {
	}
}
