/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.provider;

import static com.ubuntuone.android.music.util.LogUtils.LOGD;
import static com.ubuntuone.android.music.util.LogUtils.LOGW;
import static com.ubuntuone.android.music.util.LogUtils.makeLogTag;
import android.content.ContentValues;
import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.provider.BaseColumns;

import com.ubuntuone.android.music.UbuntuOneMusic;
import com.ubuntuone.android.music.provider.MusicContract.Albums;
import com.ubuntuone.android.music.provider.MusicContract.AlbumsColumns;
import com.ubuntuone.android.music.provider.MusicContract.ArtistAlbumsColumns;
import com.ubuntuone.android.music.provider.MusicContract.Artists;
import com.ubuntuone.android.music.provider.MusicContract.ArtistsColumns;
import com.ubuntuone.android.music.provider.MusicContract.Genres;
import com.ubuntuone.android.music.provider.MusicContract.PlaylistSongsColumns;
import com.ubuntuone.android.music.provider.MusicContract.Playlists;
import com.ubuntuone.android.music.provider.MusicContract.PlaylistsColumns;
import com.ubuntuone.android.music.provider.MusicContract.Songs;
import com.ubuntuone.android.music.provider.MusicContract.SongsColumns;
import com.ubuntuone.android.music.provider.MusicContract.StarredColumn;
import com.ubuntuone.android.music.provider.MusicContract.SyncColumns;
import com.ubuntuone.api.music.model.U1Playlist;

public class MusicDatabase extends SQLiteOpenHelper
{
	private static final String TAG = makeLogTag(MusicDatabase.class);
	
	private static final String DATABASE_NAME = "music.db";

	// NOTE: carefully update onUpgrade() when bumping database versions to make
	// sure user data is saved.

	private static final int VER_INITIAL = 1;

	private static final int DATABASE_VERSION = VER_INITIAL;
	
	//private static final String NOW = "strftime('%s', 'now')";

	public interface Tables
	{
		String ARTISTS = "artists";
		String ARTISTS_ALBUMS = "artists_albums";
		String ALBUMS = "albums";
		String SONGS = "songs";
		String PLAYLISTS = "playlists";
		String PLAYLISTS_SONGS = "playlists_songs";
		String GENRES = "genres";
		
		String ARTISTS_JOIN_ALBUMS = "artists_albums LEFT JOIN albums " +
				"ON artists_albums.album_id=albums.album_id";
		String PLAYLIST_SONGS_JOIN_SONGS = "playlists_songs LEFT JOIN songs " +
				"ON playlists_songs.song_id=songs.song_id";
	}

	private interface Triggers
	{
		String ARTIST_ALBUMS_DELETE = "artist_albums_delete";
		String ALBUM_SONGS_DELETE = "album_songs_delete";
		String PLAYLIST_PLAYLIST_SONGS_DELETE = "playlist_playlists_songs_delete";
	}
	
	/** Fully-qualified field names. */
	private interface Qualified
	{
		String ALBUMS_ARTIST_ID = Tables.ALBUMS
				+ "." + AlbumsColumns.ALBUM_ARTIST_ID;
		String SONGS_ALBUM_ID = Tables.SONGS
				+ "." + SongsColumns.SONG_ALBUM_ID;
		String PLAYLIST_SONGS_PLAYLIST_ID = Tables.PLAYLISTS_SONGS
				+ "." + PlaylistSongsColumns.PLAYLIST_ID;
	}

	/** {@code REFERENCES} clauses. */
	private interface References
	{
		/** Referencing Artists.ARTIST_ID from ArtistAlbums. */
		String ARTIST_ID = "REFERENCES " + Tables.ARTISTS + "(" + Artists.ARTIST_ID + ")";
		/** Referencing ArtistsAlbums.ALBUM_ID from Songs. */
		String ALBUM_ID = "REFERENCES " + Tables.ALBUMS + "(" + Albums.ALBUM_ID + ")";
		/** Referencing Songs.SONG_ID from PlaylistSongs. */
		String SONG_ID = "REFERENCES " + Tables.SONGS + "(" + Songs.SONG_ID + ")";
		/** Referencing Playlists.PLAYLIST_ID from PlaylistSongs. */
		String PLAYLIST_ID = "REFERENCES " + Tables.PLAYLISTS + "(" + Playlists.PLAYLIST_ID + ")";
	}

	public MusicDatabase(Context context) {
		super(context, DATABASE_NAME, null, DATABASE_VERSION);
	}

	@Override
	public SQLiteDatabase getWritableDatabase() {
		SQLiteDatabase db = super.getWritableDatabase();
		if (UbuntuOneMusic.SHOULD_USE_FK) {
			db.execSQL("PRAGMA foreign_keys = ON");
		}
		return db;
	}

	@Override
	public void onCreate(SQLiteDatabase db) {
		db.execSQL("CREATE TABLE " + Tables.ARTISTS + "("
				+ BaseColumns._ID + " INTEGER PRIMARY KEY AUTOINCREMENT, "
				+ SyncColumns.UPDATED + " INTEGER NOT NULL DEFAULT 0, "
				+ StarredColumn.STARRED + " INTEGER DEFAULT 0, "
				+ ArtistsColumns.ARTIST_ID + " TEXT NOT NULL, "
				+ ArtistsColumns.ARTIST_NAME + " TEXT NOT NULL, "
				+ ArtistsColumns.ALBUM_LIST + " TEXT, "
				+ ArtistsColumns.ALBUM_COUNT + " INTEGER NOT NULL DEFAULT 0, "
				+ ArtistsColumns.SONG_COUNT + " INTEGER NOT NULL DEFAULT 0, "
				+ ArtistsColumns.ARTIST_URL + " TEXT NOT NULL, "
				+ ArtistsColumns.ARTIST_ART_URL + " TEXT NOT NULL, "
				+ "UNIQUE (" + ArtistsColumns.ARTIST_ID + ") ON CONFLICT IGNORE)");
		
		db.execSQL("CREATE TABLE " + Tables.ARTISTS_ALBUMS + "("
				+ BaseColumns._ID + " INTEGER PRIMARY KEY AUTOINCREMENT, "
				+ SyncColumns.UPDATED + " INTEGER NOT NULL DEFAULT 0, "
				+ ArtistAlbumsColumns.ARTIST_ID + " TEXT NOT NULL "
						+ References.ARTIST_ID + ", "
				+ ArtistAlbumsColumns.ALBUM_ID + " TEXT NOT NULL, "
				+ "UNIQUE (" + ArtistAlbumsColumns.ARTIST_ID + ", "
						+ ArtistAlbumsColumns.ALBUM_ID + ") ON CONFLICT IGNORE)");
		
		db.execSQL("CREATE TABLE " + Tables.ALBUMS + "("
				+ BaseColumns._ID + " INTEGER PRIMARY KEY AUTOINCREMENT, "
				+ SyncColumns.UPDATED + " INTEGER NOT NULL DEFAULT 0, "
				+ StarredColumn.STARRED + " INTEGER DEFAULT 0, "
				+ AlbumsColumns.ALBUM_ID + " TEXT NOT NULL, "
				// TODO foreign key mismatch on artists_albums UPDATE
//						+ References.ARTIST_ALBUM_ID + ", "
				+ AlbumsColumns.ALBUM_TITLE + " TEXT NOT NULL, "
				+ AlbumsColumns.ALBUM_ARTIST + " TEXT NOT NULL, "
				+ AlbumsColumns.ALBUM_ARTIST_ID + " TEXT NOT NULL, "
				// This FK can't be used, bug/inconsistency in the API.
//						+ References.ARTIST_ID + ", "
				+ AlbumsColumns.ALBUM_YEAR + " INTEGER NOT NULL, "
				+ AlbumsColumns.ALBUM_DATE_ADDED + " INTEGER NOT NULL, "
				+ AlbumsColumns.ALBUM_URL + " TEXT NOT NULL, "
				+ AlbumsColumns.ALBUM_ART_URL + " TEXT NOT NULL, "
				+ "UNIQUE (" + AlbumsColumns.ALBUM_ID + ") ON CONFLICT IGNORE)");
		
		db.execSQL("CREATE TABLE " + Tables.SONGS + "("
				+ BaseColumns._ID + " INTEGER PRIMARY KEY AUTOINCREMENT, "
				+ SyncColumns.UPDATED + " INTEGER NOT NULL DEFAULT 0, "
				+ StarredColumn.STARRED + " INTEGER DEFAULT 0, "
				+ SongsColumns.SONG_ID + " TEXT NOT NULL, "
				+ SongsColumns.SONG_TITLE + " TEXT NOT NULL, "
				+ SongsColumns.SONG_ALBUM + " TEXT NOT NULL, "
				+ SongsColumns.SONG_ALBUM_ID + " TEXT NOT NULL "
						+ References.ALBUM_ID + ", "
				+ SongsColumns.SONG_ALBUM_ARTIST + " TEXT NOT NULL, "
				+ SongsColumns.SONG_ARTIST + " TEXT NOT NULL, "
				+ SongsColumns.SONG_ARTIST_ID + " TEXT NOT NULL "
						+ References.ARTIST_ID + ", "
				+ SongsColumns.SONG_GENRE + " TEXT NOT NULL, "
				+ SongsColumns.SONG_GENRE_ID + " TEXT NOT NULL, "
				+ SongsColumns.SONG_YEAR + " INTEGER NOT NULL, "
				+ SongsColumns.SONG_DISC_NUMBER + " INTEGER NOT NULL, "
				+ SongsColumns.SONG_TRACK + " INTEGER NOT NULL, "
				+ SongsColumns.SONG_DURATION + " INTEGER NOT NULL,"
				+ SongsColumns.SONG_SIZE + " INTEGER NOT NULL, "
				+ SongsColumns.SONG_BIT_RATE + " INTEGER NOT NULL, "
				+ SongsColumns.SONG_SUFFIX + " TEXT NOT NULL, "
				+ SongsColumns.SONG_CONTENT_TYPE + " TEXT NOT NULL, "
				+ SongsColumns.SONG_PATH + " TEXT, "
				+ SongsColumns.SONG_LAST_PLAYED_AT + " INTEGER NOT NULL DEFAULT 0, "
				+ SongsColumns.SONG_URL + " TEXT NOT NULL, "
				+ SongsColumns.SONG_ART_URL + " TEXT NOT NULL, "
				+ SongsColumns.SONG_STREAM_URL + " TEXT NOT NULL, "
				+ SongsColumns.SONG_IS_PLAYING + " INTEGER NOT NULL DEFAULT 0, "
				+ "UNIQUE (" + SongsColumns.SONG_ID + ") ON CONFLICT IGNORE)");
		
		db.execSQL("CREATE TABLE " + Tables.PLAYLISTS + "("
				+ BaseColumns._ID + " INTEGER PRIMARY KEY AUTOINCREMENT, "
				+ SyncColumns.UPDATED + " INTEGER NOT NULL DEFAULT 0, "
				+ StarredColumn.STARRED + " INTEGER DEFAULT 0, "
				+ PlaylistsColumns.PLAYLIST_ID + " TEXT NOT NULL, "
				+ PlaylistsColumns.PLAYLIST_NAME + " TEXT NOT NULL, "
				+ PlaylistsColumns.PLAYLIST_SONG_COUNT + " INTEGER NOT NULL DEFAULT 0, "
				+ PlaylistsColumns.PLAYLIST_URL + " TEXT NOT NULL, "
				+ PlaylistsColumns.DIRTY + " INTEGER NOT NULL DEFAULT 0, "
				+ "UNIQUE (" + PlaylistsColumns.PLAYLIST_ID + ") ON CONFLICT IGNORE)");
		
		// Create the play queue playlist.
		U1Playlist queuePlaylist = new U1Playlist(
				MusicProviderUtils.QUEUE_ID, "", MusicProviderUtils.QUEUE_NAME, 0);
		ContentValues values = MusicContentValues.forPlaylist(queuePlaylist);
		db.insert(Tables.PLAYLISTS, null, values);
		
		db.execSQL("CREATE TABLE " + Tables.PLAYLISTS_SONGS + "("
				+ BaseColumns._ID + " INTEGER PRIMARY KEY AUTOINCREMENT, "
				+ SyncColumns.UPDATED + " INTEGER NOT NULL DEFAULT 0, "
				+ PlaylistSongsColumns.PLAYLIST_ID + " TEXT NOT NULL "
						+ References.PLAYLIST_ID + ", "
				+ PlaylistSongsColumns.SONG_ID + " TEXT NOT NULL "
						+ References.SONG_ID + ", "
				+ PlaylistSongsColumns.SHUFFLE_ORDER + " INTEGER NOT NULL DEFAULT (RANDOM()), "
				+ PlaylistSongsColumns.WAS_PLAYED + " INTEGER NOT NULL DEFAULT 0)");
		
		db.execSQL("CREATE TABLE " + Tables.GENRES + "("
				+ BaseColumns._ID + " INTEGER PRIMARY KEY AUTOINCREMENT, "
				+ SyncColumns.UPDATED + " INTEGER NOT NULL DEFAULT 0, "
				+ Genres.GENRE_ID + " TEXT NOT NULL, "
				+ Genres.GENRE_TITLE + " TEXT NOT NULL, "
				+ "UNIQUE (" + Genres.GENRE_ID + ") ON CONFLICT IGNORE)");
		
		createArtistAlbumsDeleteTrigger(db);
		createAlbumSongsDeleteTrigger(db);
		createPlaylistPlaylistSongsDeleteTrigger(db);
	}
	
	// TODO Either add trigger, or wipe unrefreshed rows after sync completes. 
	private void createArtistAlbumsDeleteTrigger(SQLiteDatabase db) {
		db.execSQL("CREATE TRIGGER " + Triggers.ARTIST_ALBUMS_DELETE + " AFTER DELETE ON "
                + Tables.ARTISTS + " BEGIN DELETE FROM " + Tables.ALBUMS + " "
                + " WHERE " + Qualified.ALBUMS_ARTIST_ID + "=old." + Artists.ARTIST_ID
                + ";" + " END;");
	}
	
	private void createAlbumSongsDeleteTrigger(SQLiteDatabase db) {
		db.execSQL("CREATE TRIGGER " + Triggers.ALBUM_SONGS_DELETE + " AFTER DELETE ON "
                + Tables.ALBUMS + " BEGIN DELETE FROM " + Tables.SONGS + " "
                + " WHERE " + Qualified.SONGS_ALBUM_ID + "=old." + Albums.ALBUM_ID
                + ";" + " END;");
	}
	
	private void createPlaylistPlaylistSongsDeleteTrigger(SQLiteDatabase db) {
		db.execSQL("CREATE TRIGGER " + Triggers.PLAYLIST_PLAYLIST_SONGS_DELETE + " AFTER DELETE ON "
                + Tables.PLAYLISTS + " BEGIN DELETE FROM " + Tables.PLAYLISTS_SONGS + " "
                + " WHERE " + Qualified.PLAYLIST_SONGS_PLAYLIST_ID + "=old." + Playlists.PLAYLIST_ID
                + ";" + " END;");
	}

	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
	    LOGD(TAG, "onUpgrade() from " + oldVersion + " to " + newVersion);

        // NOTE: This switch statement is designed to handle cascading database
        // updates, starting at the current version and falling through to all
        // future upgrade cases. Only use "break;" when you want to drop and
        // recreate the entire database.

        int version = oldVersion;

        switch (version) {
            case VER_INITIAL:
                version = VER_INITIAL;
        }

        LOGD(TAG, "after upgrade logic, at version " + version);
        if (version != DATABASE_VERSION) {
            LOGW(TAG, "Destroying old data during upgrade");

            db.execSQL("DROP TABLE IF EXISTS " + Tables.GENRES);
            db.execSQL("DROP TABLE IF EXISTS " + Tables.PLAYLISTS_SONGS);
            db.execSQL("DROP TABLE IF EXISTS " + Tables.PLAYLISTS);
            db.execSQL("DROP TABLE IF EXISTS " + Tables.SONGS);
            db.execSQL("DROP TABLE IF EXISTS " + Tables.ALBUMS);
            db.execSQL("DROP TABLE IF EXISTS " + Tables.ARTISTS);

            db.execSQL("DROP TRIGGER IF EXISTS " + Triggers.ARTIST_ALBUMS_DELETE);
            db.execSQL("DROP TRIGGER IF EXISTS " + Triggers.ALBUM_SONGS_DELETE);

            onCreate(db);
        }
	}

	public static void deleteDatabase(Context context) {
		context.deleteDatabase(DATABASE_NAME);
	}
}
