/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.provider;

import static com.ubuntuone.android.music.util.LogUtils.LOGE;
import static com.ubuntuone.android.music.util.LogUtils.LOGV;
import static com.ubuntuone.android.music.util.LogUtils.makeLogTag;

import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Arrays;

import android.app.Activity;
import android.content.ContentProvider;
import android.content.ContentProviderOperation;
import android.content.ContentProviderResult;
import android.content.ContentResolver;
import android.content.ContentValues;
import android.content.Context;
import android.content.OperationApplicationException;
import android.content.UriMatcher;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.net.Uri;
import android.os.ParcelFileDescriptor;
import android.text.TextUtils;
import android.util.Log;

import com.ubuntuone.android.music.provider.MusicContract.Albums;
import com.ubuntuone.android.music.provider.MusicContract.ArtistAlbums;
import com.ubuntuone.android.music.provider.MusicContract.Artists;
import com.ubuntuone.android.music.provider.MusicContract.Genres;
import com.ubuntuone.android.music.provider.MusicContract.PlaylistSongs;
import com.ubuntuone.android.music.provider.MusicContract.Playlists;
import com.ubuntuone.android.music.provider.MusicContract.Songs;
import com.ubuntuone.android.music.provider.MusicDatabase.Tables;
import com.ubuntuone.android.music.util.SelectionBuilder;

/**
 * Provider that stores {@link MusicContract} data. Data is usually inserted
 * by {@link com.ubuntuone.android.music.sync.SyncHelper}, and queried by various
 * {@link Activity} instances.
 */
public class MusicProvider extends ContentProvider
{
	private static final String TAG = makeLogTag(MusicProvider.class);

	private MusicDatabase mOpenHelper;

	private static final UriMatcher sUriMatcher = buildUriMatcher();

	/** All artists. */
	private static final int ARTISTS = 100;
	/** Given artist. */
	private static final int ARTISTS_ID = 101;
	/** Starred artists. */
	private static final int ARTISTS_ID_STARRED = 102;
	/** Albums by given artist. */
	private static final int ARTISTS_ID_ALBUMS = 105;
	/** Album by given artist. */
	private static final int ARTISTS_ID_ALBUMS_ID = 106;
	/** Songs by given artist. */
	private static final int ARTISTS_ID_SONGS = 107;
	/** Mapping from artists to albums. */
	private static final int ARTISTS_ALBUMS = 110;

	/** All albums. */
	private static final int ALBUMS = 200;
	/** Given album. */
	private static final int ALBUMS_ID = 201;
	/** Starred albums. */
	private static final int ALBUMS_ID_STARRED = 202;
	/** Songs from given album. */
	private static final int ALBUMS_ID_SONGS = 203;

	/** All songs. */
	private static final int SONGS = 300;
	/** Starred songs. */
	private static final int SONGS_STARRED = 301;
	/** Cached songs. */
	private static final int SONGS_CACHED = 302;
	/** Given song. */
	private static final int SONGS_ID = 303;

	/** All playlists. */
	private static final int PLAYLISTS = 400;
	/** Given playlist. */
	private static final int PLAYLISTS_ID = 401;
	/** Starred playlists. */
	private static final int PLAYLISTS_ID_STARRED = 402;
	/** Songs from given playlist. */
	private static final int PLAYLISTS_ID_SONGS = 405;
	/** Song from given playlist. */
	private static final int PLAYLISTS_ID_SONGS_ID = 406;
	/** Mapping from playlists to songs. */
	private static final int PLAYLISTS_SONGS = 410;

	/** All genres. */
	private static final int GENRES = 600;
	/** Given genre. */
	private static final int GENRES_ID = 601;
	/** Songs from given genre. */
	private static final int GENRES_ID_SONGS = 602;

	private static final int SEARCH_SUGGEST = 800;

	/**
	 * Build and return a {@link UriMatcher} that catches all {@link Uri}
	 * variations supported by this {@link ContentProvider}.
	 */
	private static UriMatcher buildUriMatcher() {
		final UriMatcher matcher = new UriMatcher(UriMatcher.NO_MATCH);
		final String authority = MusicContract.CONTENT_AUTHORITY;

		matcher.addURI(authority, "artists", ARTISTS);
		matcher.addURI(authority, "artists/*", ARTISTS_ID);
		matcher.addURI(authority, "artists/*/starred", ARTISTS_ID_STARRED);
		matcher.addURI(authority, "artists/*/albums", ARTISTS_ID_ALBUMS); // query
		matcher.addURI(authority, "artists/*/albums/*", ARTISTS_ID_ALBUMS_ID);
		matcher.addURI(authority, "artists/*/songs", ARTISTS_ID_SONGS);
		matcher.addURI(authority, "artists_albums", ARTISTS_ALBUMS);

		matcher.addURI(authority, "albums", ALBUMS);
		matcher.addURI(authority, "albums/*", ALBUMS_ID);
		matcher.addURI(authority, "albums/*/starred", ALBUMS_ID_STARRED);
		matcher.addURI(authority, "albums/*/songs", ALBUMS_ID_SONGS);

		matcher.addURI(authority, "songs", SONGS);
		matcher.addURI(authority, "songs/starred", SONGS_STARRED);
		matcher.addURI(authority, "songs/cached", SONGS_CACHED);
		matcher.addURI(authority, "songs/*", SONGS_ID);

		matcher.addURI(authority, "playlists", PLAYLISTS);
		matcher.addURI(authority, "playlists/*", PLAYLISTS_ID);
		matcher.addURI(authority, "playlists/*/starred", PLAYLISTS_ID_STARRED);
		matcher.addURI(authority, "playlists/*/songs", PLAYLISTS_ID_SONGS);
		matcher.addURI(authority, "playlists/*/songs/*", PLAYLISTS_ID_SONGS_ID);
		matcher.addURI(authority, "playlists_songs", PLAYLISTS_SONGS);

		matcher.addURI(authority, "genres", GENRES);
		matcher.addURI(authority, "genres/*", GENRES_ID);
		matcher.addURI(authority, "genres/*/songs", GENRES_ID_SONGS);

		matcher.addURI(authority, "search_suggest_query", SEARCH_SUGGEST);

		return matcher;
	}

	@Override
	public boolean onCreate() {
		mOpenHelper = new MusicDatabase(getContext());
		return true;
	}

	private void deleteDatabase() {
		mOpenHelper.close();
		Context context = getContext();
		MusicDatabase.deleteDatabase(context);
		mOpenHelper = new MusicDatabase(getContext());
	}

	/** {@inheritDoc} */
	@Override
	public String getType(Uri uri) {
		final int match = sUriMatcher.match(uri);
		switch (match) {
		case ARTISTS:
			return Artists.CONTENT_TYPE;
		case ARTISTS_ID:
			return Artists.CONTENT_ITEM_TYPE;
		case ARTISTS_ID_STARRED:
			return Artists.CONTENT_ITEM_TYPE;
		case ARTISTS_ID_ALBUMS:
			return Albums.CONTENT_TYPE;
		case ARTISTS_ID_ALBUMS_ID:
			return ArtistAlbums.CONTENT_ITEM_TYPE;
		case ARTISTS_ID_SONGS:
			return Songs.CONTENT_TYPE;

		case ALBUMS:
			return Albums.CONTENT_TYPE;
		case ALBUMS_ID:
			return Albums.CONTENT_ITEM_TYPE;
		case ALBUMS_ID_STARRED:
			return Albums.CONTENT_TYPE;
		case ALBUMS_ID_SONGS:
			return Songs.CONTENT_TYPE;

		case SONGS:
			return Songs.CONTENT_TYPE;
		case SONGS_ID:
			return Songs.CONTENT_ITEM_TYPE;
		case SONGS_STARRED:
			return Songs.CONTENT_TYPE;
		case SONGS_CACHED:
			return Songs.CONTENT_TYPE;

		case PLAYLISTS:
			return Playlists.CONTENT_TYPE;
		case PLAYLISTS_ID:
			return Playlists.CONTENT_ITEM_TYPE;
		case PLAYLISTS_ID_STARRED:
			return Playlists.CONTENT_TYPE;
		case PLAYLISTS_ID_SONGS:
			return Songs.CONTENT_TYPE;
		case PLAYLISTS_ID_SONGS_ID:
			return Songs.CONTENT_ITEM_TYPE;

		case GENRES:
			return Genres.CONTENT_TYPE;
		case GENRES_ID:
			return Genres.CONTENT_ITEM_TYPE;
		case GENRES_ID_SONGS:
			return Songs.CONTENT_TYPE;

		default:
			LOGE(TAG, "Unknown uri: " + uri);
			return null;
		}
	}

	/** {@inheritDoc} */
	@Override
	public Cursor query(Uri uri, String[] projection, String selection,
			String[] selectionArgs, String sortOrder) {
		LOGV(TAG, "query(uri=" + uri + ", proj=" + Arrays.toString(projection)
				+ ")");
		final SQLiteDatabase db = mOpenHelper.getReadableDatabase();

		final SelectionBuilder builder = buildExpandedSelection(uri);
		Cursor cursor = builder.where(selection, selectionArgs).query(db,
				projection, sortOrder);
		ContentResolver resolver = getContext().getContentResolver();
		cursor.setNotificationUri(resolver, uri);
		return cursor;
	}

	/** {@inheritDoc} */
	@Override
	public Uri insert(Uri uri, ContentValues values) {
		LOGV(TAG, "insert(uri=" + uri + ", values=" + values.toString() + ")");
		final SQLiteDatabase db = mOpenHelper.getWritableDatabase();
		final int match = sUriMatcher.match(uri);
		boolean syncToNetwork = !MusicContract.hasCallerIsSyncAdapterParameter(uri);
		switch (match) {
		case ARTISTS: {
			Log.d(TAG, "Inserting into artists: " + values.toString());
			db.insertOrThrow(Tables.ARTISTS, null, values);
			getContext().getContentResolver().notifyChange(uri, null, false);
			return Artists.buildArtistUri(values.getAsString(Artists.ARTIST_ID));
		}
		case ARTISTS_ID_ALBUMS:
			//$FALL-THROUGH$ cheat for saving memory when generating general URI
		case ARTISTS_ID_ALBUMS_ID: {
			Log.d(TAG, "Inserting into album: " + values.toString());
			db.insertOrThrow(Tables.ARTISTS_ALBUMS, null, values);
			getContext().getContentResolver().notifyChange(uri, null, false);
			return Artists.buildAlbumUri(
					values.getAsString(ArtistAlbums.ARTIST_ID),
					values.getAsString(ArtistAlbums.ALBUM_ID));
		}
		case ALBUMS: {
			Log.d(TAG, "Inserting into albums: " + values.toString());
			db.insertOrThrow(Tables.ALBUMS, null, values);
			getContext().getContentResolver().notifyChange(uri, null, false);
			return Albums.buildAlbumUri(values.getAsString(Albums.ALBUM_ID));
		}
		case SONGS: {
			Log.d(TAG, "Inserting into songs: " + values.toString());
			db.insertOrThrow(Tables.SONGS, null, values);
			getContext().getContentResolver().notifyChange(uri, null, false);
			insertGenre(db, values.getAsString(Songs.SONG_GENRE));
			return Songs.buildSongUri(values.getAsString(Songs.SONG_ID));
		}
		case PLAYLISTS: {
			Log.d(TAG, "Inserting into playlists: " + values.toString());
			db.insertOrThrow(Tables.PLAYLISTS, null, values);
			getContext().getContentResolver().notifyChange(uri, null, syncToNetwork);
			return Playlists.buildPlaylistUri(values
					.getAsString(Playlists.PLAYLIST_ID));
		}
		case PLAYLISTS_ID_SONGS:
			//$FALL-THROUGH$ cheat for saving memory when generating general URI
		case PLAYLISTS_ID_SONGS_ID: {
			Log.i(TAG, "Inserting into playlist: " + values.toString());
			db.insertOrThrow(Tables.PLAYLISTS_SONGS, null, values);
			getContext().getContentResolver().notifyChange(uri, null, syncToNetwork);
			return Playlists.buildPlaylistSongUri(
					values.getAsString(PlaylistSongs.PLAYLIST_ID),
					values.getAsString(PlaylistSongs.SONG_ID));
		}
		case GENRES: {
			Log.d(TAG, "Inserting into genres: " + values.toString());
			db.insertOrThrow(Tables.GENRES, null, values);
			getContext().getContentResolver().notifyChange(uri, null, false);
			return Genres.buildGenreUri(values.getAsString(Genres.GENRE_ID));
		}
		default: {
			LOGE(TAG, "Unknown uri: " + uri);
			return null;
		}
		}
	}
	
	private void insertGenre(SQLiteDatabase db, String genre) {
		String lcGenre = genre != null ? genre.trim().toLowerCase() : "";
		if (!TextUtils.isEmpty(lcGenre) && !lcGenre.equals("null") && !lcGenre.equals("genre")) {
			// XXX Do we want all genres uppercase? If so, toUpperCase() genre below.
			ContentValues genreValues = MusicContentValues.forGenre(genre.trim());
			db.insertOrThrow(Tables.GENRES, null, genreValues);
		}
	}

	/** {@inheritDoc} */
	@Override
	public int update(Uri uri, ContentValues values, String selection,
			String[] selectionArgs) {
		LOGV(TAG, "update(uri=" + uri + ", values=" + values.toString() + ")");
		final SQLiteDatabase db = mOpenHelper.getWritableDatabase();
		final SelectionBuilder builder = buildSimpleSelection(uri);
		int retVal = builder.where(selection, selectionArgs).update(db, values);
		boolean syncToNetwork = !MusicContract.hasCallerIsSyncAdapterParameter(uri);
		ContentResolver resolver = getContext().getContentResolver();
		resolver.notifyChange(uri, null, syncToNetwork);

		String genre = values.getAsString(Songs.SONG_GENRE);
		if (genre != null) insertGenre(db, genre);
		
		return retVal;
	}

	/** {@inheritDoc} */
	@Override
	public int delete(Uri uri, String selection, String[] selectionArgs) {
		LOGV(TAG, "delete(uri=" + uri + ")");
		if (uri == MusicContract.BASE_CONTENT_URI) {
			// Handle whole database deletes (e.g. when signing out)
			deleteDatabase();
			getContext().getContentResolver().notifyChange(uri, null, false);
			return 1;
		}
		final SQLiteDatabase db = mOpenHelper.getWritableDatabase();
		final SelectionBuilder builder = buildSimpleSelection(uri);
		int retVal = builder.where(selection, selectionArgs).delete(db);
		getContext().getContentResolver().notifyChange(uri, null, false);
		// !MusicContract.hasCallerIsSyncAdapterParameter(uri));
		return retVal;
	}

	/**
	 * Apply the given set of {@link ContentProviderOperation}, executing inside
	 * a {@link SQLiteDatabase} transaction. All changes will be rolled back if
	 * any single one fails.
	 */
	@Override
	public ContentProviderResult[] applyBatch(
			ArrayList<ContentProviderOperation> operations)
			throws OperationApplicationException {
		final SQLiteDatabase db = mOpenHelper.getWritableDatabase();
		db.beginTransaction();
		try {
			final int numOperations = operations.size();
			final ContentProviderResult[] results = new ContentProviderResult[numOperations];
			for (int i = 0; i < numOperations; i++) {
				results[i] = operations.get(i).apply(this, results, i);
				
			}
			db.setTransactionSuccessful();
			return results;
		} finally {
			db.endTransaction();
		}
	}

	/** {@link SelectionBuilder} for UPDATE and DELETE operations. */
	private SelectionBuilder buildSimpleSelection(Uri uri) {
		final SelectionBuilder builder = new SelectionBuilder();
		final int match = sUriMatcher.match(uri);
		switch (match) {
		case ARTISTS: {
			return builder.table(Tables.ARTISTS);
		}
		case ARTISTS_ID: {
			final String artistId = Artists.getArtistId(uri);
			return builder.table(Tables.ARTISTS)
					.where(Artists.ARTIST_ID + "=?", artistId);
		}
		case ARTISTS_ID_STARRED: {
			return builder.table(Tables.ARTISTS)
					.where(Artists.STARRED + "=1");
		}
		case ARTISTS_ID_ALBUMS: {
			return builder.table(Tables.ARTISTS_ALBUMS);
		}
		case ARTISTS_ID_ALBUMS_ID: {
			final String artistId = Artists.getArtistId(uri);
			final String albumId = Artists.getAlbumId(uri);
			return builder.table(Tables.ARTISTS_ALBUMS)
					.where(ArtistAlbums.ARTIST_ID + "=?", artistId)
					.where(ArtistAlbums.ALBUM_ID + "=?", albumId);
		}
		case ARTISTS_ID_SONGS: {
			final String artistId = Artists.getArtistId(uri);
			return builder.table(Tables.SONGS)
					.where(Songs.SONG_ARTIST_ID + "=?", artistId);
		}
		case ARTISTS_ALBUMS: {
			return builder.table(Tables.ARTISTS_ALBUMS);
		}
		
		case ALBUMS: {
			return builder.table(Tables.ALBUMS);
		}
		case ALBUMS_ID: {
			final String albumId = Albums.getAlbumId(uri);
			return builder.table(Tables.ALBUMS)
					.where(Albums.ALBUM_ID + "=?", albumId);
		}
		case ALBUMS_ID_STARRED: {
			return builder.table(Tables.ALBUMS)
					.where(Albums.STARRED + "=1");
		}
		case ALBUMS_ID_SONGS: {
			final String albumId = Albums.getAlbumId(uri);
			return builder.table(Tables.SONGS)
					.where(Songs.SONG_ALBUM_ID + "=?", albumId);
		}
		
		case SONGS: {
			return builder.table(Tables.SONGS);
		}
		case SONGS_ID: {
			final String songId = Songs.getSongId(uri);
			return builder.table(Tables.SONGS)
					.where(Songs.SONG_ID + "=?", songId);
		}
		case SONGS_STARRED: {
			return builder.table(Tables.SONGS)
					.where(Songs.STARRED + "=1");
		}
		case SONGS_CACHED: {
			return builder.table(Tables.SONGS)
					.where(Songs.SONG_PATH + " IS NOT NULL");
		}
		
		case PLAYLISTS: {
			return builder.table(Tables.PLAYLISTS);
		}
		case PLAYLISTS_ID: {
			final String playlistId = Playlists.getPlaylistId(uri);
			return builder.table(Tables.PLAYLISTS)
					.where(Playlists.PLAYLIST_ID + "=?", playlistId);
		}
		case PLAYLISTS_ID_STARRED: {
			return builder.table(Tables.PLAYLISTS)
					.where(Playlists.STARRED + "=1");
		}
		case PLAYLISTS_ID_SONGS: {
			final String playlistId = Playlists.getPlaylistId(uri);
			return builder.table(Tables.PLAYLISTS_SONGS)
					.where(PlaylistSongs.PLAYLIST_ID + "=?", playlistId);
		}
		case PLAYLISTS_ID_SONGS_ID: {
			final String playlistId = Playlists.getPlaylistId(uri);
			final String songId = Playlists.getSongId(uri);
			return builder.table(Tables.PLAYLISTS_SONGS)
					.where(Playlists.PLAYLIST_ID + "=?", playlistId)
					.where(Songs.SONG_ID + "=?", songId);
		}
		case PLAYLISTS_SONGS: {
			return builder.table(Tables.PLAYLISTS_SONGS);
		}
		
		case GENRES: {
			return builder.table(Tables.GENRES);
		}
		case GENRES_ID: {
			final String genreId = Genres.getGenreId(uri);
			return builder.table(Tables.GENRES)
					.where(Genres.GENRE_ID + "=?", genreId);
		}
		case GENRES_ID_SONGS: {
			final String genreId = Genres.getGenreId(uri);
			return builder.table(Tables.SONGS)
					.where(Songs.SONG_GENRE_ID + "=?", genreId);
		}
		
		default: {
			LOGE(TAG, "Unknown uri: " + uri);
			return null;
		}
		}
	}

	/**
	 * Build an expanded {@link SelectionBuilder} to match the requested
	 * {@link Uri}, suitable to QUERY.
	 */
	private SelectionBuilder buildExpandedSelection(Uri uri) {
		final SelectionBuilder builder = new SelectionBuilder();
		final int match = sUriMatcher.match(uri);
		switch (match) {
		case ARTISTS: {
			return builder.table(Tables.ARTISTS);
		}
		case ARTISTS_ID: {
			final String artistId = Artists.getArtistId(uri);
			return builder.table(Tables.ARTISTS)
					.where(Artists.ARTIST_ID + "=?", artistId);
		}
		case ARTISTS_ID_STARRED: {
			return builder.table(Tables.ARTISTS)
					.where(Artists.STARRED + "=1");
		}
		case ARTISTS_ID_ALBUMS: {
			final String artistId = Artists.getArtistId(uri);
			return builder.table(Tables.ARTISTS_JOIN_ALBUMS)
					.where(ArtistAlbums.ARTIST_ID + "=?", artistId)
					.mapToTable(Albums._ID, Tables.ALBUMS)
					.mapToTable(Albums.ALBUM_ID, Tables.ALBUMS);
		}
		case ARTISTS_ID_ALBUMS_ID: {
			final String artistId = Artists.getArtistId(uri);
			final String albumId = Artists.getAlbumId(uri);
			return builder.table(Tables.ARTISTS_JOIN_ALBUMS)
					.where(ArtistAlbums.ARTIST_ID + "=?", artistId)
					.where(ArtistAlbums.ALBUM_ID + "=?", albumId)
					.mapToTable(Albums._ID, Tables.ALBUMS)
					.mapToTable(Albums.ALBUM_ID, Tables.ALBUMS);
		}
		case ARTISTS_ID_SONGS: {
			final String artistId = Artists.getArtistId(uri);
			return builder.table(Tables.SONGS)
					.where(Songs.SONG_ARTIST_ID + "=?", artistId);
		}
		
		case ALBUMS: {
			return builder.table(Tables.ALBUMS);
		}
		case ALBUMS_ID: {
			final String albumId = Albums.getAlbumId(uri);
			return builder.table(Tables.ALBUMS)
					.where(Albums.ALBUM_ID + "=?", albumId);
		}
		case ALBUMS_ID_STARRED: {
			return builder.table(Tables.ALBUMS)
					.where(Albums.STARRED + "=1");
		}
		case ALBUMS_ID_SONGS: {
			final String albumId = Albums.getAlbumId(uri);
			return builder.table(Tables.SONGS)
					.where(Songs.SONG_ALBUM_ID + "=?", albumId);
		}
		
		case SONGS: {
			return builder.table(Tables.SONGS);
		}
		case SONGS_ID: {
			final String songId = Songs.getSongId(uri);
			return builder.table(Tables.SONGS)
					.where(Songs.SONG_ID + "=?", songId);
		}
		case SONGS_STARRED: {
			return builder.table(Tables.SONGS).where(Songs.STARRED + "=1");
		}
		case SONGS_CACHED: {
			return builder.table(Tables.SONGS).where(Songs.SONG_PATH + " IS NOT NULL");
		}
		
		case PLAYLISTS: {
			return builder.table(Tables.PLAYLISTS);
		}
		case PLAYLISTS_ID: {
			final String playlistId = Playlists.getPlaylistId(uri);
			return builder.table(Tables.PLAYLISTS)
					.where(Playlists.PLAYLIST_ID + "=?", playlistId);
		}
		case PLAYLISTS_ID_STARRED: {
			return builder.table(Tables.PLAYLISTS)
					.where(Playlists.STARRED + "=1");
		}
		case PLAYLISTS_ID_SONGS: {
			final String playlistId = Playlists.getPlaylistId(uri);
			return builder.table(Tables.PLAYLIST_SONGS_JOIN_SONGS)
					.where(PlaylistSongs.PLAYLIST_ID + "=?", playlistId)
					.mapToTable(PlaylistSongs._ID, Tables.PLAYLISTS_SONGS)
					.mapToTable(PlaylistSongs.SONG_ID, Tables.SONGS);
		}
		case PLAYLISTS_ID_SONGS_ID: {
			final String playlistId = Playlists.getPlaylistId(uri);
			final String songId = Playlists.getSongId(uri);
			return builder.table(Tables.PLAYLIST_SONGS_JOIN_SONGS)
					.where(PlaylistSongs.PLAYLIST_ID + "=?", playlistId)
					.where(PlaylistSongs.SONG_ID + "=?", songId)
					.mapToTable(PlaylistSongs._ID, Tables.PLAYLISTS_SONGS)
					.mapToTable(PlaylistSongs.SONG_ID, Tables.SONGS)
					.mapToTable(PlaylistSongs.STARRED, Tables.SONGS);
		}
		
		case GENRES: {
			return builder.table(Tables.GENRES);
		}
		case GENRES_ID: {
			final String genreId = Genres.getGenreId(uri);
			return builder.table(Tables.GENRES)
					.where(Genres.GENRE_ID + "=?", genreId);
		}
		case GENRES_ID_SONGS: {
			final String genreId = Genres.getGenreId(uri);
			return builder.table(Tables.SONGS)
					.where(Songs.SONG_GENRE_ID + "=?", genreId);
		}
		default: {
			LOGE(TAG, "Unknown uri: " + uri);
			return null;
		}
		}
	}

	@Override
	public ParcelFileDescriptor openFile(Uri uri, String mode)
			throws FileNotFoundException {
		final int match = sUriMatcher.match(uri);
		switch (match) {
		default: {
			throw new IllegalArgumentException("Unknown uri: " + uri);
		}
		}
	}
	
	@Override
	public void shutdown() {
		// Do nothing. Tests close the fixture database themselves.
	}

	/**
	 * {@link MusicContract} fields that are fully qualified with a specific
	 * parent {@link Tables}. Used when needed to work around SQL ambiguity.
	 */
	@SuppressWarnings("unused")
	private interface Qualified
	{
	}
	
	MusicDatabase getOpenHelperForTest() {
		return mOpenHelper;
	}
}
