/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.provider;

import static com.ubuntuone.android.music.util.LogUtils.makeLogTag;

import java.util.ArrayList;
import java.util.Random;

import android.content.ContentResolver;
import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.net.Uri;
import android.util.Log;

import com.ubuntuone.android.music.provider.MusicContract.Albums;
import com.ubuntuone.android.music.provider.MusicContract.Artists;
import com.ubuntuone.android.music.provider.MusicContract.Genres;
import com.ubuntuone.android.music.provider.MusicContract.PlaylistSongs;
import com.ubuntuone.android.music.provider.MusicContract.Playlists;
import com.ubuntuone.android.music.provider.MusicContract.Songs;
import com.ubuntuone.android.music.provider.MusicContract.StarredColumn;
import com.ubuntuone.android.music.util.HashUtils;

public class MusicProviderUtils
{
	private static final String TAG = makeLogTag(MusicProviderUtils.class);
	
	/* package */ static final String STARRED_NAME = "_starred";
	public static final String STARRED_ID = HashUtils.md5(STARRED_NAME);
	
	/* package */ static final String CACHED_NAME = "_cached";
	public static final String CACHED_ID = HashUtils.md5(CACHED_NAME);
	
	public static final String QUEUE_NAME = "_queue";
	public static final String QUEUE_ID = HashUtils.md5(QUEUE_NAME);
	
	public static final Uri QUEUE_URI = Playlists.buildPlaylistSongsUri(QUEUE_ID);
	
	public static int getArtistSongCount(Context context, Uri artistUri) {
		ContentResolver resolver = context.getContentResolver();
		
		String[] projection = new String[] { Artists.SONG_COUNT };
		Cursor cursor = null;
		int count = 0;
		try {
			cursor = resolver.query(artistUri, projection, null, null, null);
			if (cursor != null && cursor.moveToFirst()) {
				count= cursor.getInt(cursor.getColumnIndex(Artists.SONG_COUNT));
			}
		} finally {
			if (cursor != null) cursor.close();
		}
		return count;
	}
	
	public static int getAlbumSongCount(Context context, Uri albumUri) {
		// API idea: Album entity could hold its computed song count.
		ContentResolver resolver = context.getContentResolver();
		String albumId = Albums.getAlbumId(albumUri);
		Uri albumSongsUri = Albums.buildSongsUri(albumId);
		
		String[] projection = new String[] { Songs._ID };
		Cursor cursor = null;
		int count = 0;
		try {
			cursor = resolver.query(albumSongsUri, projection, null, null, null);
			count = (cursor != null) ? cursor.getCount() : 0;
		} finally {
			if (cursor != null) cursor.close();
		}
		return count;
	}
	
	/* package */ static int getSongCount(Context context, String selection) {
		ContentResolver resolver = context.getContentResolver();
		String[] projection = new String[] { Songs._ID };
		Cursor cursor = null;
		int count = 0;
		try {
			cursor = resolver.query(Songs.CONTENT_URI, projection, selection, null, null);
			count = (cursor != null) ? cursor.getCount() : 0;
		} finally {
			if (cursor != null) cursor.close();
		}
		return count;
	}
	
	public static int getSongCount(Context context) {
		return getSongCount(context, null);
	}
	
	public static int getStarredSongCount(Context context) {
		String selection = Songs.STARRED + "=1";
		return getSongCount(context, selection);
	}
	
	public static int getSongStarred(Context context, Uri songUri) {
		int starred = 0;
		ContentResolver resolver = context.getContentResolver();
		Cursor cursor = null;
		try {
			String[] projection = new String[] { Songs.STARRED };
			cursor = resolver.query(songUri, projection, null, null, null);
			if (cursor != null && cursor.moveToFirst()) {
				starred = cursor.getInt(cursor.getColumnIndex(Songs.STARRED));
				starred = (starred != 0) ? 1 : 0;
			}
		} finally {
			if (cursor != null) cursor.close();
		}
		return starred;
	}
	
	public static void setSongStarred(Context context, Uri songUri, boolean isStarred) {
		ContentResolver resolver = context.getContentResolver();
		
		int starred = isStarred ? 1 : 0;
		ContentValues values = new ContentValues();
		values.put(Songs.STARRED, starred);
		
		resolver.update(songUri, values, null, null);
	}
	
	public static void toggleSongStarred(Context context, Uri songUri) {
		ContentResolver resolver = context.getContentResolver();
		int starred = 0;
		
		Cursor cursor = null;
		String albumId = null;
		try {
			String[] projection = new String[] { Songs.SONG_ALBUM_ID, Songs.STARRED };
			cursor = resolver.query(songUri, projection, null, null, null);
			if (cursor != null && cursor.moveToFirst()) {
				albumId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ALBUM_ID));
				starred = cursor.getInt(cursor.getColumnIndex(Songs.STARRED));
				starred = starred != 0 ? 1 : 0;
			}
		} finally {
			if (cursor != null) cursor.close();
		}
		
		// Flip the starred flag.
		starred = 1 - starred;
		ContentValues values = new ContentValues();
		values.put(Songs.STARRED, starred);
		
		resolver.update(songUri, values, null, null);
		
		if (starred == 0 && albumId != null) {
			// Unstar the album.
			Uri albumUri = Albums.buildAlbumUri(albumId);
			resolver.update(albumUri, values, null, null);
		}
	}
	
	// TODO test
	public static void toggleAlbumStarred(Context context, Uri albumUri) {
		ContentResolver resolver = context.getContentResolver();
		int starred = 0;
		String albumId = Albums.getAlbumId(albumUri);
		Uri albumSongsUri = Albums.buildSongsUri(albumId);
		
		Cursor cursor = null;
		try {
			String[] projection = new String[] { Albums.STARRED };
			cursor = resolver.query(albumUri, projection, null, null, null);
			if (cursor != null && cursor.moveToFirst()) {
				starred = cursor.getInt(cursor.getColumnIndex(Songs.STARRED));
				starred = starred != 0 ? 1 : 0;
			}
		} finally {
			if (cursor != null) cursor.close();
		}
		
		// Flip the starred flag.
		starred = 1 - starred;
		ContentValues values = new ContentValues();
		values.put(StarredColumn.STARRED, starred);
		
		resolver.update(albumUri, values, null, null);
		resolver.update(albumSongsUri, values, null, null);
	}
	
	public static int getCachedSongCount(Context context) {
		String selection = Songs.SONG_PATH + " IS NOT NULL";
		return getSongCount(context, selection);
	}
	
	public static int getPlaylistSongCount(Context context, Uri playlistUri) {
		ContentResolver resolver = context.getContentResolver();
		String playlistId = Playlists.getPlaylistId(playlistUri);
		Uri playlistSongsUri = Playlists.buildPlaylistSongsUri(playlistId);
		
		String[] projection = new String[] { Songs._ID };
		Cursor cursor = null;
		int count = -1;
		try {
			cursor = resolver.query(playlistSongsUri, projection, null, null, null);
			count = (cursor != null) ? cursor.getCount() : 0;
		} finally {
			if (cursor != null) cursor.close();
		}
		return count;
	}
	
	public static ArrayList<String> getPlaylistSongIdList(Context context, String playlistId) {
		ContentResolver resolver = context.getContentResolver();
		Uri playlistSongsUri = Playlists.buildPlaylistSongsUri(playlistId);
		ArrayList<String> songIdList = new ArrayList<String>();
		
		String[] projection = new String[] { PlaylistSongs.SONG_ID };
		Cursor cursor = null;
		try {
			cursor = resolver.query(playlistSongsUri, projection, null, null, null);
			if (cursor != null && cursor.isBeforeFirst()) {
				while (cursor.moveToNext()) {
					String songId = cursor.getString(cursor.getColumnIndex(PlaylistSongs.SONG_ID));
					songIdList.add(songId);
				}
			}
		} finally {
			if (cursor != null) cursor.close();
		}
		
		return songIdList;
	}
	
	public static int getPlayQueueSongCount(Context context) {
		Uri playQueueUri = Playlists.buildPlaylistSongsUri(QUEUE_ID);
		return getPlaylistSongCount(context, playQueueUri);
	}

	public static void clearPlaylist(Context context, Uri playlistUri) {
		ContentResolver resolver = context.getContentResolver();
		String playlistId = Playlists.getPlaylistId(playlistUri);
		Uri playlistSongsUri = Playlists.buildPlaylistSongsUri(playlistId);
		resolver.delete(playlistSongsUri, null, null);
	}

	public static void clearPlayQueue(Context context) {
		Uri playQueueUri = Playlists.buildPlaylistSongsUri(MusicProviderUtils.QUEUE_ID);
		clearPlaylist(context, playQueueUri);
	}
	
	/**
	 * Updates playlist song count, saves one request to the server upon
	 * enqueuing content.
	 * 
	 * @param context
	 *            The context to use.
	 * @param playlistUri
	 *            Playlist Uri of which song count to update.
	 */
	public static void updatePlaylistSongCount(Context context, Uri playlistUri) {
		ContentResolver resolver = context.getContentResolver();
		ContentValues values = new ContentValues();
		int songCount = MusicProviderUtils.getPlaylistSongCount(context, playlistUri);
		values.put(Playlists.PLAYLIST_SONG_COUNT, songCount);
		resolver.update(playlistUri, values, null, null);
	}
	
	/**
	 * Flags the playlist as dirty for sync up. Next time metadata is refreshed,
	 * dirty playlists are first saved to the server.
	 * 
	 * @param context
	 *            The context to use.
	 * @param playlistUri
	 *            Uri of the playlist to flag.
	 * @param isDirty
	 *            true to mark for sync up, false otherwise.
	 */
	public static void flagPlaylistForSync(Context context, Uri playlistUri, boolean isDirty) {
		ContentResolver resolver = context.getContentResolver();
		ContentValues values = new ContentValues();
		values.put(Playlists.DIRTY, isDirty);
		resolver.update(playlistUri, values, null, null);
	}
	
	public static void enqueueSong(Context context, Uri songUri, Uri playlistUri) {
		ContentResolver resolver = context.getContentResolver();
		String playlistId = Playlists.getPlaylistId(playlistUri);
		String songId = Songs.getSongId(songUri);
		Uri playlistSongsUri = Playlists.buildPlaylistSongsUri(playlistId);
		
		ContentValues values = MusicContentValues.forPlaylistSong(playlistId, songId);
		resolver.insert(playlistSongsUri, values);
		
		if (!QUEUE_ID.equals(playlistId)) {
			updatePlaylistSongCount(context, playlistUri);
			flagPlaylistForSync(context, playlistUri, true);
		}
	}
	
	public static void enqueueSong(Context context, Uri songUri) {
		enqueueSong(context, songUri, QUEUE_URI);
	}
	
	public static void clearAndEnqueueAllSongs(Context context) {
		MusicProviderUtils.clearPlayQueue(context);
		
		ContentResolver resolver = context.getContentResolver();
		String[] projection = new String[] { Songs.SONG_ID };
		Cursor cursor = null;
		try {
			cursor = resolver.query(Songs.CONTENT_URI, projection, null, null, null);
			if (cursor != null && cursor.isBeforeFirst()) {
				while (cursor.moveToNext()) {
					String songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
					ContentValues values = MusicContentValues.forPlaylistSong(
							MusicProviderUtils.QUEUE_ID, songId);
					resolver.insert(MusicProviderUtils.QUEUE_URI, values);
				}
			}
		} finally {
			if (cursor != null) cursor.close();
		}
	}
	
	public static void clearAndEnqueueSong(Context context, Uri songUri, Uri playlistUri) {
		clearPlaylist(context, playlistUri);
		enqueueSong(context, songUri, playlistUri);
	}
	
	public static void clearAndEnqueueSong(Context context, Uri songUri) {
		clearPlayQueue(context);
		enqueueSong(context, songUri);
	}

	public static void enqueueArtist(Context context, Uri artistUri, Uri playlistUri) {
		ContentResolver resolver = context.getContentResolver();
		String artistId = Artists.getArtistId(artistUri);
		Uri artistSongsUri = Artists.buildSongsUri(artistId);
		String[] projection = new String[] { Songs.SONG_ID, Songs.SONG_TRACK };
		// Same song sort as in albums. Do we need anything more fancy?
		String sortOrder = Albums.DEFAULT_SONG_SORT;
		
		Cursor cursor = null;
		try {
			cursor = resolver.query(artistSongsUri, projection, null, null, sortOrder);
			if (cursor != null && cursor.isBeforeFirst()) {
				while (cursor.moveToNext()) {
					String songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
					Uri songUri = Songs.buildSongUri(songId);
					MusicProviderUtils.enqueueSong(context, songUri, playlistUri);
				}
			}
		} finally {
			if (cursor != null) cursor.close();
		}
	}
	
	public static void enqueueArtist(Context context, Uri artistUri) {
		enqueueArtist(context, artistUri, QUEUE_URI);
	}

	public static void clearAndEnqueueArtist(Context context, Uri artistUri) {
		MusicProviderUtils.clearPlayQueue(context);
		enqueueArtist(context, artistUri);
	}

	public static void enqueueAlbum(Context context, Uri albumUri, Uri playlistUri) {
		ContentResolver resolver = context.getContentResolver();
		String albumId = Albums.getAlbumId(albumUri);
		Uri albumSongsUri = Albums.buildSongsUri(albumId);
		String[] projection = new String[] { Songs.SONG_ID, Songs.SONG_TRACK };
		String sortOrder = Albums.DEFAULT_SONG_SORT;
		
		Cursor cursor = null;
		try {
			cursor = resolver.query(albumSongsUri, projection, null, null, sortOrder);
			if (cursor != null && cursor.isBeforeFirst()) {
				while (cursor.moveToNext()) {
					String songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
					Uri songUri = Songs.buildSongUri(songId);
					MusicProviderUtils.enqueueSong(context, songUri, playlistUri);
				}
			}
		} finally {
			if (cursor != null) cursor.close();
		}
	}
	
	public static void enqueueAlbum(Context context, Uri albumUri) {
		enqueueAlbum(context, albumUri, QUEUE_URI);
	}

	public static void clearAndEnqueueAlbum(Context context, Uri albumUri) {
		MusicProviderUtils.clearPlayQueue(context);
		enqueueAlbum(context, albumUri);
	}
	
	public static void enqueuePlaylist(Context context, Uri srcPlaylistUri, Uri playlistUri) {
		ContentResolver resolver = context.getContentResolver();
		String srcPlaylistId = Playlists.getPlaylistId(srcPlaylistUri);
		Uri srcPlaylistSongsUri = null;
		String sortOrder = null;
		if (srcPlaylistId.equals(STARRED_ID)) {
			srcPlaylistSongsUri = Songs.STARRED_SONGS_URI;
			sortOrder = Songs.DEFAULT_SORT;
		} else if (srcPlaylistId.equals(CACHED_ID)) {
			srcPlaylistSongsUri = Songs.CACHED_SONGS_URI;
			sortOrder = Songs.DEFAULT_SORT;
		} else {
			srcPlaylistSongsUri = Playlists.buildPlaylistSongsUri(srcPlaylistId);
			sortOrder = PlaylistSongs.DEFAULT_SORT;
		}
		String[] projection = new String[] { Songs.SONG_ID, Songs.SONG_TRACK };
		
		Cursor cursor = null;
		try {
			cursor = resolver.query(srcPlaylistSongsUri, projection, null, null, sortOrder);
			if (cursor != null && cursor.isBeforeFirst()) {
				while (cursor.moveToNext()) {
					String songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
					Uri songUri = Songs.buildSongUri(songId);
					MusicProviderUtils.enqueueSong(context, songUri, playlistUri);
				}
			}
		} finally {
			if (cursor != null) cursor.close();
		}
	}
	
	public static void enqueuePlaylist(Context context, Uri playlistUri) {
		enqueuePlaylist(context, playlistUri, QUEUE_URI);
	}

	public static void clearAndEnqueuePlaylist(Context context, Uri playlistUri) {
		MusicProviderUtils.clearPlayQueue(context);
		enqueuePlaylist(context, playlistUri);
	}
	
	public static void enqueueGenre(Context context, Uri genreUri) {
		ContentResolver resolver = context.getContentResolver();
		String genreId = Genres.getGenreId(genreUri);
		Uri genreSongsUri = Genres.buildGenreSongsUri(genreId);
		String[] projection = new String[] { Songs.SONG_ID, Songs.SONG_TRACK };
		String sortOrder = Albums.DEFAULT_SONG_SORT;
		
		Cursor cursor = null;
		try {
			cursor = resolver.query(genreSongsUri, projection, null, null, sortOrder);
			if (cursor != null && cursor.isBeforeFirst()) {
				while (cursor.moveToNext()) {
					String songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
					Uri songUri = Songs.buildSongUri(songId);
					MusicProviderUtils.enqueueSong(context, songUri, QUEUE_URI);
				}
			}
		} finally {
			if (cursor != null) cursor.close();
		}
	}
	
	public static void clearAndEnqueueGenre(Context context, Uri genreUri) {
		MusicProviderUtils.clearPlayQueue(context);
		enqueueGenre(context, genreUri);
	}
	
	public static void deletePlaylist(Context context, Uri playlistUri) {
		ContentResolver resolver = context.getContentResolver();
		String playlistId = Playlists.getPlaylistId(playlistUri);
		if (!MusicProviderUtils.QUEUE_ID.equals(playlistId)) {
			MusicProviderUtils.clearPlaylist(context, playlistUri);
			resolver.delete(playlistUri, null, null);
		}
	}
	
	// TODO test
	public static Uri getPlayingSong(Context context) {
		ContentResolver resolver = context.getContentResolver();
		String[] projection = new String[] { Songs.SONG_ID };
		Cursor cursor = null;
		Uri songUri = null;
		try {
			cursor = resolver.query(Songs.CONTENT_URI, projection,
					Songs.SONG_IS_PLAYING + "!=0", null, null);
			if (cursor != null && cursor.moveToFirst()) {
				String songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
				songUri = Songs.buildSongUri(songId);
			}
		} finally {
			if (cursor != null) cursor.close();
		}
		return songUri;
	}
	
	// TODO test
	public static void setPlayingSong(Context context, Uri songUri) {
		ContentResolver resolver = context.getContentResolver();
		ContentValues values = new ContentValues();
		values.put(Songs.SONG_IS_PLAYING, 0);
		// Reset any songs with SONG_IS_PLAYING flag set.
		String selection = Songs.SONG_IS_PLAYING + "!=0";
		resolver.update(Songs.CONTENT_URI, values, selection, null);
		// Set the flag on a single song.
		values.clear();
		values.put(Songs.SONG_IS_PLAYING, 1);
		resolver.update(songUri, values, null, null);
		// Update play queue list.
		resolver.notifyChange(QUEUE_URI, null);
	}
	
	public static void resetPlayingSongFlags(Context context) {
		ContentResolver resolver = context.getContentResolver();
		ContentValues values = new ContentValues();
		values.put(Songs.SONG_IS_PLAYING, 0);
		// Reset any songs with SONG_IS_PLAYING flag set.
		String selection = Songs.SONG_IS_PLAYING + "!=0";
		resolver.update(Songs.CONTENT_URI, values, selection, null);
		// Update play queue list.
		resolver.notifyChange(QUEUE_URI, null);
	}
	
	public static Uri getFirstPlayQueueSong(Context context, boolean shuffle) {
		Uri firstSongUri = null;
		if (shuffle) {
			firstSongUri = getFirstRandomPlayQueueSong(context);
		} else {
			firstSongUri = getFirstPlayQueueSong(context);
		}
		
		Log.d(TAG, "First song uri: " + firstSongUri);
		return firstSongUri;
	}
	
	// TODO test
	private static Uri getFirstPlayQueueSong(Context context) {
		ContentResolver resolver = context.getContentResolver();
		String[] projection = new String[] { Songs.SONG_ID };
		Cursor cursor = null;
		Uri songUri = null;
		try {
			cursor = resolver.query(QUEUE_URI, projection,
					null, null, PlaylistSongs.DEFAULT_SORT);
			if (cursor != null && cursor.moveToFirst()) {
				String songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
				songUri = Songs.buildSongUri(songId);
			} else {
				Log.e(TAG, "Could not get first play queue song.");
			}
		} finally {
			if (cursor != null) cursor.close();
		}
		return songUri;
	}
	
	// TODO test
	private static Uri getFirstRandomPlayQueueSong(Context context) {
		ContentResolver resolver = context.getContentResolver();
		String[] projection = new String[] { Songs.SONG_ID };
		Cursor cursor = null;
		Uri songUri = null;
		try {
			cursor = resolver.query(QUEUE_URI, projection,
					null, null, PlaylistSongs.SHUFFLE_SORT);
			if (cursor != null && cursor.moveToFirst()) {
				String songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
				songUri = Songs.buildSongUri(songId);
			}
		} finally {
			if (cursor != null) cursor.close();
		}
		return songUri;
	}
	
	public static Uri getLastPlayQueueSong(Context context, boolean shuffle) {
		Uri lastSongUri = null;
		if (shuffle) {
			lastSongUri = getLastRandomPlayQueueSong(context);
		} else {
			lastSongUri = getLastPlayQueueSong(context);
		}
		
		Log.d(TAG, "Last song uri: " + lastSongUri);
		return lastSongUri;
	}
	
	// TODO test
	private static Uri getLastPlayQueueSong(Context context) {
		ContentResolver resolver = context.getContentResolver();
		String[] projection = new String[] { Songs.SONG_ID };
		Cursor cursor = null;
		Uri songUri = null;
		try {
			cursor = resolver.query(QUEUE_URI, projection,
					null, null, PlaylistSongs.DEFAULT_SORT);
			if (cursor != null && cursor.moveToLast()) {
				String songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
				songUri = Songs.buildSongUri(songId);
			}
		} finally {
			if (cursor != null) cursor.close();
		}
		return songUri;
	}
	
	// TODO test
	private static Uri getLastRandomPlayQueueSong(Context context) {
		ContentResolver resolver = context.getContentResolver();
		String[] projection = new String[] { Songs.SONG_ID };
		Cursor cursor = null;
		Uri songUri = null;
		try {
			cursor = resolver.query(QUEUE_URI, projection,
					null, null, PlaylistSongs.SHUFFLE_SORT);
			if (cursor != null && cursor.moveToLast()) {
				String songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
				songUri = Songs.buildSongUri(songId);
			}
		} finally {
			if (cursor != null) cursor.close();
		}
		return songUri;
	}

	@SuppressWarnings("unused")
	private static long getPlayQueueSongRowId(Context context, Uri songUri) {
		ContentResolver resolver = context.getContentResolver();
		String[] projection = new String[] { Songs._ID }; 
		String songId = Songs.getSongId(songUri);
		String selection = "playlists_songs.song_id=?";
		String[] selectionArgs = new String[] { songId };
		Cursor cursor = null;
		
		long id = -1;
		try {
			cursor = resolver.query(QUEUE_URI, projection,
					selection, selectionArgs, null);
			if (cursor != null && cursor.moveToFirst()) {
				id = cursor.getLong(cursor.getColumnIndex(Songs._ID));
			}
		} finally {
			if (cursor != null) cursor.close();
		}
		return id;
	}
	
	private static long getPlayQueueSongShuffleOrder(Context context, Uri songUri) {
		ContentResolver resolver = context.getContentResolver();
		String[] projection = new String[] { PlaylistSongs.SHUFFLE_ORDER };
		
		String songId = Songs.getSongId(songUri);
		Uri queueSongUri = Playlists.buildPlaylistSongUri(QUEUE_ID, songId);
		
		long shuffleOrder = 0;
		Cursor cursor = null;
		try {
			cursor = resolver.query(queueSongUri, projection, null, null, null);
			if (cursor != null && cursor.moveToFirst()) {
				shuffleOrder = cursor.getLong(cursor.getColumnIndex(PlaylistSongs.SHUFFLE_ORDER));
			} else {
				Log.w(TAG, "Could not get play queue song shuffle order.");
			}
		} finally {
			if (cursor != null) cursor.close();
		}
		return shuffleOrder;
	}
	
//	public static void printPlayQueue(Context context) {
//		ContentResolver resolver = context.getContentResolver();
//		String[] projection = new String[] { PlaylistSongs._ID, PlaylistSongs.SHUFFLE_ORDER };
//		
//		Cursor cursor = null;
//		try {
//			cursor = resolver.query(QUEUE_URI, projection, null, null, PlaylistSongs.DEFAULT_SORT);
//			if (cursor != null && cursor.isBeforeFirst()) {
//				while (cursor.moveToNext()) {
//					long rowId = cursor.getLong(cursor.getColumnIndex(PlaylistSongs._ID));
//					long order = cursor.getLong(cursor.getColumnIndex(PlaylistSongs.SHUFFLE_ORDER));
//					
//					Log.i(TAG, String.format("queue: %d %d", rowId, order));
//				}
//			}
//		} finally {
//			if (cursor != null) cursor.close();
//		}
//	}
	
	// TODO test
	public static void reshufflePlayQueue(Context context) {
		ContentResolver resolver = context.getContentResolver();
		String[] projection = new String[] {
				PlaylistSongs.PLAYLIST_ID, PlaylistSongs.SONG_ID, PlaylistSongs._ID
			};
		
		Cursor cursor = null;
		Random random = new Random(System.currentTimeMillis());
		try {
			cursor = resolver.query(QUEUE_URI, projection, null, null, PlaylistSongs.DEFAULT_SORT);
			if (cursor != null && cursor.isBeforeFirst()) {
				while (cursor.moveToNext()) {
					// TODO Update based on row _id rather than full song URI.
					String playlistId = cursor.getString(cursor.getColumnIndex(PlaylistSongs.PLAYLIST_ID));
					String songId = cursor.getString(cursor.getColumnIndex(PlaylistSongs.SONG_ID));
					Uri queueSongUri = Playlists.buildPlaylistSongUri(playlistId, songId);
					ContentValues values = MusicContentValues.forShuffleOrder(random);
					
					resolver.update(queueSongUri, values, null, null);
				}
			} else {
				Log.w(TAG, "Could not reshuffle play queue.");
			}
		} finally {
			if (cursor != null) cursor.close();
		}
	}
	
	public static Uri getPrevPlayQueueSong(Context context, Uri songUri, boolean shuffle) {
		Uri prevSongUri = null;
		if (shuffle) {
			prevSongUri = getPrevRandomPlayQueueSong(context);
		} else {
			prevSongUri = getPrevPlayQueueSong(context, songUri);
		}
		
		Log.d(TAG, "Prev song uri: " + prevSongUri);
		return prevSongUri;
	}
	
	// TODO Fix this, this solution is super dumb.
	private static Uri getPrevPlayQueueSong(Context context, Uri songUri) {
		Log.d(TAG, "getPrevPlayQueueSong");
		
		if (songUri == null) {
			Log.w(TAG, "No prev queue song.");
			return null;
		}
		Log.d(TAG, "Looking up perv song uri for: " + songUri);

		ContentResolver resolver = context.getContentResolver();
		String[] projection = new String[] { Songs.SONG_ID };
		
		Cursor cursor = null;
		Uri prevSongUri = null;
		try {
			cursor = resolver.query(QUEUE_URI, projection, null, null, PlaylistSongs.DEFAULT_SORT);
			if (cursor != null && cursor.moveToFirst()) {
				final String songId = Songs.getSongId(songUri);
				String prevSongId = null;
				String currentSongId = null;
				do {
					prevSongId = currentSongId;
					currentSongId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
					if (!cursor.moveToNext()) {
						currentSongId = null;
						break;
					}
				} while (!songId.equals(currentSongId));
				if (prevSongId != null) {
					prevSongUri = Songs.buildSongUri(prevSongId);
				}
			} else {
				Log.w(TAG, "Could not get previous play queue song.");
			}
		} finally {
			if (cursor != null) cursor.close();
		}
		Log.d(TAG, "Prev song uri: " + prevSongUri);
		return prevSongUri;
	}
	
	// TODO test
	private static Uri getPrevRandomPlayQueueSong(Context context) {
		Log.d(TAG, "getPrevRandomPlayQueueSong");
		
		Uri songUri = getPlayingSong(context);
		if (songUri == null) return getFirstRandomPlayQueueSong(context);
		
		long shuffleOrder = getPlayQueueSongShuffleOrder(context, songUri);
		
		ContentResolver resolver = context.getContentResolver();
		String[] projection = new String[] { Songs.SONG_ID };
		String selection = PlaylistSongs.WAS_PLAYED + "=? AND "
				+ PlaylistSongs.SHUFFLE_ORDER + "<?";
		String[] selectionArgs = new String[] { "0", String.valueOf(shuffleOrder) };
		
		Cursor cursor = null;
		Uri prevSongUri = null;
		try {
			cursor = resolver.query(QUEUE_URI, projection,
					selection, selectionArgs, PlaylistSongs.SHUFFLE_SORT);
			if (cursor != null && cursor.moveToLast()) {
				String nextSongId = cursor.getString(
						cursor.getColumnIndex(PlaylistSongs.SONG_ID));
				prevSongUri = Songs.buildSongUri(nextSongId);
			} else {
				Log.w(TAG, "Could not get previous random play queue song.");
			}
		} finally {
			if (cursor != null) cursor.close();
		}
		
		return prevSongUri;
	}
	
	public static Uri getNextPlayQueueSong(Context context, Uri songUri, boolean shuffle) {
		Uri nextSongUri = null;
		if (shuffle) {
			nextSongUri = getNextRandomPlayQueueSong(context);
		} else {
			nextSongUri = getNextPlayQueueSong(context, songUri);
		}
		
		Log.d(TAG, "Next song uri: " + nextSongUri);
		return nextSongUri;
	}
	
	// TODO Fix this, this solution is super dumb.
	private static Uri getNextPlayQueueSong(Context context, Uri songUri) {
		if (songUri == null) {
			Log.w(TAG, "No next queue song.");
			return null;
		}
		Log.d(TAG, "Looking up next song uri for: " + songUri);

		ContentResolver resolver = context.getContentResolver();
		String[] projection = new String[] { Songs.SONG_ID };
		
		Cursor cursor = null;
		Uri nextSongUri = null;
		try {
			cursor = resolver.query(QUEUE_URI, projection, null, null, PlaylistSongs.DEFAULT_SORT);
			if (cursor != null && cursor.moveToFirst()) {
				final String songId = Songs.getSongId(songUri);
				String currentSongId = null;
				do {
					currentSongId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
					if (!cursor.moveToNext()) {
						currentSongId = null;
						break;
					}
				} while (!songId.equals(currentSongId));
				if (currentSongId != null) {
					String nextSongId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
					nextSongUri = Songs.buildSongUri(nextSongId);
				}
			} else {
				Log.w(TAG, "Could not get next play queue song.");
			}
		} finally {
			if (cursor != null) cursor.close();
		}
		return nextSongUri;
	}
	
	private static Uri getNextRandomPlayQueueSong(Context context) {
		Uri songUri = getPlayingSong(context);
		if (songUri == null) return getFirstRandomPlayQueueSong(context);
		
		long shuffleOrder = getPlayQueueSongShuffleOrder(context, songUri);
		
		ContentResolver resolver = context.getContentResolver();
		String[] projection = new String[] { Songs.SONG_ID };
		String selection = PlaylistSongs.WAS_PLAYED + "=? AND "
				+ PlaylistSongs.SHUFFLE_ORDER + ">?";
		String[] selectionArgs = new String[] { "0", String.valueOf(shuffleOrder) };
		
		Cursor cursor = null;
		Uri nextSongUri = null;
		try {
			cursor = resolver.query(QUEUE_URI, projection,
					selection, selectionArgs, PlaylistSongs.SHUFFLE_SORT);
			if (cursor != null && cursor.moveToFirst()) {
				String nextSongId = cursor.getString(
						cursor.getColumnIndex(PlaylistSongs.SONG_ID));
				nextSongUri = Songs.buildSongUri(nextSongId);
			} else {
				Log.w(TAG, "Could not get next random play queue song.");
			}
		} finally {
			if (cursor != null) cursor.close();
		}
		
		return nextSongUri;
	}
	
	// TODO test
	public static void setSongWasPlayedFlag(Context context, Uri songUri) {
		ContentResolver resolver = context.getContentResolver();
		String selection = PlaylistSongs.SONG_ID + "=?";
		String[] selectionArgs = new String[] { Songs.getSongId(songUri) };
		
		ContentValues values = new ContentValues();
		values.put(PlaylistSongs.WAS_PLAYED, true);
		
		resolver.update(QUEUE_URI, values, selection, selectionArgs);
	}
	
	// TODO test
	public static void resetSongWasPlayedFlags(Context context) {
		ContentResolver resolver = context.getContentResolver();
		
		ContentValues values = new ContentValues();
		values.put(PlaylistSongs.WAS_PLAYED, false);
		
		resolver.update(QUEUE_URI, values, null, null);
	}
	
	public static void setSongLastPlayedPosition(Context context, Uri songUri, int lastPosition) {
		ContentResolver resolver = context.getContentResolver();
		
		ContentValues values = new ContentValues();
		values.put(Songs.SONG_LAST_PLAYED_AT, lastPosition);
		
		resolver.update(songUri, values, null, null);
	}
	
	public static int getSongLastPlayedPosition(Context context, Uri songUri) {
		ContentResolver resolver = context.getContentResolver();
		String[] projection = new String[] { Songs.SONG_LAST_PLAYED_AT };
		int position = 0;
		
		Cursor cursor = null;
		try {
			cursor = resolver.query(songUri, projection, null, null, null);
			if (cursor != null && cursor.moveToFirst()) {
				position = cursor.getInt(cursor.getColumnIndex(Songs.SONG_LAST_PLAYED_AT));
			}
		} catch (Exception e) {
			if (cursor != null) cursor.close();
		}
		
		return position;
	}
	
	// Debug method.
	public static long printPlayQueueSongRowIds(Context context) {
		ContentResolver resolver = context.getContentResolver();
		String[] projection = new String[] { Songs._ID }; 
		Cursor cursor = null;
		
		long id = -1;
		try {
			cursor = resolver.query(QUEUE_URI, projection,
					null, null, null);
			if (cursor != null && cursor.isBeforeFirst()) {
				while (cursor.moveToNext()) {
					id = cursor.getLong(cursor.getColumnIndex(Songs._ID));
					Log.i(TAG, "id: " + id);
				}
			}
		} finally {
			if (cursor != null) cursor.close();
		}
		return id;
	}
}
