package com.ubuntuone.android.music.provider.dao;

import java.util.ArrayList;

import android.content.ContentResolver;
import android.content.ContentValues;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteStatement;
import android.net.Uri;

import com.ubuntuone.android.music.provider.MusicContentValues;
import com.ubuntuone.android.music.provider.MusicContract.Albums;
import com.ubuntuone.android.music.provider.MusicContract.ArtistAlbums;
import com.ubuntuone.android.music.provider.MusicContract.Artists;
import com.ubuntuone.android.music.provider.MusicDatabase;
import com.ubuntuone.android.music.provider.MusicDatabase.Tables;
import com.ubuntuone.android.music.provider.MusicProvider;
import com.ubuntuone.api.music.model.U1Artist;

public class ArtistAlbumDao extends Dao
{
	public ArtistAlbumDao() {
	}
	
	/**
	 * Updates the artists albums. If the artists albums don't exist, inserts artists albums.
	 * Backed by {@link MusicProvider}.
	 * 
	 * @param resolver
	 *            The content resolver.
	 * @param artist
	 *            The {@link U1Artist} whose albums to save.
	 */
	public void updateOrInsert(ContentResolver resolver, U1Artist artist) {
		ArrayList<ContentValues> albumsValues = MusicContentValues.forArtistAlbums(artist);
		for (ContentValues albumValues : albumsValues) {
			Uri artistAlbumUri = Artists.buildAlbumUri(
					artist.getId(), albumValues.getAsString(Albums.ALBUM_ID));
			int count = resolver.update(artistAlbumUri, albumValues, null, null);
			if (count == 0) {
				resolver.insert(artistAlbumUri, albumValues);
			}
		}
	}
	
	/**
	 * Updates the artists albums. If the artists albums don't exist, inserts artists albums.
	 * Backed by {@link MusicDatabase}.
	 * 
	 * @param db
	 *            The {@link MusicDatabase} instance.
	 * @param artist
	 *            The {@link U1Artist} whose albums to save.
	 * @return True if update or insert succeeded, false otherwise.
	 */
	public boolean updateOrInsert(SQLiteDatabase db, U1Artist artist) {
		boolean isSuccess = true;
		SQLiteStatement sqlUpdate = db.compileStatement(getUpdateSql());
		SQLiteStatement sqlInsert = db.compileStatement(getInsertSql());
		
		final String artistId = artist.getId();
		final ArrayList<String> albumIds = artist.getAlbumIds();
		for (String albumId : albumIds) {
			int i = 1; // Index to the parameter to bind.
			sqlUpdate.clearBindings();
			sqlUpdate.bindLong(i++, System.currentTimeMillis());
			sqlUpdate.bindString(i++, artistId);
			sqlUpdate.bindString(i++, albumId);
			// Can't use executeUpdateDelete on API < 11
			sqlUpdate.execute();
			long rowsUpdated = getChanges(db);
			if (rowsUpdated == 0L) {
				i = 1;
				sqlInsert.clearBindings();
				sqlInsert.bindLong(i++, System.currentTimeMillis());
				sqlInsert.bindString(i++, artistId);
				sqlInsert.bindString(i++, albumId);
				
				long rowId = sqlInsert.executeInsert();
				isSuccess = isSuccess && (rowId != -1);
			} else {
				isSuccess = isSuccess && (rowsUpdated == 1);
			}
		}
		return isSuccess;
	}

	@Override
	public String onGetInsertSql() {
		return "INSERT INTO " + Tables.ARTISTS_ALBUMS + "(" +
				ArtistAlbums.UPDATED + ", " +
				ArtistAlbums.ARTIST_ID + ", " +
				ArtistAlbums.ALBUM_ID +
				") VALUES (?, ?, ?)";
	}

	@Override
	public String onGetUpdateSql() {
		return "UPDATE " + Tables.ARTISTS_ALBUMS + " SET " + ArtistAlbums.UPDATED + "=? " +
				"WHERE " + ArtistAlbums.ARTIST_ID + "=? AND " + ArtistAlbums.ALBUM_ID + "=?";
	}
}
