package com.ubuntuone.android.music.provider.dao;

import android.content.ContentResolver;
import android.content.ContentValues;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteStatement;
import android.net.Uri;

import com.ubuntuone.android.music.provider.MusicContentValues;
import com.ubuntuone.android.music.provider.MusicContract.Artists;
import com.ubuntuone.android.music.provider.MusicDatabase;
import com.ubuntuone.android.music.provider.MusicDatabase.Tables;
import com.ubuntuone.android.music.provider.MusicProvider;
import com.ubuntuone.android.music.util.StringUtils;
import com.ubuntuone.api.music.model.U1Artist;

public class ArtistDao extends Dao
{
	public ArtistDao() {
	}
	
	/**
	 * Updates the artist. If the artist doesn't exist, inserts the artist.
	 * Backed by {@link MusicProvider}.
	 * 
	 * @param resolver
	 *            The content resolver.
	 * @param artist
	 *            The {@link U1Artist} to save.
	 */
	public void updateOrInsert(ContentResolver resolver, U1Artist artist) {
		ContentValues values = MusicContentValues.forArtist(artist);
		Uri artistUri = Artists.buildArtistUri(artist.getId());
		int count = resolver.update(artistUri, values, null, null);
		if (count == 0) {
			resolver.insert(Artists.CONTENT_URI, values);
		}
	}
	
	/**
	 * Updates the artist. If the artist doesn't exist, inserts the artist.
	 * Backed by {@link MusicDatabase}.
	 * 
	 * @param db
	 *            The {@link MusicDatabase} instance.
	 * @param artist
	 *            The {@link U1Artist} to save.
	 * @return True if update or insert succeeded, false otherwise.
	 */
	public boolean updateOrInsert(SQLiteDatabase db, U1Artist artist) {
		boolean isSuccess = false;
		
		SQLiteStatement sqlStmt = db.compileStatement(getUpdateSql());
		int i = 1; // Index to the parameter to bind.
		
		final String albumList = StringUtils.join(artist.getAlbumIds(), ",");
		final long albumListSize = artist.getAlbumIds().size();
		
		sqlStmt.bindLong(i++, System.currentTimeMillis());
		
		sqlStmt.bindString(i++, artist.getArtist());
		sqlStmt.bindString(i++, albumList);
		sqlStmt.bindLong(i++, albumListSize);
		sqlStmt.bindLong(i++, artist.getSongCount());
		
		sqlStmt.bindString(i++, artist.getArtistUrl());
		sqlStmt.bindString(i++, artist.getArtistArtUrl());
		
		sqlStmt.bindString(i++, artist.getId());
		// Can't use executeUpdateDelete on API < 11
		sqlStmt.execute();
		long rowsUpdated = getChanges(db);
		
		if (rowsUpdated == 0L) {
			sqlStmt = db.compileStatement(getInsertSql());
			i = 1;
			sqlStmt.bindLong(i++, System.currentTimeMillis());
			sqlStmt.bindString(i++, artist.getId());
			
			sqlStmt.bindString(i++, artist.getArtist());
			sqlStmt.bindString(i++, albumList);
			sqlStmt.bindLong(i++, albumListSize);
			sqlStmt.bindLong(i++, artist.getSongCount());
			
			sqlStmt.bindString(i++, artist.getArtistUrl());
			sqlStmt.bindString(i++, artist.getArtistArtUrl());
			
			long rowId = sqlStmt.executeInsert();
			isSuccess = rowId != -1;
		} else {
			isSuccess = rowsUpdated == 1;
		}
		return isSuccess;
	}

	@Override
	public String onGetInsertSql() {
		return "INSERT INTO " + Tables.ARTISTS + "(" +
				Artists.UPDATED + ", " +
				Artists.ARTIST_ID + ", " +
				
				Artists.ARTIST_NAME + ", " +
				Artists.ALBUM_LIST + ", " +
				Artists.ALBUM_COUNT + ", " +
				Artists.SONG_COUNT + ", " +
				
				Artists.ARTIST_URL + ", " +
				Artists.ARTIST_ART_URL +
				
				") VALUES (?, ?, " +
				"?, ?, ?, ?, " +
				"?, ?" +
				")";
	}

	@Override
	public String onGetUpdateSql() {
		return "UPDATE " + Tables.ARTISTS + " SET " +
				Artists.UPDATED + "=?, " +
				
				Artists.ARTIST_NAME + "=?, " +
				Artists.ALBUM_LIST + "=?, " +
				Artists.ALBUM_COUNT + "=?, " +
				Artists.SONG_COUNT + "=?, " +
				
				Artists.ARTIST_URL + "=?, " +
				Artists.ARTIST_ART_URL + "=? " +
				
				"WHERE " + Artists.ARTIST_ID + "=?";
	}
}
