package com.ubuntuone.android.music.provider.dao;

import static com.ubuntuone.android.music.provider.MusicContentValues.getGenreId;
import android.content.ContentResolver;
import android.content.ContentValues;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteStatement;
import android.net.Uri;

import com.ubuntuone.android.music.provider.MusicContentValues;
import com.ubuntuone.android.music.provider.MusicContract.Songs;
import com.ubuntuone.android.music.provider.MusicDatabase;
import com.ubuntuone.android.music.provider.MusicDatabase.Tables;
import com.ubuntuone.android.music.provider.MusicProvider;
import com.ubuntuone.api.music.model.U1Song;

public class SongDao extends Dao
{
	public SongDao() {
	}
	
	/**
	 * Updates the song. If the song doesn't exist, inserts the song. Backed by
	 * {@link MusicProvider}.
	 * 
	 * @param resolver
	 *            The content resolver.
	 * @param song
	 *            The {@link U1Song} to save.
	 */
	public void updateOrInsert(ContentResolver resolver, U1Song song) {
		ContentValues values = MusicContentValues.forSong(song);
		Uri songUri = Songs.buildSongUri(song.getId());
		int rowsUpdated = resolver.update(songUri, values, null, null);
		if (rowsUpdated == 0) {
			resolver.insert(Songs.CONTENT_URI, values);
		}
	}
	
	/**
	 * Updates the song. If the song doesn't exist, inserts the song. Backed by
	 * {@link MusicDatabase}.
	 * 
	 * @param db
	 *            The {@link MusicDatabase} instance.
	 * @param song
	 *            The {@link U1Song} to save.
	 * @return True if update or insert succeeded, false otherwise.
	 */
	public boolean updateOrInsert(SQLiteDatabase db, U1Song song) {
		boolean isSuccess = false;
		
		SQLiteStatement sqlStmt = db.compileStatement(getUpdateSql());
		int i = 1; // Index to the parameter to bind.
		sqlStmt.bindLong(i++, System.currentTimeMillis());
		
		sqlStmt.bindString(i++, song.getTitle());
		sqlStmt.bindString(i++, song.getAlbum());
		sqlStmt.bindString(i++, song.getAlbumId());
		sqlStmt.bindString(i++, song.getAlbumArtist());
		
		sqlStmt.bindString(i++, song.getArtist());
		sqlStmt.bindString(i++, song.getArtistId());
		sqlStmt.bindString(i++, song.getGenre());
		sqlStmt.bindString(i++, getGenreId(song.getGenre()));
		
		sqlStmt.bindLong(i++, song.getYear());
		sqlStmt.bindLong(i++, song.getDiscNumber());
		sqlStmt.bindLong(i++, song.getTrack());
		sqlStmt.bindLong(i++, song.getDuration());
		
		sqlStmt.bindLong(i++, song.getSize());
		sqlStmt.bindLong(i++, song.getBitRate());
		sqlStmt.bindString(i++, song.getSuffix());
		sqlStmt.bindString(i++, song.getContentType());
		
		sqlStmt.bindString(i++, song.getSongUrl());
		sqlStmt.bindString(i++, song.getSongArtUrl());
		sqlStmt.bindString(i++, song.getSongStreamUrl());
		
		sqlStmt.bindString(i++, song.getId());
		// Can't use executeUpdateDelete on API < 11
		sqlStmt.execute();
		long rowsUpdated = getChanges(db);
		
		if (rowsUpdated == 0L) {
			sqlStmt = db.compileStatement(getInsertSql());
			i = 1;
			sqlStmt.bindLong(i++, System.currentTimeMillis());
			sqlStmt.bindString(i++, song.getId());
			
			sqlStmt.bindString(i++, song.getTitle());
			sqlStmt.bindString(i++, song.getAlbum());
			sqlStmt.bindString(i++, song.getAlbumId());
			sqlStmt.bindString(i++, song.getAlbumArtist());
			
			sqlStmt.bindString(i++, song.getArtist());
			sqlStmt.bindString(i++, song.getArtistId());
			sqlStmt.bindString(i++, song.getGenre());
			sqlStmt.bindString(i++, getGenreId(song.getGenre()));
			
			sqlStmt.bindLong(i++, song.getYear());
			sqlStmt.bindLong(i++, song.getDiscNumber());
			sqlStmt.bindLong(i++, song.getTrack());
			sqlStmt.bindLong(i++, song.getDuration());
			
			sqlStmt.bindLong(i++, song.getSize());
			sqlStmt.bindLong(i++, song.getBitRate());
			sqlStmt.bindString(i++, song.getSuffix());
			sqlStmt.bindString(i++, song.getContentType());
			
			sqlStmt.bindString(i++, song.getSongUrl());
			sqlStmt.bindString(i++, song.getSongArtUrl());
			sqlStmt.bindString(i++, song.getSongStreamUrl());
			
			long rowId = sqlStmt.executeInsert();
			isSuccess = rowId != -1;
		} else {
			isSuccess = rowsUpdated == 1;
		}
		return isSuccess;
	}

	@Override
	public String onGetInsertSql() {
		return "INSERT INTO " + Tables.SONGS + "(" +
				Songs.UPDATED + ", " +
				Songs.SONG_ID + ", " +
				
				Songs.SONG_TITLE + ", " +
				Songs.SONG_ALBUM + ", " +
				Songs.SONG_ALBUM_ID + ", " +
				Songs.SONG_ALBUM_ARTIST + ", " +
				
				Songs.SONG_ARTIST + ", " +
				Songs.SONG_ARTIST_ID + ", " +
				Songs.SONG_GENRE + ", " +
				Songs.SONG_GENRE_ID + ", " +
				
				Songs.SONG_YEAR + ", " +
				Songs.SONG_DISC_NUMBER + ", " +
				Songs.SONG_TRACK + ", " +
				Songs.SONG_DURATION + ", " +
				
				Songs.SONG_SIZE + ", " +
				Songs.SONG_BIT_RATE + ", " +
				Songs.SONG_SUFFIX + ", " +
				Songs.SONG_CONTENT_TYPE + ", " +
				
				Songs.SONG_URL + ", " +
				Songs.SONG_ART_URL + ", " +
				Songs.SONG_STREAM_URL +
				
				") VALUES (?, ?, " +
				"?, ?, ?, ?, " +
				"?, ?, ?, ?, " +
				"?, ?, ?, ?, " +
				"?, ?, ?, ?, " +
				"?, ?, ?" +
				")";
	}

	@Override
	public String onGetUpdateSql() {
		return "UPDATE " + Tables.SONGS + " SET " +
				Songs.UPDATED + "=?, " +
				
				Songs.SONG_TITLE + "=?, " +
				Songs.SONG_ALBUM + "=?, " +
				Songs.SONG_ALBUM_ID + "=?, " +
				Songs.SONG_ALBUM_ARTIST + "=?, " +
				
				Songs.SONG_ARTIST + "=?, " +
				Songs.SONG_ARTIST_ID + "=?, " +
				Songs.SONG_GENRE + "=?, " +
				Songs.SONG_GENRE_ID + "=?, " +
				
				Songs.SONG_YEAR + "=?, " +
				Songs.SONG_DISC_NUMBER + "=?, " +
				Songs.SONG_TRACK + "=?, " +
				Songs.SONG_DURATION + "=?, " +
				
				Songs.SONG_SIZE + "=?, " +
				Songs.SONG_BIT_RATE + "=?, " +
				Songs.SONG_SUFFIX + "=?, " +
				Songs.SONG_CONTENT_TYPE + "=?, " +
				
				Songs.SONG_URL + "=?, " +
				Songs.SONG_ART_URL + "=?, " +
				Songs.SONG_STREAM_URL + "=? " +
				
				"WHERE " + Songs.SONG_ID + "=?";
	}
}
