/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.service;

import static com.ubuntuone.android.music.model.PlayerState.COMPLETED;
import static com.ubuntuone.android.music.model.PlayerState.DOWNLOADING;
import static com.ubuntuone.android.music.model.PlayerState.IDLE;
import static com.ubuntuone.android.music.model.PlayerState.PAUSED;
import static com.ubuntuone.android.music.model.PlayerState.PREPARED;
import static com.ubuntuone.android.music.model.PlayerState.PREPARING;
import static com.ubuntuone.android.music.model.PlayerState.STARTED;
import static com.ubuntuone.android.music.model.PlayerState.STOPPED;
import static com.ubuntuone.android.music.util.LogUtils.makeLogTag;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.LinkedList;

import android.annotation.TargetApi;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.database.sqlite.SQLiteDatabaseLockedException;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.media.MediaPlayer.OnErrorListener;
import android.net.Uri;
import android.net.wifi.WifiManager;
import android.os.Build.VERSION;
import android.os.Build.VERSION_CODES;
import android.os.Handler;
import android.os.IBinder;
import android.os.PowerManager;
import android.preference.PreferenceManager;
import android.util.Log;

import com.ubuntuone.android.music.UbuntuOneMusic;
import com.ubuntuone.android.music.model.PlayerState;
import com.ubuntuone.android.music.model.Song;
import com.ubuntuone.android.music.provider.MusicContract.Songs;
import com.ubuntuone.android.music.provider.MusicProviderUtils;
import com.ubuntuone.android.music.util.NotAuthenticatedException;
import com.ubuntuone.android.music.util.UIUtils;
import com.ubuntuone.api.music.U1MusicAPI;

@TargetApi(16)
public class MusicService extends Service implements OnErrorListener
{
	private static final String TAG = makeLogTag(MusicService.class);
	
	private static final String SHUFFLE_PREFERENCE_KEY = "shuffle_play";
	private static final String REPEAT_PREFERENCE_KEY = "repeat_play";
	
	private final IBinder mBinder = new MusicServiceBinder<MusicService>(this);
	private final LinkedList<MusicServiceCallback> mCallbacks =
			new LinkedList<MusicServiceCallback>();
	private final Handler mHandler = new Handler();
	
	private static MusicService sInstance;
	
	private MusicServiceSupport mSupport = new MusicServiceSupport(this);

	private PowerManager.WakeLock mWakeLock;
	private WifiManager.WifiLock mWifiLock;
	
	private U1MusicAPI mApi;
	
	private Song mCurrentDownloading = null;
	private Song mCurrentPlaying = null;
	private MediaPlayer mMediaPlayer;
	private PlayerState mPlayerState;
	
	private boolean mShufflePlay;
	private boolean mRepeatPlay;
	
	@Override
	public void onCreate() {
		super.onCreate();
		
		sInstance = this;
		
		try {
			mApi = UbuntuOneMusic.getInstance().getMusicApi();
		} catch (NotAuthenticatedException e1) {
			Log.w(TAG, "Not authenticated.");
			stopSelf();
			return;
		}
		
		mSupport.onCreate();
		
		PowerManager pm = (PowerManager) getSystemService(POWER_SERVICE);
		mWakeLock = pm.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, MusicService.class.getName());
		mWakeLock.setReferenceCounted(false);
		
		int lockType = 0;
		if (VERSION.SDK_INT >= VERSION_CODES.HONEYCOMB_MR1) {
			lockType = WifiManager.WIFI_MODE_FULL_HIGH_PERF;
		} else {
			lockType = WifiManager.WIFI_MODE_FULL;
		}
		
		WifiManager wm = (WifiManager) getSystemService(WIFI_SERVICE);
		mWifiLock = wm.createWifiLock(lockType, MusicService.class.getName());
		mWifiLock.setReferenceCounted(false);
		
		this.mMediaPlayer = new MediaPlayer();
		this.mMediaPlayer.setWakeMode(this, PowerManager.PARTIAL_WAKE_LOCK);
		this.mMediaPlayer.setOnErrorListener(this);
		this.mPlayerState = IDLE;
		
		SharedPreferences sharedPrefs = PreferenceManager.getDefaultSharedPreferences(this);
		this.mShufflePlay = sharedPrefs.getBoolean(SHUFFLE_PREFERENCE_KEY, false);
		this.mRepeatPlay = sharedPrefs.getBoolean(REPEAT_PREFERENCE_KEY, false);
		
		try {
			Uri songUri = MusicProviderUtils.getPlayingSong(this);
			if (songUri != null) {
				mCurrentPlaying = new Song(this, songUri);
			}
		} catch (SQLiteDatabaseLockedException e) {
			// Ignore. Assume no song has been playing.
		}
	}
	
	public static MusicService getInstance() {
		return sInstance;
	}
	
	@Override
	public int onStartCommand(Intent intent, int flags, int startId) {
		int result = super.onStartCommand(intent, flags, startId);
		mSupport.onStartCommand(intent);
		mWifiLock.acquire();
		return result;
	}
	
	public void register(MusicServiceCallback callback) {
		if (!mCallbacks.contains(callback)) {
			mCallbacks.add(callback);
		}
		Uri songUri = mCurrentPlaying != null ? mCurrentPlaying.getSongUri() : null;
		Log.i(TAG, "Registering MusicServiceCallback. Current song is: " + songUri);
		// Call back with current song.
		callback.onSongChanged(songUri);
		// Optionally call back with song download progress.
		if (mCurrentPlaying != null && mCurrentPlaying.isCompleteFileAvailable()) {
			callback.onSongDownload(songUri, 100);
		}
		// Call back with player state.
		callback.onPlayerStateChanged(mPlayerState);
	}
	
	public void unregister(MusicServiceCallback callback) {
		mCallbacks.remove(callback);
	}
	
	@Override
	public void onDestroy() {
		SharedPreferences sharedPrefs = PreferenceManager.getDefaultSharedPreferences(this);
		sharedPrefs.edit()
				.putBoolean(SHUFFLE_PREFERENCE_KEY, mShufflePlay)
				.putBoolean(REPEAT_PREFERENCE_KEY, mRepeatPlay)
				.commit();
		
		if (mMediaPlayer != null) {
			mMediaPlayer.release();
			mMediaPlayer = null;
		}
		
		mSupport.onDestroy();
		
		if (mWifiLock != null && mWifiLock.isHeld()) {
			mWifiLock.release();
		}
		
		if (mWakeLock != null && mWakeLock.isHeld()) {
			mWakeLock.release();
		}
		
		super.onDestroy();
	}

	@Override
	public IBinder onBind(Intent intent) {
		return mBinder;
	}
	
	public boolean isShufflePlayEnabled() {
		return mShufflePlay;
	}
	
	public synchronized void setShufflePlay(boolean shufflePlay) {
		// Randomize play queue sort column values?
		Log.i(TAG, "Shuffle play " + (shufflePlay ? "enabled" : "disabled"));
		this.mShufflePlay = shufflePlay;
	}
	
	public boolean isRepeatPlayEnabled() {
		return mRepeatPlay;
	}
	
	public synchronized void setRepeatPlay(boolean repeatPlay) {
		Log.i(TAG, "Repeat play " + (repeatPlay ? "enabled" : "disabled"));
		this.mRepeatPlay = repeatPlay;
	}
	
	public PlayerState getPlayerState() {
		return mPlayerState;
	}
	
	private synchronized void setPlayerState(PlayerState state) {
		// TODO show notification based on state change.
		if (state != PAUSED && state != STOPPED) {
			mHandler.post(new OnProgressCallbackRunnable());
		}
		this.mPlayerState = state;
		callbackOnPlayerStateChanged(state);
	}
	
	public Song getCurrentPlaying() {
		return mCurrentPlaying;
	}
	
	public Song getCurrentDownloading() {
		return mCurrentDownloading;
	}
	
	private synchronized void callbackOnSongChanged() {
		int i = mCallbacks.size();
		for (i--; i >= 0; i--) {
			MusicServiceCallback callback = mCallbacks.get(i);
			try {
				callback.onSongChanged(mCurrentPlaying.getSongUri());
			} catch (Exception e) {
				Log.e(TAG, "Exception thrown while calling back.", e);
				mCallbacks.remove(callback);
			}
		}
	}
	
	private synchronized void callbackOnPlayerStateChanged(PlayerState state) {
		int i = mCallbacks.size();
		for (i--; i >= 0; i--) {
			MusicServiceCallback callback = mCallbacks.get(i);
			try {
				callback.onPlayerStateChanged(state);
			} catch (Exception e) {
				Log.e(TAG, "Exception thrown while calling back.", e);
				mCallbacks.remove(callback);
			}
		}
	}
	
	boolean switchLocated = false;
	long length = 0;
	long offset = 0;
	MediaPlayer secondMediaPlayer = new MediaPlayer();
	
	public synchronized void callbackOnSongDownload(Uri songUri, int percent) {		
		int i = mCallbacks.size();
		for (i--; i >= 0; i--) {
			MusicServiceCallback callback = mCallbacks.get(i);
			try {
				callback.onSongDownload(songUri, percent);
			} catch (Exception e) {
				Log.e(TAG, "Exception thrown while calling back.", e);
				mCallbacks.remove(callback);
			}
		}
	}
	
	public synchronized void callbackOnSongProgress() {
		final Uri songUri = mCurrentPlaying.getSongUri();
		final int position = mMediaPlayer.getCurrentPosition();
		final int total = mCurrentPlaying.getDuration() * 1000;
		if (total == 0) return;
		
		int percent = (int) (100 * position / (double) total);
		
		int i = mCallbacks.size();
		for (i--; i >= 0; i--) {
			MusicServiceCallback callback = mCallbacks.get(i);
			try {
				callback.onSongProgress(songUri, percent);
			} catch (Exception e) {
				Log.e(TAG, "Exception thrown while calling back.", e);
				mCallbacks.remove(callback);
			}
		}
	}
	
	public int getPlayerPosition() {
		try {
			if (mPlayerState == IDLE
					|| mPlayerState == DOWNLOADING
					|| mPlayerState == PlayerState.PREPARING) {
				return 0;
			}
			return mMediaPlayer.getCurrentPosition();
		} catch (Exception e) {
			handleError(e);
			return 0;
		}
	}
	
	public int getPlayerDuration() {
		if (mCurrentPlaying != null) {
			return mCurrentPlaying.getDuration() * 1000;
		}
		if (mPlayerState != IDLE
				&& mPlayerState != DOWNLOADING
				&& mPlayerState != PlayerState.PREPARING) {
			try {
				return mMediaPlayer.getDuration();
			} catch (Exception e) {
				handleError(e);
			}
		}
		return 0;
	}
	
	private synchronized void setCurrentPlaying(Uri uri) {
		// Make sure this is a song Uri, rather than playlist song Uri.
		// This has to be changed when "now playing" column is moved to the PlaylistSongs table.
		String songId = Songs.getSongId(uri);
		Uri songUri = Songs.buildSongUri(songId);
		
		MusicProviderUtils.setPlayingSong(this, songUri);
		this.mCurrentPlaying = new Song(this, songUri);
		callbackOnSongChanged();
		if (mCurrentPlaying.isCompleteFileAvailable()) {
			callbackOnSongDownload(uri, 100);
		}
	}
	
	public synchronized void reset() {
		Log.d(TAG, "reset");
		try {
			mMediaPlayer.reset();
			setPlayerState(IDLE);
		} catch (Exception e) {
			handleError(e);
		}
	}
	
	public synchronized void play(Uri songUri) {
		play(songUri, 0);
	}

	public synchronized void play(Uri songUri, int position) {
		Log.d(TAG, "play");
		boolean isSameSong = songUri != null && mCurrentPlaying != null
				&& songUri.equals(mCurrentPlaying.getSongUri());
		if (!isSameSong) {
			setCurrentPlaying(songUri);
		}
		// Always update now playing flag.
		MusicProviderUtils.setPlayingSong(this, songUri);
		
		if (mPlayerState == PAUSED && isSameSong) {
			resume();
		} else {
			checkDownloads();
			if (mCurrentPlaying.isCompleteFileAvailable()) {
				doPlay(mCurrentPlaying, position);
			}
		}
	}
	
	public synchronized void pause() {
		try {
			int position = mMediaPlayer.getCurrentPosition();
			
			mMediaPlayer.pause();
			setPlayerState(PAUSED);
			
			MusicProviderUtils.setSongLastPlayedPosition(
					this, mCurrentPlaying.getSongUri(), position);
		} catch (Exception e) {
			reset();
			handleError(e);
		}
	}
	
	public synchronized void resume() {
		try {
			if (UbuntuOneMusic.SHOULD_PLAY) {
				mMediaPlayer.start();
			}
			setPlayerState(STARTED);
		} catch (Exception e) {
			reset();
			handleError(e);
		}
	}
	
	public synchronized void stop() {
		try {
			mMediaPlayer.stop();
			setPlayerState(IDLE);
		} catch (Exception e) {
			handleError(e);
		}
	}
	
	public synchronized void seek(int msec) {
		if (mMediaPlayer.isPlaying()) {
			try {
				mMediaPlayer.seekTo(msec);
			} catch (Exception e) {
				handleError(e);
			}
		}
	}
	
	public synchronized void prev(boolean isUser) {
		Log.d(TAG, "prev");
		boolean wasPlaying = mPlayerState == STARTED || mPlayerState == COMPLETED;
		if (mMediaPlayer.isPlaying()) stop();
		
		Uri songUri = MusicProviderUtils.getPlayingSong(this);
		Uri prevSongUri = null;
		
		if (songUri == null) {
			// No playing song, start from the first.
			prevSongUri = MusicProviderUtils.getFirstPlayQueueSong(this, mShufflePlay);
		} else {
			// Already playing, jump to the previous.
			prevSongUri = MusicProviderUtils.getPrevPlayQueueSong(this, songUri, mShufflePlay);
			if (prevSongUri == null) {
				// Start of play queue has been reached.
				if (mRepeatPlay || isUser) {
					if (mShufflePlay) MusicProviderUtils.resetSongWasPlayedFlags(this);
					// Jump to the end.
					prevSongUri = MusicProviderUtils.getLastPlayQueueSong(this, mShufflePlay);
				}
			}
		}
		if (prevSongUri != null) {
			if (wasPlaying) {
				play(prevSongUri);
			} else {
				setCurrentPlaying(prevSongUri);
			}
		}
	}
	
	public synchronized void next(boolean isUser) {
		Log.d(TAG, "next");
		boolean wasPlaying = mPlayerState == STARTED || mPlayerState == COMPLETED;
		if (mMediaPlayer.isPlaying()) stop();
		
		Uri songUri = MusicProviderUtils.getPlayingSong(this);
		Uri nextSongUri = null;
		
		if (songUri == null) {
			// No playing song, start from the first.
			nextSongUri = MusicProviderUtils.getFirstPlayQueueSong(this, mShufflePlay);
		} else {
			MusicProviderUtils.setSongLastPlayedPosition(this, songUri, 0);
			
			// Already playing, start with the next.
			nextSongUri = MusicProviderUtils.getNextPlayQueueSong(this, songUri, mShufflePlay);
			if (nextSongUri == null) {
				// End of play queue has been reached.
				if (mRepeatPlay || isUser) {
					if (mShufflePlay) MusicProviderUtils.resetSongWasPlayedFlags(this);
					// Start from the beginning.
					nextSongUri = MusicProviderUtils.getFirstPlayQueueSong(this, mShufflePlay);
				}
			}
		}
		if (nextSongUri != null) {
			if (wasPlaying) {
				play(nextSongUri);
			} else {
				setCurrentPlaying(nextSongUri);
			}
		}
	}
	
	private void handleError(Exception e) {
		if (mCurrentPlaying != null) {
			// TODO Show error notification.
		}
		mMediaPlayer.reset();
		setPlayerState(IDLE);
	}
	
	@Override
	public boolean onError(MediaPlayer mp, int what, int extra) {
		// TODO add analytics
		switch (what) {
		case MediaPlayer.MEDIA_ERROR_NOT_VALID_FOR_PROGRESSIVE_PLAYBACK:
			
			break;
		case MediaPlayer.MEDIA_INFO_BAD_INTERLEAVING:
			
			break;
		case MediaPlayer.MEDIA_ERROR_UNKNOWN:
			// Taken from Subsonic source:
			if ((extra == 31) || (extra == -1004)) { // observed
				// File is corrupt.
			} else {
				// Unknown error.
			}
			break;
		case MediaPlayer.MEDIA_INFO_METADATA_UPDATE:
			
			return true;
		default:
			// TODO Handle this. Add analytics?
			break;
		}
		reset();
		next(false);
		return false;
	}
	
	public synchronized void doPlay(final Song song, int position) {
		try {
			reset();
			final File file = song.isCompleteFileAvailable()
					? song.getCompleteFile() : song.getPartialFile();
			if (song.isCompleteFileAvailable()) {
				// In case the song's been already on the device.
				song.saveAsComplete(this);
			}
			
			mMediaPlayer.setOnCompletionListener(null);
			mMediaPlayer.reset();
			setPlayerState(IDLE);
			
			mMediaPlayer.setAudioStreamType(AudioManager.STREAM_MUSIC);
			if ("m4a".equals(song.getSuffix())) {
				mMediaPlayer.setDataSource(file.getAbsolutePath());
			} else {
				FileInputStream stream = new FileInputStream(file.getAbsolutePath());
				try {
					mMediaPlayer.setDataSource(stream.getFD());
				} catch (IOException e) {
					MusicService.this.reset();
					MusicService.this.next(false);
					// TODO Count in analytics?
					throw new Exception("File may be corrupt.");
				} finally {
					stream.close();
				}
			}
			
			setPlayerState(PREPARING);
			try {
				mMediaPlayer.prepare();
			} catch (Exception e) {
				MusicService.this.reset();
				MusicService.this.next(false);
				// TODO Count in analytics?
				throw new Exception("File may be corrupt.");
			}
			setPlayerState(PREPARED);
			
			mMediaPlayer.setOnCompletionListener(new MediaPlayer.OnCompletionListener() {
				@Override
				public void onCompletion(MediaPlayer mp) {
					Log.d(TAG, "mediaPlayer complete; end has been reached during playback.");
					
					// MediaPlayer releases its wake lock, so acquire.
					mWakeLock.acquire(30 * 1000);
					
					setPlayerState(COMPLETED);
					
					if (!file.equals(song.getPartialFile())) {
						// We are done playing that song for real.
						next(false);
						return;
					}
					
					// We reached end of partially downloaded song.
					int position = mMediaPlayer.getCurrentPosition();
					// TODO Track in analytics?
					if (position < 0) {
						return;
					}
					
					synchronized (MusicService.this) {
						mMediaPlayer.reset();
						// Buffer and play from position.
						doPlay(song, position);
					}
				}
			});
			
			if (position > 0) {
				mMediaPlayer.seekTo(position);
			}
			
			if (UbuntuOneMusic.SHOULD_PLAY) {
				mMediaPlayer.start();
			}
			setPlayerState(STARTED);
			
			MusicProviderUtils.setSongWasPlayedFlag(this, song.getSongUri());
		} catch (Exception e) {
			handleError(e);
		}
	}
	
	protected synchronized void checkDownloads() {
		Log.d(TAG, "checkDownloads");
		
		if (!UIUtils.isNetworkConnected(this) || !UIUtils.isExternalStoragePresent()) {
			Log.w(TAG, "No network or external storage.");
			return;
		}
		
		if (MusicProviderUtils.getPlayQueueSongCount(this) == 0) {
			Log.w(TAG, "Play queue empty.");
			return;
		}
		
		// Download currently playing?
		if (mCurrentPlaying != null
				&& mCurrentPlaying != mCurrentDownloading
				&& !mCurrentPlaying.isCompleteFileAvailable()) {
			
			Log.d(TAG, "Downloading currently playing song.");
			
			if (mCurrentDownloading != null) {
				mCurrentDownloading.cancelDownload();
			}
			
			mCurrentDownloading = mCurrentPlaying;
			if (mApi != null) {
				mCurrentDownloading.download(this, mHandler, 0, mApi, true);
			}
		} else if (mCurrentPlaying != null && mCurrentPlaying.isCompleteFileAvailable()) {
			callbackOnSongDownload(mCurrentPlaying.getSongUri(), 100);
		}
		
		// No? Maybe download something else?
		if (mCurrentDownloading == null || !mCurrentDownloading.isCompleteFileAvailable()) {
			Log.d(TAG, "Looking for download candidates.");
			
			// TODO prefetch play queue songs
		}
		
		// Cleanup.
	}
	
	private class OnProgressCallbackRunnable implements Runnable
	{
		@Override
		public void run() {
			if (mMediaPlayer.isPlaying()) {
				callbackOnSongProgress();
				mHandler.postDelayed(this, 1000);
			}
		}
	}
	
	public static void startFrom(Context context) {
		Intent intent = new Intent(context, MusicService.class);
		context.startService(intent);
	}
}
