/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.service;

import static android.content.Context.AUDIO_SERVICE;
import static android.content.Context.TELEPHONY_SERVICE;
import static android.telephony.TelephonyManager.CALL_STATE_IDLE;
import static android.telephony.TelephonyManager.CALL_STATE_OFFHOOK;
import static android.telephony.TelephonyManager.CALL_STATE_RINGING;
import static com.ubuntuone.android.music.model.PlayerState.COMPLETED;
import static com.ubuntuone.android.music.model.PlayerState.IDLE;
import static com.ubuntuone.android.music.model.PlayerState.PAUSED;
import static com.ubuntuone.android.music.model.PlayerState.STARTED;
import static com.ubuntuone.android.music.model.PlayerState.STOPPED;
import static com.ubuntuone.android.music.util.LogUtils.makeLogTag;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.media.AudioManager;
import android.net.Uri;
import android.telephony.PhoneStateListener;
import android.telephony.TelephonyManager;
import android.util.Log;
import android.view.KeyEvent;

import com.ubuntuone.android.music.model.PlayerState;
import com.ubuntuone.android.music.provider.MusicProviderUtils;
import com.ubuntuone.android.music.util.UIUtils;

public class MusicServiceSupport
{
	private static final String TAG = makeLogTag(MusicServiceSupport.class);
	
	private static final int HEADSET_UNPLUGGED = 0;
	
	private MusicService mMusicService;
	private BroadcastReceiver mHeadsetBroadcastReceiver;
	private PhoneStateListener mPhoneStateListener;
	
	public MusicServiceSupport(MusicService service) {
		this.mMusicService = service;
		this.mHeadsetBroadcastReceiver = new U1HeadsetBroadcastReceiver();
		this.mPhoneStateListener = new U1PhoneStateListener();
	}
	
	public void onCreate() {
		registerHeadsetBroadcastReceiver();
		registerMediaButtonBroadcastReceiver();
		registerPhoneStateListener();
	}

	public void onStartCommand(Intent intent) {
		if (intent != null && intent.hasExtra(Intent.EXTRA_KEY_EVENT)) {
			KeyEvent event = (KeyEvent) intent.getParcelableExtra(Intent.EXTRA_KEY_EVENT);
			if (event != null) {
				handleKeyEvent(event);
			}
		}
	}
	
	public void onDestroy() {
		unregisterHeadsetBroadcastReceiver();
		unregisterMediaButtonBroadcastReceiver();
		unregisterPhoneStateListener();
	}

	private void registerHeadsetBroadcastReceiver() {
		mMusicService.registerReceiver(mHeadsetBroadcastReceiver,
				new IntentFilter(Intent.ACTION_HEADSET_PLUG));
	}
	
	private void unregisterHeadsetBroadcastReceiver() {
		mMusicService.unregisterReceiver(mHeadsetBroadcastReceiver);
	}
	
	private void registerMediaButtonBroadcastReceiver() {
		UIUtils.registerMediaButtonEventReceiver(mMusicService);
	}
	
	private void unregisterMediaButtonBroadcastReceiver() {
		UIUtils.unregisterMediaButtonEventReceiver(mMusicService);
	}
	
	private void registerPhoneStateListener() {
		TelephonyManager tm = (TelephonyManager) mMusicService.getSystemService(TELEPHONY_SERVICE);
		tm.listen(mPhoneStateListener, PhoneStateListener.LISTEN_CALL_STATE);
	}
	
	private void unregisterPhoneStateListener() {
		TelephonyManager tm = (TelephonyManager) mMusicService.getSystemService(TELEPHONY_SERVICE);
		tm.listen(mPhoneStateListener, PhoneStateListener.LISTEN_NONE);
	}
	
	private void handleKeyEvent(KeyEvent event) {
		if (event.getAction() != KeyEvent.ACTION_DOWN || event.getRepeatCount() > 0) return;
		
		PlayerState state = mMusicService.getPlayerState();
		switch (event.getKeyCode()) {
		case KeyEvent.KEYCODE_MEDIA_PLAY_PAUSE:
			//$FALL-THROUGH$
		case KeyEvent.KEYCODE_HEADSETHOOK:
			if (state == PAUSED || state == COMPLETED) {
				mMusicService.resume();
			} else if (state == STOPPED || state == IDLE) {
				Uri songUri = MusicProviderUtils.getPlayingSong(mMusicService);
				if (songUri == null) {
					songUri = MusicProviderUtils.getFirstPlayQueueSong(
							mMusicService, mMusicService.isShufflePlayEnabled());
				}
				mMusicService.play(songUri);
			} else if (state == STARTED) {
				mMusicService.pause();
			}
			break;
		case KeyEvent.KEYCODE_MEDIA_PREVIOUS:
			mMusicService.prev(true);
			break;
		case KeyEvent.KEYCODE_MEDIA_NEXT:
			mMusicService.next(true);
			break;
		case KeyEvent.KEYCODE_MEDIA_STOP:
			mMusicService.reset();
			break;

		default:
			// TODO Log unhandled event.
			break;
		}
	}
	
	private class U1HeadsetBroadcastReceiver extends BroadcastReceiver
	{
		@Override
		public void onReceive(Context context, Intent intent) {
			if (intent.getAction().equals(Intent.ACTION_HEADSET_PLUG)) {
				String name = intent.getStringExtra("name");
				int state = intent.getIntExtra("state", 0);
				
				Log.i(TAG, String.format("Headset broadcast: name %s, state %d", name, state));
				boolean isPlaying = mMusicService.getPlayerState() == STARTED;
				if (state == HEADSET_UNPLUGGED && isPlaying) {
					mMusicService.pause();
				}
			}
		}
	}
	
	private class U1PhoneStateListener extends PhoneStateListener
	{
		private boolean resumeAfterCall;

		@Override
		public void onCallStateChanged(int state, String incomingNumber) {
			switch (state) {
			case CALL_STATE_RINGING:
				AudioManager am = (AudioManager) mMusicService.getSystemService(AUDIO_SERVICE);
				
				int ringerVolume = am.getStreamVolume(AudioManager.STREAM_RING);
				if (ringerVolume <= 0) break;
				//$FALL-THROUGH$
			case CALL_STATE_OFFHOOK:
				if (mMusicService.getPlayerState() == STARTED) {
					resumeAfterCall = true;
					mMusicService.pause();
				}
				break;
			case CALL_STATE_IDLE:
				if (resumeAfterCall) {
					resumeAfterCall = false;
					mMusicService.resume();
				}
				break;
			}
		}
	}
}
