/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.service;

import static com.ubuntuone.android.music.util.LogUtils.makeLogTag;

import java.util.ArrayList;

import android.app.IntentService;
import android.content.ContentResolver;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.net.Uri;
import android.os.Bundle;
import android.os.ResultReceiver;
import android.util.Log;

import com.ubuntuone.android.music.R;
import com.ubuntuone.android.music.UbuntuOneMusic;
import com.ubuntuone.android.music.provider.MusicContract.Albums;
import com.ubuntuone.android.music.provider.MusicContract.Artists;
import com.ubuntuone.android.music.provider.MusicContract.Playlists;
import com.ubuntuone.android.music.provider.MusicContract.Songs;
import com.ubuntuone.android.music.provider.MusicContract.SyncColumns;
import com.ubuntuone.android.music.provider.MusicDatabase;
import com.ubuntuone.android.music.provider.MusicProviderUtils;
import com.ubuntuone.android.music.provider.dao.AlbumDao;
import com.ubuntuone.android.music.provider.dao.ArtistAlbumDao;
import com.ubuntuone.android.music.provider.dao.ArtistDao;
import com.ubuntuone.android.music.provider.dao.GenreDao;
import com.ubuntuone.android.music.provider.dao.PlaylistDao;
import com.ubuntuone.android.music.provider.dao.PlaylistSongDao;
import com.ubuntuone.android.music.provider.dao.SongDao;
import com.ubuntuone.android.music.util.NotAuthenticatedException;
import com.ubuntuone.android.music.util.UIUtils;
import com.ubuntuone.api.music.U1MusicAPI;
import com.ubuntuone.api.music.client.Failure;
import com.ubuntuone.api.music.model.U1Album;
import com.ubuntuone.api.music.model.U1Artist;
import com.ubuntuone.api.music.model.U1Playlist;
import com.ubuntuone.api.music.model.U1Song;
import com.ubuntuone.api.music.request.U1AlbumListener;
import com.ubuntuone.api.music.request.U1ArtistListener;
import com.ubuntuone.api.music.request.U1CallbackListener;
import com.ubuntuone.api.music.request.U1PlaylistListener;
import com.ubuntuone.api.music.request.U1SongListener;

public class SyncService extends IntentService
{
	public static final String TAG = makeLogTag(SyncService.class);
	
	public static final String ACTION_SYNC_MUSIC_METADATA =
			"com.ubuntuone.android.music.ACTION_SYNC_MUSIC_METADATA";
	public static final String ACTION_SYNC_PLAYLISTS =
			"com.ubuntuone.android.music.ACTION_SYNC_PLAYLISTS";
	
	public static final String EXTRA_RECEIVER = "result_receiver";
	public static final String EXTRA_LOADER_ID = "loader_id";
	
	public static final int SYNC_STARTED = 1;
	public static final int SYNC_PROGRESS = 2;
	public static final int SYNC_FINISHED = 3;
	
	private static final boolean DIRECT_SQL = true;
	
	private static int sSyncStatus = 0;
	
	private U1MusicAPI mApi;
	
	private ContentResolver mResolver;
	
	private ResultReceiver mReceiver;
	
	private long mTimeSyncStarted = 0;
	
	public SyncService() {
		super(SyncService.class.getName());
	}
	
	@Override
	public void onCreate() {
		super.onCreate();
		mResolver = getContentResolver();
	}

	@Override
	protected void onHandleIntent(Intent intent) {
		final String action = intent.getAction();
		if (action == null) return;
		
		try {
			mApi = UbuntuOneMusic.getInstance().getMusicApi();
		} catch (NotAuthenticatedException e) {
			Log.w(TAG, "Not authenticated, shutting down SyncService.");
			return;
		}
		
		mReceiver = intent.getParcelableExtra(EXTRA_RECEIVER);
		mTimeSyncStarted = System.currentTimeMillis();
		if (UbuntuOneMusic.SHOULD_SYNC) {
			sSyncStatus = SYNC_STARTED;
			if (mReceiver != null) mReceiver.send(SYNC_STARTED, Bundle.EMPTY);
			
			if (ACTION_SYNC_MUSIC_METADATA.equals(action)) {
				syncPlaylistMetadata();
				syncMusicMetadata();
			} else if (ACTION_SYNC_PLAYLISTS.equals(action)) {
				syncPlaylistMetadata();
			}
			
			if (mReceiver != null) mReceiver.send(SYNC_FINISHED, Bundle.EMPTY);
			sSyncStatus = SYNC_FINISHED;
		}
		long now = System.currentTimeMillis();
		Log.i(TAG, "Ubuntu One music metadata sync sync took " + (now - mTimeSyncStarted) + " ms");
	}
	
	private void syncMusicMetadata() {
		syncArtists();
		syncAlbums();
		syncSongs();
		syncPlaylists();
		syncPlaylistSongs();
		// Purge only if network connection still present.
		if (UIUtils.isNetworkConnected(this)) {
			purgeInvalidData(mTimeSyncStarted);
		}
	}
	
	private void syncPlaylistMetadata() {
		syncUpPlaylists();
	}
	
	private void syncArtists() {
		final long timeMillis = System.currentTimeMillis();
		mApi.getArtists(mArtistListener);
		final long nowMillis = System.currentTimeMillis();
		Log.i(TAG, "Artists sync took " + (nowMillis - timeMillis) + "ms");
	}
	
	private void syncAlbums() {
		final long timeMillis = System.currentTimeMillis();
		mApi.getAlbums(mAlbumListener);
		final long nowMillis = System.currentTimeMillis();
		Log.i(TAG, "Albums sync took " + (nowMillis - timeMillis) + "ms");
	}
	
	private void syncSongs() {
		final long timeMillis = System.currentTimeMillis();
		mApi.getSongs(mSongListener);
		final long nowMillis = System.currentTimeMillis();
		Log.i(TAG, "Songs sync took " + (nowMillis - timeMillis) + "ms");
	}
	
	private void syncPlaylists() {
		final long timeMillis = System.currentTimeMillis();
		mApi.getPlaylists(mPlaylistListener);
		final long nowMillis = System.currentTimeMillis();
		Log.i(TAG, "Playlists sync took " + (nowMillis - timeMillis) + "ms");
	}
	
	private void syncUpPlaylists() {
		final long timeMillis = System.currentTimeMillis();
		Cursor cursor = null;
		try {
			String[] projection = new String[] { Playlists.PLAYLIST_ID, Playlists.PLAYLIST_NAME };
			String selection = Playlists.DIRTY + "=1";
			cursor = getContentResolver().query(Playlists.CONTENT_URI,
					projection, selection, null, null);
			if (cursor != null && cursor.isBeforeFirst()) {
				while (cursor.moveToNext()) {
					final String playlistId = cursor.getString(cursor.getColumnIndex(Playlists.PLAYLIST_ID));
					String playlistName = cursor.getString(cursor.getColumnIndex(Playlists.PLAYLIST_NAME));
					ArrayList<String> songIdList = MusicProviderUtils.getPlaylistSongIdList(this, playlistId);
					
					mApi.updatePlaylist(playlistId, playlistName, songIdList, new U1CallbackListener() {
						@Override
						public void onSuccess() {
							Log.i(TAG, "Updated playlist: " + playlistId);
							Uri playlistUri = Playlists.buildPlaylistUri(playlistId);
							MusicProviderUtils.flagPlaylistForSync(SyncService.this, playlistUri, false);
							// TODO Persist playlist entity from response once CSRF fix is rolled out.
						}
					});
				}
			}
		} finally {
			if (cursor != null) cursor.close();
		}
		final long nowMillis = System.currentTimeMillis();
		Log.i(TAG, "Playlist songs sync up took " + (nowMillis - timeMillis) + "ms");
	}
	
	private void syncPlaylistSongs() {
		final long timeMillis = System.currentTimeMillis();
		Cursor cursor = null;
		try {
			String[] projection = new String[] { Playlists.PLAYLIST_ID };
			cursor = getContentResolver().query(Playlists.CONTENT_URI, projection,
					null, null, null);
			while (cursor.moveToNext()) {
				String playlistId = cursor.getString(cursor.getColumnIndex(Playlists.PLAYLIST_ID));
				if (MusicProviderUtils.QUEUE_ID.equals(playlistId)) continue;
				mPlaylistSongListener.setPlaylist(playlistId);
				mApi.getPlaylist(playlistId, mPlaylistSongListener);
			}
		} finally {
			if (cursor != null) cursor.close();
		}
		final long nowMillis = System.currentTimeMillis();
		Log.i(TAG, "Playlist songs sync down took " + (nowMillis - timeMillis) + "ms");
	}
	
	private U1ArtistListener mArtistListener = new U1ArtistListener() {
		private SQLiteDatabase db;
		private ArtistDao artistDao = new ArtistDao();
		private ArtistAlbumDao artistAlbumDao = new ArtistAlbumDao();
		private Bundle progressBundle = new Bundle();
		private Failure failure = null;
		private int artistCount = 0;
		
		@Override
		public void onStart() {
			Log.d(TAG, "Syncing artists...");
			if (DIRECT_SQL) {
				db = new MusicDatabase(SyncService.this).getWritableDatabase();
				db.beginTransaction();
			}
			progressBundle.putInt(EXTRA_LOADER_ID, R.id.artist_loader_id);
		}

		@Override
		public void onSuccess(U1Artist artist) {
			artistCount++;
			if (DIRECT_SQL) {
				if (artistDao.updateOrInsert(db, artist)) {
					if (!artistAlbumDao.updateOrInsert(db, artist)) {
						Log.w(TAG, "Artist album not inserted: " + artist.getId());
					}
				}
			} else {
				artistDao.updateOrInsert(mResolver, artist);
				artistAlbumDao.updateOrInsert(mResolver, artist);
			}
			mReceiver.send(SYNC_PROGRESS, progressBundle);
		}

		@Override
		public void onFailure(Failure failure) {
			this.failure = failure;
			Log.e(TAG, String.format("Failed to get artists: %d -- %s",
					failure.getStatusCode(), failure.getMessage()));
		}

		@Override
		public void onFinish() {
			if (DIRECT_SQL) {
				if (failure == null) db.setTransactionSuccessful();
				db.endTransaction();
				db.close();
			}
			Log.d(TAG, String.format("Finished syncing artists " +
					"(got %d artists).", artistCount));
		}
	};
	
	private U1AlbumListener mAlbumListener = new U1AlbumListener() {
		private SQLiteDatabase db;
		private AlbumDao albumDao = new AlbumDao();
		private Bundle progressBundle = new Bundle();
		private Failure failure = null;
		private int albumCount = 0;
		
		@Override
		public void onStart() {
			Log.d(TAG, "Syncing albums...");
			if (DIRECT_SQL) {
				db = new MusicDatabase(SyncService.this).getWritableDatabase();
				db.beginTransaction();
			}
			progressBundle.putInt(EXTRA_LOADER_ID, R.id.album_loader_id);
		}
		
		@Override
		public void onSuccess(U1Album album) {
			albumCount++;
			if (DIRECT_SQL) {
				albumDao.updateOrInsert(db, album);
			} else {
				albumDao.updateOrInsert(mResolver, album);
			}
			mReceiver.send(SYNC_PROGRESS, progressBundle);
		}

		@Override
		public void onFailure(Failure failure) {
			this.failure = failure;
			Log.e(TAG, String.format("Failed to get albums: %d -- %s",
					failure.getStatusCode(), failure.getMessage()));
		}
		
		@Override
		public void onFinish() {
			if (DIRECT_SQL) {
				if (failure == null) db.setTransactionSuccessful();
				db.endTransaction();
				db.close();
			}
			Log.d(TAG, String.format("Finished syncing albums (got %s albums).",
					albumCount));
		}
	};
	
	private U1SongListener mSongListener = new U1SongListener() {
		private SQLiteDatabase db;
		private SongDao songDao = new SongDao();
		private GenreDao genreDao = new GenreDao();
		private Bundle progressBundle = new Bundle();
		private Failure failure = null;
		private int songCount = 0;
		
		@Override
		public void onStart() {
			Log.d(TAG, "Syncing songs...");
			if (DIRECT_SQL) {
				db = new MusicDatabase(SyncService.this).getWritableDatabase();
				db.beginTransaction();
			}
			progressBundle.putInt(EXTRA_LOADER_ID, R.id.song_loader_id);
		}
		
		@Override
		public void onSuccess(U1Song song) {
			songCount++;
			if (DIRECT_SQL) {
				if (songDao.updateOrInsert(db, song)) {
					if (!genreDao.insert(db, song.getGenre())) {
						Log.w(TAG, "Genre not inserted: " + song.getGenre());
					}
				}
			} else {
				songDao.updateOrInsert(mResolver, song);
			}
			mReceiver.send(SYNC_PROGRESS, progressBundle);
		}

		@Override
		public void onFailure(Failure failure) {
			this.failure = failure;
			Log.e(TAG, String.format("Failed to get songs: %d -- %s",
					failure.getStatusCode(), failure.getMessage()));
		}
		
		@Override
		public void onFinish() {
			if (DIRECT_SQL) {
				if (failure == null) db.setTransactionSuccessful();
				db.endTransaction();
				db.close();
			}
			Log.d(TAG, String.format("Finished syncing songs (got %d songs).",
					songCount));
		}
	};
	
	private U1PlaylistListener mPlaylistListener = new U1PlaylistListener() {
		private SQLiteDatabase db;
		private PlaylistDao playlistDao = new PlaylistDao();
		private Bundle progressBundle = new Bundle();
		private Failure failure = null;
		private int playlistCount = 0;
		
		@Override
		public void onStart() {
			Log.d(TAG, "Syncing playlists...");
			if (DIRECT_SQL) {
				db = new MusicDatabase(SyncService.this).getWritableDatabase();
				db.beginTransaction();
			}
			progressBundle.putInt(EXTRA_LOADER_ID, R.id.playlist_loader_id);
		}
		
		@Override
		public void onSuccess(U1Playlist playlist) {
			playlistCount++;
			if (DIRECT_SQL) {
				playlistDao.updateOrInsert(db, playlist);
			} else {
				playlistDao.updateOrInsert(mResolver, playlist);
			}
			mReceiver.send(SYNC_PROGRESS, progressBundle);
		}

		@Override
		public void onFailure(Failure failure) {
			this.failure = failure;
			Log.e(TAG, String.format("Failed to get playlists: %d -- %s",
					failure.getStatusCode(), failure.getMessage()));
		}
		
		@Override
		public void onFinish() {
			if (DIRECT_SQL) {
				if (failure == null) db.setTransactionSuccessful();
				db.endTransaction();
				db.close();
			}
			Log.d(TAG, String.format("Finished syncing playlists (got %d playlists).",
					playlistCount));
		}
	};
	
	private abstract class PlaylistSongListener extends U1SongListener {
		protected String mPlaylistId;
		
		public void setPlaylist(String playlistId) {
			this.mPlaylistId = playlistId;
		}
	}
	
	private PlaylistSongListener mPlaylistSongListener = new PlaylistSongListener() {
		private SQLiteDatabase db;
		private PlaylistSongDao playlistSongDao = new PlaylistSongDao();
		private Failure failure = null;
		private int playlistSongCount = 0;
		
		@Override
		public void onStart() {
			playlistSongCount = 0; // This listener is reused, so reset.
			Log.d(TAG, "Syncing playlist songs...");
			if (DIRECT_SQL) {
				db = new MusicDatabase(SyncService.this).getWritableDatabase();
				db.beginTransaction();
			}
		}
		
		@Override
		public void onSuccess(U1Song song) {
			playlistSongCount++;
			if (DIRECT_SQL) {
				playlistSongDao.updateOrInsert(db, mPlaylistId, song.getId());
			} else {
				playlistSongDao.updateOrInsert(mResolver, mPlaylistId, song);
			}
		}

		@Override
		public void onFailure(Failure failure) {
			this.failure = failure;
			Log.e(TAG, String.format("Failed to get playlist songs: %d -- %s",
					failure.getStatusCode(), failure.getMessage()));
		}
		
		@Override
		public void onFinish() {
			if (DIRECT_SQL) {
				if (failure == null) db.setTransactionSuccessful();
				db.endTransaction();
				db.close();
			}
			Log.d(TAG, String.format("Finished syncing playlist songs (got %s playlist songs).",
					playlistSongCount));
		}
	};
	
	private void purgeInvalidData(long lastSyncStart) {
		String where = SyncColumns.UPDATED + "<?";
		String[] selectionArgs = new String[] { String.valueOf(lastSyncStart) };
		int entries = 0, entriesTotal = 0;
		
		// Do not purge play queue playlist and its songs.
		String playlistSelection = where + " AND " + Playlists.PLAYLIST_ID + " != ?";
		String[] playlistSelectionArgs = new String[] {
				String.valueOf(lastSyncStart), MusicProviderUtils.QUEUE_ID
			};
		entries = mResolver.delete(Playlists.getPlaylistsSongsUri(),
				playlistSelection, playlistSelectionArgs);
		entriesTotal += entries;
		Log.d(TAG, String.format("Cleaned up %s old playlist songs.", entries));
		entries = mResolver.delete(Playlists.CONTENT_URI,
				playlistSelection, playlistSelectionArgs);
		entriesTotal += entries;
		Log.d(TAG, String.format("Cleaned up %s old playlists.", entries));
		
		entries = mResolver.delete(Songs.CONTENT_URI, where, selectionArgs);
		entriesTotal += entries;
		Log.d(TAG, String.format("Cleaned up %s old songs.", entries));
		entries = mResolver.delete(Albums.CONTENT_URI, where, selectionArgs);
		entriesTotal += entries;
		Log.d(TAG, String.format("Cleaned up %s old albums.", entries));
		entries = mResolver.delete(Artists.getArtistsAlbumsUri(), where, selectionArgs);
		entriesTotal += entries;
		Log.d(TAG, String.format("Cleaned up %s old artist albums.", entries));
		entries = mResolver.delete(Artists.CONTENT_URI, where, selectionArgs);
		entriesTotal += entries;
		Log.i(TAG, String.format("Cleaned up total of %s old entries.", entriesTotal));
	}
	
	public static boolean isSyncRunning() {
		return sSyncStatus == SYNC_STARTED;
	}
	
	public static void syncMusicMetadata(Context context, ResultReceiver receiver) {
		if (!isSyncRunning()) {
			Intent intent = new Intent(ACTION_SYNC_MUSIC_METADATA);
			intent.putExtra(EXTRA_RECEIVER, receiver);
			context.startService(intent);
		} else {
			Log.i(TAG, "Music metadata sync already running.");
		}
	}
}
