/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.ui;

import static com.ubuntuone.android.music.util.LogUtils.makeLogTag;
import android.app.Activity;
import android.content.ComponentName;
import android.content.ContentResolver;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.support.v4.content.CursorLoader;
import android.support.v4.content.Loader;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.ImageButton;
import android.widget.ListView;
import android.widget.SeekBar;
import android.widget.SeekBar.OnSeekBarChangeListener;
import android.widget.SlidingDrawer;
import android.widget.Toast;

import com.ubuntuone.android.music.R;
import com.ubuntuone.android.music.adapter.U1SongAdapter;
import com.ubuntuone.android.music.adapter.holder.PlayerViewHolder;
import com.ubuntuone.android.music.model.PlayerState;
import com.ubuntuone.android.music.model.Song;
import com.ubuntuone.android.music.provider.MusicContract.PlaylistSongs;
import com.ubuntuone.android.music.provider.MusicContract.Playlists;
import com.ubuntuone.android.music.provider.MusicContract.Songs;
import com.ubuntuone.android.music.provider.MusicProviderUtils;
import com.ubuntuone.android.music.service.MusicService;
import com.ubuntuone.android.music.service.MusicServiceBinder;
import com.ubuntuone.android.music.service.MusicServiceCallback;
import com.ubuntuone.android.music.util.U1ImageDownloader;
import com.ubuntuone.android.music.widget.ColoringToggleButton;

public class PlayerFragment extends LoadingFragment<U1SongAdapter> implements OnClickListener,
		MusicServiceCallback, OnSeekBarChangeListener
{
	public static final String TAG = makeLogTag(PlayerFragment.class);
	
	private ServiceConnection mConn;
	private boolean mIsBound = false;
	private MusicService mService;
	
	private U1ImageDownloader mImageDownloader;
	
	private PlayerActivity mActivity;
	private Handler mHandler;
	
	private SlidingDrawer mDrawer;
	private PlayerViewHolder mViewHolder;
	
	private ColoringToggleButton mShuffle;
	private ImageButton mPrev;
	private ImageButton mPlay;
	private ImageButton mNext;
	private ColoringToggleButton mRepeat;
	
	private Boolean mShufflePlay = Boolean.FALSE;
	private Boolean mRepeatPlay = Boolean.FALSE;
	
	// State:
	private boolean mAutoPlay = false;
	private Uri mSongUri;

	@Override
	public int onGetHeaderResId() {
		return 0;
	}

	@Override
	public int onGetLoaderId() {
		return R.id.queue_loader_id;
	}

	@Override
	public int onGetFragmentLayoutResId() {
		return R.layout.fragment_player;
	}

	@Override
	public int onGetEmptyListResId() {
		return R.string.queue_no_found;
	}

	@Override
	public int onGetLoadingListResId() {
		return R.string.queue_loading;
	}
	
	@Override
	public void onAttach(Activity activity) {
		super.onAttach(activity);
		mActivity = (PlayerActivity) activity;
	}
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		mHandler = new Handler();
		mImageDownloader = U1ImageDownloader.getAlbumArtDownloader();
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		View contentView = super.onCreateView(inflater, container, savedInstanceState);
		
		mViewHolder = new PlayerViewHolder(contentView);
		mViewHolder.albumArt.setOnClickListener(this);
		mViewHolder.progress.setOnSeekBarChangeListener(this);
		
		mDrawer = (SlidingDrawer) contentView.findViewById(R.id.drawer);
		
		mShuffle = (ColoringToggleButton) contentView.findViewById(R.id.shuffle);
		mShuffle.setOnClickListener(this);
		mPrev = (ImageButton) contentView.findViewById(R.id.prev);
		mPrev.setOnClickListener(this);
		mPlay = (ImageButton) contentView.findViewById(R.id.play);
		mPlay.setOnClickListener(this);
		mNext = (ImageButton) contentView.findViewById(R.id.next);
		mNext.setOnClickListener(this);
		mRepeat = (ColoringToggleButton) contentView.findViewById(R.id.repeat);
		mRepeat.setOnClickListener(this);
		
		return contentView;
	}

	@Override
	public void onActivityCreated(Bundle savedInstanceState) {
		super.onActivityCreated(savedInstanceState);
		
		Intent intent = getSherlockActivity().getIntent();
		if (intent != null) {
			if (intent.hasExtra(PlayerActivity.EXTRA_SONG)) {
				mSongUri = intent.getParcelableExtra(PlayerActivity.EXTRA_SONG);
				mAutoPlay = true;
			}
			
			if (intent.hasExtra(PlayerActivity.EXTRA_SHUFFLE_PLAY)) {
				mShufflePlay = intent.getBooleanExtra(PlayerActivity.EXTRA_SHUFFLE_PLAY, false);
				mShuffle.setChecked(mShufflePlay);
			}
			
			if (intent.hasExtra(PlayerActivity.EXTRA_REPEAT_PLAY)) {
				mRepeatPlay = intent.getBooleanExtra(PlayerActivity.EXTRA_REPEAT_PLAY, false);
				mRepeat.setChecked(mRepeatPlay);
			}
			
			if (intent.hasExtra(PlayerActivity.EXTRA_AUTO_PLAY)) {
				mAutoPlay = true;
			}
			
			if (intent.hasExtra(PlayerActivity.EXTRA_SHOW_QUEUE)) {
				mDrawer.open();
			}
		}
		
		if (savedInstanceState != null) {
			// We're resuming, don't cause restart due to auto play flag.
			mAutoPlay = false;
			
			if (savedInstanceState.containsKey("songUri")) {
				mSongUri = savedInstanceState.getParcelable("songUri");
			}
		}
		
		if (mSongUri == null) {
			mSongUri = MusicProviderUtils.getFirstPlayQueueSong(mActivity, mShufflePlay);
		}
	}
	
	@Override
	public void onSaveInstanceState(Bundle outState) {
		super.onSaveInstanceState(outState);
		if (mSongUri != null) {
			outState.putParcelable("songUri", mSongUri);
		}
	}

	@Override
	public void onResume() {
		super.onResume();
		
		Intent service = new Intent(mActivity, MusicService.class);
		mConn = new ServiceConnection() {
			@Override
			public void onServiceDisconnected(ComponentName name) {
				mIsBound = false;
				mService = null;
			}
			
			@SuppressWarnings("unchecked")
			@Override
			public void onServiceConnected(ComponentName name, IBinder service) {
				mService = ((MusicServiceBinder<MusicService>) service).getService();
				mIsBound = true;
				PlayerFragment.this.onServiceConnected();
			}
		};
		if (!mIsBound) {
			mActivity.bindService(service, mConn, Activity.BIND_AUTO_CREATE | Activity.BIND_IMPORTANT);
		}
	}
	
	public void onServiceConnected() {
		if (mAutoPlay) {
			if (mService.getPlayerState() != PlayerState.IDLE) {
				mService.reset();
			}
			mService.play(mSongUri);
			mPlay.setImageResource(android.R.drawable.ic_media_pause);
		} else {
			if (mService.getPlayerState() == PlayerState.IDLE ||
					mService.getPlayerState() == PlayerState.PAUSED ||
					mService.getPlayerState() == PlayerState.STOPPED) {
				mPlay.setImageResource(android.R.drawable.ic_media_play);
			} else {
				mPlay.setImageResource(android.R.drawable.ic_media_pause);
			}
		}
		if (mShufflePlay == null) {
			mShufflePlay = mService.isShufflePlayEnabled();
		}
		if (mRepeatPlay == null) {
			mRepeatPlay = mService.isRepeatPlayEnabled();
		}
		
		Song currentSong = mService.getCurrentPlaying();
		if (currentSong != null) {
			int currentTime = mService.getPlayerPosition();
			int totalTime = mService.getPlayerDuration();
			mViewHolder.bindTimeViews(currentTime, totalTime);
		}
		
		mService.register(this);
	}
	
	@Override
	public void onPause() {
		if (mIsBound && mService != null) {
			try {
				mActivity.unbindService(mConn);
			} catch (IllegalArgumentException e) {
				// Ignore. Service not registered.
			}
		}
		super.onPause();
	}

	@Override
	public void onDestroy() {
		if (mIsBound && mService != null) mService.unregister(this);
		super.onDestroy();
	}

	@Override
	public void onDetach() {
		mActivity = null;
		super.onDetach();
	}
	
	@Override
	public void onListItemClick(ListView l, View v, int position, long id) {
		Cursor cursor = (Cursor) l.getItemAtPosition(position);
		if (cursor != null) {
			String songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
			mSongUri = Songs.buildSongUri(songId);
			if (mIsBound && mService != null) mService.play(mSongUri);
		}
	}

	@Override
	public void onClick(View v) {
		if (!mIsBound || mService == null) {
			return;
		}
		
		switch (v.getId()) {
		case R.id.play:
			onPlayClick(mService);
			break;
		case R.id.prev:
			onPrevClick(mService);
			break;
		case R.id.next:
			onNextClick(mService);
			break;
		case R.id.shuffle:
			onShuffleClick(mService);
			break;
		case R.id.repeat:
			onRepeatClick(mService);
			break;
		case R.id.album_art: // FIXME
			onAlbumArtClick();
			break;

		default:
			Log.w(TAG, "Unhandled click on " + v.getClass().getName());
			break;
		}
	}
	
	public void onPlayClick(MusicService service) {
		if (mSongUri == null) {
			Log.e(TAG, "onPlayClick, mSongUri must not be null.");
			return;
		}
		
		PlayerState state = service.getPlayerState();
		switch (state) {
		case IDLE:
			Log.i(TAG, "onPlayClick, was IDLE");
			//$FALL-THROUGH$
		case STOPPED:
			Log.i(TAG, "onPlayClick, was STOPPED");
			//$FALL-THROUGH$
		case COMPLETED:
			Log.i(TAG, "onPlayClick, was COMPLETED");
			int position = MusicProviderUtils.getSongLastPlayedPosition(mActivity, mSongUri);
			service.play(mSongUri, position);
			break;
		case STARTED:
			Log.i(TAG, "onPlayClick, was STARTED");
			service.pause();
			break;
		case PAUSED:
			Log.i(TAG, "onPlayClick, was PAUSED");
			service.resume();
			break;
		}
	}
	
	public void onPrevClick(MusicService service) {
		service.prev(true);
	}
	
	public void onNextClick(MusicService service) {
		service.next(true);
	}
	
	public void onShuffleClick(MusicService service) {
		mShufflePlay = mShuffle.isChecked();
		service.setShufflePlay(mShufflePlay);
		
		if (mShufflePlay && isAdded()) {
			PlayerActivity activity = (PlayerActivity) getSherlockActivity();
			if (activity != null) {
				activity.reshuffleQueueSongs();
			}
		}
	}
	
	public void onRepeatClick(MusicService service) {
		boolean mRepeatPlay = mRepeat.isChecked();
		service.setRepeatPlay(mRepeatPlay);
	}
	
	private void onAlbumArtClick() {
		mActivity.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				mDrawer.toggle();
			}
		});
	}
	
	@Override
	public Loader<Cursor> onCreateLoader(int arg0, Bundle arg1) {
		Log.d(TAG, "onCreateLoader()");

		Uri playlistSongsUri = Playlists.buildPlaylistSongsUri(MusicProviderUtils.QUEUE_ID);
		CursorLoader loader = new CursorLoader(getSherlockActivity(), playlistSongsUri,
				Songs.getDefaultProjection(), null, null, PlaylistSongs.DEFAULT_SORT);
		
		setIsLoading(true);
		return loader;
	}

	@Override
	public U1SongAdapter onCreateAdapter(Context context, Cursor cursor) {
		return new U1SongAdapter(context, null, true, cursor, true);
	}
	
	@Override
	public IBinder asBinder() {
		return new MusicServiceBinder<MusicServiceCallback>(this);
	}

	@Override
	public void onSongChanged(Uri songUri) {
		if (mActivity == null) {
			return;
		}
		
		mSongUri = songUri;
		
		if (songUri == null) {
			updateNowPlayingOnUIThread(null, null, null, null, null);
			return;
		}
		
		ContentResolver resolver = mActivity.getContentResolver();
		String[] projection = new String[] { Songs.SONG_TITLE,
				Songs.SONG_LAST_PLAYED_AT, Songs.SONG_DURATION,
				Songs.SONG_ARTIST, Songs.SONG_ALBUM_ID};
		Cursor cursor = null;
		try {
			cursor = resolver.query(songUri, projection, null, null, null);
			if (cursor != null && cursor.moveToFirst()) {
				String title = cursor.getString(cursor.getColumnIndex(Songs.SONG_TITLE));
				Integer position = cursor.getInt(cursor.getColumnIndex(Songs.SONG_LAST_PLAYED_AT)) / 1000;
				Integer duration = cursor.getInt(cursor.getColumnIndex(Songs.SONG_DURATION));
				String artist = cursor.getString(cursor.getColumnIndex(Songs.SONG_ARTIST));
				String albumId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ALBUM_ID));
				updateNowPlayingOnUIThread(title, position, duration, artist, albumId);
			} else {
				updateNowPlayingOnUIThread(null, null, null, null, null);
			}
		} catch (Exception e) {
			if (cursor != null) cursor.close();
		}
	}
	
	@Override
	public void onPlayerStateChanged(PlayerState state) {
		switch (state) {
		case IDLE:
			//$FALL-THROUGH$
		case COMPLETED:
			//$FALL-THROUGH$
			mViewHolder.setCurrentTime(0);
		case STOPPED:
			mViewHolder.progress.setProgress(0);
			//$FALL-THROUGH$
		case PAUSED:
			//$FALL-THROUGH$
			mPlay.setImageResource(android.R.drawable.ic_media_play);
			break;
		case DOWNLOADING:
			//$FALL-THROUGH$
		case PREPARING:
			//$FALL-THROUGH$
		case PREPARED:
			//$FALL-THROUGH$
		case STARTED:
			mPlay.setImageResource(android.R.drawable.ic_media_pause);
			break;

		default:
			mPlay.setImageResource(android.R.drawable.ic_media_play);
			break;
		}
	}

	public void updateNowPlayingOnUIThread(final String title,
			final Integer position, final Integer duration,
			final String artist, final String albumId) {
		mHandler.postDelayed(new Runnable() {
			@Override
			public void run() {
				if (mViewHolder != null) {
					mViewHolder.bindView(title, artist, albumId, mImageDownloader);
					mViewHolder.bindTimeViews(0, duration);
				} else {
					mHandler.postDelayed(this, 500);
				}
			}
		}, 500);
		// It seems options menu is asynchronously created after activity has been created.
		// Initial 'now playing' state in UI is not critical, but this hack should cause
		// the condition mNowPlaying != null to be true shortly after activity is created.
	}
	
	Toast t;
	
	@Override
	public void onSongDownload(Uri songUri, final int percent) {
		if (songUri.equals(mSongUri)) {
			final SeekBar progress = mViewHolder.progress;
			mActivity.runOnUiThread(new Runnable() {
				@Override
				public void run() {
					progress.setSecondaryProgress(percent);
				}
			});
		}
	}

	@Override
	public void onSongProgress(Uri songUri, final int percent) {
		if (songUri.equals(mSongUri) && !mIsSeeking) {
			final SeekBar progress = mViewHolder.progress;
			mActivity.runOnUiThread(new Runnable() {
				@Override
				public void run() {
					progress.setProgress(percent);
					int currentTime = (int) (mService.getPlayerPosition() / 1000.0);
					int totalTime = (int) (mService.getPlayerDuration() / 1000.0);
					mViewHolder.bindTimeViews(currentTime, totalTime);
				}
			});
		}
	}
	
	boolean mIsSeeking = false;
	
	@Override
	public void onStartTrackingTouch(SeekBar seekBar) {
		// Do nothing.
		mIsSeeking = true;
	}
	
	@Override
	public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
		Song song = mService.getCurrentPlaying();
		final int cached = seekBar.getSecondaryProgress();
		if (progress < cached) {
			int sec = song.getDuration() * progress / 100;
			mViewHolder.setCurrentTime(sec);
		}
	}

	@Override
	public void onStopTrackingTouch(SeekBar seekBar) {
		mIsSeeking = false;
		
		if (mService.getPlayerState() == PlayerState.STARTED ||
				mService.getPlayerState() == PlayerState.PAUSED) {
			Song song = mService.getCurrentPlaying();
			final int progress = seekBar.getProgress();
			final int cached = seekBar.getSecondaryProgress();
			if (progress < cached) {
				int msec = (song.getDuration() * 1000 / 100) * progress;
				mService.seek(msec);
			}
		}
	}
}
