/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.ui;

import static com.ubuntuone.android.music.ui.HomeActivity.ERROR_DIALOG_ID;
import static com.ubuntuone.android.music.util.LogUtils.makeLogTag;
import android.app.Activity;
import android.content.ContentResolver;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v4.app.LoaderManager;
import android.support.v4.content.Loader;
import android.support.v4.widget.CursorAdapter;
import android.util.Log;
import android.widget.Adapter;
import android.widget.Toast;

import com.actionbarsherlock.app.SherlockFragmentActivity;
import com.actionbarsherlock.view.MenuItem;
import com.ubuntuone.android.music.R;
import com.ubuntuone.android.music.provider.MusicContract.Albums;
import com.ubuntuone.android.music.provider.MusicContract.Artists;
import com.ubuntuone.android.music.provider.MusicContract.Playlists;
import com.ubuntuone.android.music.provider.MusicContract.Songs;
import com.ubuntuone.android.music.provider.MusicProviderUtils;
import com.ubuntuone.android.music.service.SyncService;
import com.ubuntuone.android.music.ui.PlaylistsFragment.OnPlaylistSelectedListener;
import com.ubuntuone.android.music.ui.dialog.InfoAlertDialog.InfoAlertDialogListener;
import com.ubuntuone.android.music.util.UIUtils;

public class PlaylistsActivity extends SherlockFragmentActivity implements
		OnPlaylistSelectedListener, InfoAlertDialogListener
{
	private static final String TAG = makeLogTag(PlaylistActivity.class);
	
	public static final String EXTRA_ENQUEUE_CONTENT_ID = "content_uri";
	
	private Uri mContentUri;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_playlists);
		
		mContentUri = getIntent().getParcelableExtra(EXTRA_ENQUEUE_CONTENT_ID);
		
		if (mContentUri == null) {
			throw new IllegalStateException("You must provide content_uri which is " +
					"to be added to a playlist.");
		}

		LoaderManager lm = getSupportLoaderManager();
		Loader<?> loader = lm.getLoader(R.id.playlist_songs_loader_id);
		if (loader != null) {
			loader.stopLoading();
		}
	}
	
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case android.R.id.home:
			finish();
			return true;

		default:
			return super.onOptionsItemSelected(item);
		}
	}

	@Override
	public void onPlaylistSelected(Adapter adapter, int position, boolean hasHeaders) {
		if (hasHeaders) {
			throw new IllegalArgumentException(
					"PlaylistsActivity must not contain custom playlists.");
		}
		
		Cursor cursor = ((CursorAdapter) adapter).getCursor();
		cursor.moveToPosition(position);
		
		String playlistId = cursor.getString(cursor.getColumnIndex(Playlists.PLAYLIST_ID));
		Uri playlistUri = Playlists.buildPlaylistUri(playlistId);
		
		if (mContentUri != null) {
			AppendToPlaylistTask appendTask = new AppendToPlaylistTask();
			appendTask.execute(playlistUri, mContentUri);
		}
	}
	
	private class AppendToPlaylistTask extends AsyncTask<Uri, Void, Uri> {
		@Override
		protected Uri doInBackground(Uri... params) {
			if (params.length != 2) {
				throw new IllegalArgumentException(
						"AppendToPlaylistTask takes two arguments, playlist uri and content uri");
			}
			final Uri playlistUri = params[0];
			final Uri contentUri = params[1];
			
			ContentResolver resolver = getContentResolver();
			
			String mimeType = resolver.getType(contentUri);
			Activity activity = PlaylistsActivity.this;
			if (mimeType.equals(Artists.CONTENT_ITEM_TYPE)) {
				Log.e(TAG, "artist");
				MusicProviderUtils.enqueueArtist(activity, contentUri, playlistUri);
			} else if (mimeType.equals(Albums.CONTENT_ITEM_TYPE)) {
				Log.e(TAG, "album");
				MusicProviderUtils.enqueueAlbum(activity, contentUri, playlistUri);
			} else if (mimeType.equals(Songs.CONTENT_ITEM_TYPE)) {
				Log.e(TAG, "song");
				MusicProviderUtils.enqueueSong(activity, contentUri, playlistUri);
			} else {
				Log.e(TAG, "Unknown content uri to enqueue to playlist: " + contentUri.toString());
			}
			return playlistUri;
		}

		@Override
		protected void onPostExecute(Uri playlistUri) {
			if (playlistUri == null) {
				Log.e(TAG, "PlaylistsActivity#AppendToPlaylistTask#onPostExecute null playlistUri");
				return;
			}
			
			String playlistName = null;
			Cursor cursor = null;
			try {
				final String[] projection = new String[] { Playlists.PLAYLIST_NAME };
				cursor = getContentResolver().query(playlistUri, projection, null, null, null);
				if (cursor != null && cursor.moveToFirst()) {
					playlistName = cursor.getString(cursor.getColumnIndex(Playlists.PLAYLIST_NAME));
				}
			} finally {
				cursor.close();
			}
			
			String msg = getString(R.string.added_to_fmt, playlistName);
			Toast.makeText(PlaylistsActivity.this, msg, Toast.LENGTH_SHORT).show();
			
			if (UIUtils.isNetworkConnected(PlaylistsActivity.this)) {
				Intent syncPlaylists = new Intent(SyncService.ACTION_SYNC_PLAYLISTS);
				startService(syncPlaylists);
			}
			
			finish();
		}
	}

	@Override
	public void onPositiveButtonClicked(int dialogId) {
		if (dialogId == ERROR_DIALOG_ID) {
			finish();
		}
	}
}
