/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.ui;

import static com.ubuntuone.android.music.ui.AlbumActivity.EXTRA_ALBUM_ID;
import static com.ubuntuone.android.music.ui.GenreActivity.EXTRA_GENRE_ID;
import static com.ubuntuone.android.music.ui.PlaylistActivity.EXTRA_PLAYLIST_ID;
import static com.ubuntuone.android.music.util.LogUtils.makeLogTag;
import android.app.Activity;
import android.content.ContentResolver;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.support.v4.app.LoaderManager;
import android.support.v4.content.CursorLoader;
import android.support.v4.content.Loader;
import android.util.Log;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ListView;

import com.actionbarsherlock.view.Menu;
import com.ubuntuone.android.music.R;
import com.ubuntuone.android.music.adapter.U1SongAdapter;
import com.ubuntuone.android.music.provider.MusicContract.Albums;
import com.ubuntuone.android.music.provider.MusicContract.Genres;
import com.ubuntuone.android.music.provider.MusicContract.PlaylistSongs;
import com.ubuntuone.android.music.provider.MusicContract.Playlists;
import com.ubuntuone.android.music.provider.MusicContract.Songs;
import com.ubuntuone.android.music.provider.MusicProviderUtils;
import com.ubuntuone.android.music.service.SyncService;
import com.ubuntuone.android.music.util.UIUtils;

public class SongsFragment extends LoadingFragment<U1SongAdapter> implements OnClickListener
{
	private static final String TAG = makeLogTag(SongsFragment.class);
	
	private String mAlbumId;
	private String mPlaylistId;
	private String mGenreId;
	
	@Override
	public int onGetHeaderResId() {
		return R.string.songs_title;
	}
	
	@Override
	public int onGetLoaderId() {
		return R.id.song_loader_id;
	}
	
	@Override
	public int onGetFragmentLayoutResId() {
		return R.layout.fragment_songs;
	}
	
	@Override
	public int onGetEmptyListResId() {
		return R.string.songs_no_found;
	}
	
	@Override
	public int onGetLoadingListResId() {
		return R.string.songs_loading;
	}
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		Bundle args = getArguments();
		if (args != null) {
			mAlbumId = args.getString(EXTRA_ALBUM_ID);
			mPlaylistId = args.getString(EXTRA_PLAYLIST_ID);
			mGenreId = args.getString(EXTRA_GENRE_ID);
		}
	}
	
	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		final View contentView = super.onCreateView(inflater, container, savedInstanceState);
		
		boolean isDetailsView = mAlbumId != null || mPlaylistId != null || mGenreId != null;
		View header = contentView.findViewById(R.id.header);
		header.setVisibility(isDetailsView ? View.GONE : View.VISIBLE);
		
		View shuffleAll = contentView.findViewById(R.id.shuffle_all);
		shuffleAll.setVisibility(isDetailsView ? View.GONE : View.VISIBLE);
		if (!isDetailsView) shuffleAll.setOnClickListener(this);
		
		return contentView;
	}
	
	@Override
	public U1SongAdapter onCreateAdapter(Context context, Cursor cursor) {
		Log.d(TAG, "SongsFragment#onCreateAdapter");
		
		return new U1SongAdapter(context, mAlbumId, false, cursor, true);
	}
	
	@Override
	public Loader<Cursor> onCreateLoader(int id, Bundle args) {
		Log.d(TAG, "SongsFragment#onCreateLoader");
		
		CursorLoader loader;
		if (mAlbumId != null) {
			Uri albumSongsUri = Albums.buildSongsUri(mAlbumId);
			loader = new CursorLoader(getSherlockActivity(), albumSongsUri,
					Songs.getDefaultProjection(), null, null, Albums.DEFAULT_SONG_SORT);
		} else if (mPlaylistId != null) {
			if (mPlaylistId.equals(MusicProviderUtils.STARRED_ID)) {
				loader = new CursorLoader(getSherlockActivity(), Songs.STARRED_SONGS_URI,
						Songs.getDefaultProjection(), null, null, Songs.DEFAULT_SORT);
				loader.onContentChanged();
			} else if (mPlaylistId.equals(MusicProviderUtils.CACHED_ID)) {
				loader = new CursorLoader(getSherlockActivity(), Songs.CACHED_SONGS_URI,
						Songs.getDefaultProjection(), null, null, Songs.DEFAULT_SORT);
				loader.onContentChanged();
			} else {
				Uri playlistSongsUri = Playlists.buildPlaylistSongsUri(mPlaylistId);
				loader = new CursorLoader(getSherlockActivity(), playlistSongsUri,
						PlaylistSongs.DEFAULT_PROJECTION.clone(), null, null, PlaylistSongs.DEFAULT_SORT);
			}
		} else if (mGenreId != null) {
			Uri genreSongsUri = Genres.buildGenreSongsUri(mGenreId);
			loader = new CursorLoader(getSherlockActivity(), genreSongsUri,
					Songs.getDefaultProjection(), null, null, Songs.DEFAULT_SORT);
		} else {
			loader = new CursorLoader(getSherlockActivity(), Songs.CONTENT_URI,
					Songs.getDefaultProjection(), null, null, Songs.DEFAULT_SORT);
		}
		setIsLoading(true);
		return loader;
	}
	
	private void clearQueueAndPlay(ListView l, int position) {
		Cursor cursor = (Cursor) l.getAdapter().getItem(position);
		if (cursor != null && !cursor.isClosed()) {
			String songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
			if (songId != null && isAdded()) {
				Uri songUri = Songs.buildSongUri(songId);
				Context context = getSherlockActivity();
				if (mAlbumId != null) {
					Uri albumUri = Albums.buildAlbumUri(mAlbumId);
					MusicProviderUtils.clearAndEnqueueAlbum(context, albumUri);
				} else if (mPlaylistId != null) {
					Uri playlistUri = Playlists.buildPlaylistUri(mPlaylistId);
					MusicProviderUtils.clearAndEnqueuePlaylist(context, playlistUri);
				} else if (mGenreId != null) {
					Uri genreUri = Genres.buildGenreUri(mGenreId);
					MusicProviderUtils.clearAndEnqueueGenre(context, genreUri);
				} else {
					MusicProviderUtils.clearAndEnqueueSong(context, songUri);
				}
				
				PlayerActivity.startFrom(context, songUri);
			}
		}
	}

	@Override
	public void onListItemClick(ListView l, View v, int position, long id) {
		clearQueueAndPlay(l, position);
	}

	@Override
	public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
		if (v.getId() == android.R.id.list) {
			menu.add(Menu.NONE, R.id.context_play, 0,
					R.string.description_play);
			menu.add(Menu.NONE, R.id.context_star, 0,
					R.string.description_star);
			menu.add(Menu.NONE, R.id.context_add_to_playlist, 0,
					R.string.description_add_to_playlist);
			if (mPlaylistId != null) {
				menu.add(Menu.NONE, R.id.context_remove_from_playlist, 0,
						R.string.description_remove_from_playlist);
			}
			menu.add(Menu.NONE, R.id.context_add_to_queue, 0,
					R.string.description_add_to_queue);
		} else {
			super.onCreateContextMenu(menu, v, menuInfo);
		}
	}

	@Override
	public boolean onContextItemSelected(MenuItem item) {
		AdapterView.AdapterContextMenuInfo info =
				(AdapterView.AdapterContextMenuInfo) item.getMenuInfo();
		int position = info.position;
		
		switch (item.getItemId()) {
		case R.id.context_play:
			return onPlayContextItemSelected(position);
		case R.id.context_star:
			return onStarContextItemSelected(position);
		case R.id.context_add_to_playlist:
			return onAddToPlaylistContextItemSelected(position);
		case R.id.context_remove_from_playlist:
			return onRemoveFromPlaylistContextItemSelected(position);
		case R.id.context_add_to_queue:
			return onAddToPlayQueueContextItemSelected(position);
		default:
			return super.onContextItemSelected(item);
		}
	}
	
	private void forceReload() {
		int loaderId = getLoaderId();
		LoaderManager lm = getSherlockActivity().getSupportLoaderManager();
		lm.getLoader(loaderId).forceLoad();
	}
	
	private boolean onPlayContextItemSelected(int position) {
		clearQueueAndPlay(mListView, position);
		return true;
	}
	
	private boolean onStarContextItemSelected(int position) {
		Cursor cursor = (Cursor) mListView.getItemAtPosition(position);
		if (cursor != null && !cursor.isClosed()) {
			String songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
			if (songId != null && isAdded()) {
				Uri songUri = Songs.buildSongUri(songId);
				Context context = getSherlockActivity();
				
				MusicProviderUtils.toggleSongStarred(context, songUri);
				
				forceReload();
			}
		}
		return true;
	}
	
	private boolean onAddToPlaylistContextItemSelected(int position) {
		Cursor cursor = (Cursor) mListView.getItemAtPosition(position);
		String songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
		Uri songUri = Songs.buildSongUri(songId);
		
		Intent intent = new Intent(getActivity(), PlaylistsActivity.class);
		intent.putExtra(PlaylistsActivity.EXTRA_ENQUEUE_CONTENT_ID, songUri);
		startActivity(intent);
		return true;
	}
	
	private boolean onRemoveFromPlaylistContextItemSelected(int position) {
		Cursor cursor = (Cursor) mListView.getItemAtPosition(position);
		String rowId = cursor.getString(cursor.getColumnIndex(Songs._ID));
		
		ContentResolver resolver = getActivity().getContentResolver();
		String selection = Songs._ID + "=?";
		String[] selectionArgs = new String[] { rowId };
		resolver.delete(PlaylistSongs.CONTENT_URI, selection, selectionArgs);
		
		Activity activity = getActivity();
		Uri playlistUri = Playlists.buildPlaylistUri(mPlaylistId);
		MusicProviderUtils.flagPlaylistForSync(activity, playlistUri, true);
		
		forceReload();
		if (activity != null && UIUtils.isNetworkConnected(activity)) {
			Intent syncPlaylists = new Intent(SyncService.ACTION_SYNC_PLAYLISTS);
			activity.startService(syncPlaylists);
		}
		return true;
	}
	
	// TODO test
	private boolean onAddToPlayQueueContextItemSelected(int position) {
		Cursor cursor = (Cursor) mListView.getItemAtPosition(position);
		String songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
		Uri songUri = Songs.buildSongUri(songId);
		Activity activity = getSherlockActivity();
		if (activity != null) MusicProviderUtils.enqueueSong(activity, songUri);
		return true;
	}

	@Override
	public void onClick(View v) {
		if (R.id.shuffle_all == v.getId()) {
			onShuffleAllButtonClicked();
		}
	}
	
	private void onShuffleAllButtonClicked() {
		if (isAdded()) {
			HomeActivity activity = (HomeActivity) getSherlockActivity();
			if (activity != null) {
				activity.shuffleAllSongs();
				PlayerActivity.startFrom(getSherlockActivity(), null);
			}
		}
	}
}
