/*
 * Copyright 2012 Google Inc.
 * Copyright 2012-2013 Canonical Ltd.
 * 
 * Updated for the needs of Ubuntu One Music.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.ubuntuone.android.music.util;

import static com.ubuntuone.android.music.util.LogUtils.LOGE;
import static com.ubuntuone.android.music.util.LogUtils.makeLogTag;
import android.accounts.Account;
import android.accounts.AccountManager;
import android.accounts.AccountManagerCallback;
import android.accounts.AccountManagerFuture;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.text.TextUtils;

import com.ubuntuone.android.music.BuildConfig;
import com.ubuntuone.android.music.ui.LoginActivity;

/**
 * An assortment of authentication and login helper utilities.
 */
public class AccountUtils {
	private static final String TAG = makeLogTag(AccountUtils.class);

	private static final String PREF_CHOSEN_ACCOUNT = "chosen_account";
	private static final String PREF_AUTH_TOKEN = "auth_token";

	public static final String U1F_PACKAGE = "com.ubuntuone.android.files";
	public static final String ACTION_SIGN_IN = "com.ubuntuone.android.files.ACTION_SIGN_IN";

	// Account type that will be used in the future by Ubuntu One Files authenticator.
	public static final String U1_ACCOUNT_TYPE = "com.ubuntu.one";
	// Account type currently used by Ubuntu One Files:
	public static final String OLD_U1_ACCOUNT_TYPE = "com.ubuntu";
	// Default authentication token type of Ubuntu One account.
	public static final String AUTH_TOKEN_TYPE = "ubuntuone";
	
	public static final String NOT_AM_ACCOUNT = "no-am-account";

//    // The auth scope required for the app. In our case we use the "conference API"
//    // (not currently open source) which requires the developerssite (and readonly variant) scope.
//    private static final String AUTH_TOKEN_TYPE =
//            "oauth2:"
//                    + "https://www.googleapis.com/auth/developerssite "
//                    + "https://www.googleapis.com/auth/developerssite.readonly ";

	public static boolean isAuthenticated(Context context) {
		return BuildConfig.DEBUG || !TextUtils.isEmpty(getChosenAccountName(context));
	}

	public static void ensureAccountExists(Context context, String accountName)
			throws NotAuthenticatedException {
		if (NOT_AM_ACCOUNT.equals(accountName)) {
			// U1M self created token. We don't need to look at existing accounts.
			return;
		}
		AccountManager accountManager = AccountManager.get(context);
		String accountType = getPreferredAccountType(context);
		final Account[] accounts = accountManager.getAccountsByType(accountType);

		final int size = accounts.length;
		for (int i = 0; i < size; i++) {
			if (accountName.equals(accounts[i].name))
				return;
		}
		throw new NotAuthenticatedException();
	}

	public static String getPreferredAccountType(Context context) {
		AccountManager accountManager = AccountManager.get(context);
		Account[] accounts = accountManager.getAccountsByType(U1_ACCOUNT_TYPE);
		if (accounts.length > 0) {
			return U1_ACCOUNT_TYPE;
		}
		// Otherwise, always return the old account type, regardless if the account exists.
		return OLD_U1_ACCOUNT_TYPE;
	}

    public static String getChosenAccountName(final Context context) {
        SharedPreferences sp = PreferenceManager.getDefaultSharedPreferences(context);
        return sp.getString(PREF_CHOSEN_ACCOUNT, null);
    }

    // private
    public static void setChosenAccountName(final Context context, final String accountName) {
        SharedPreferences sp = PreferenceManager.getDefaultSharedPreferences(context);
                        sp.edit().putString(PREF_CHOSEN_ACCOUNT, accountName).commit();
    }

    public static String getAuthToken(final Context context) {
    	if (BuildConfig.DEBUG) {
    		return "a:b:c:d"; // Fake OAuth serialized token.
    	} else {
    		SharedPreferences sp = PreferenceManager.getDefaultSharedPreferences(context);
    		return sp.getString(PREF_AUTH_TOKEN, null);
    	}
    }

    // private
    public static void setAuthToken(final Context context, final String authToken) {
        SharedPreferences sp = PreferenceManager.getDefaultSharedPreferences(context);
        sp.edit().putString(PREF_AUTH_TOKEN, authToken).commit();
    }

    public static void invalidateAuthToken(final Context context) {
    	// Only U1F can invalidate the token.
        //AccountManager am = AccountManager.get(context);
        //am.invalidateAuthToken(GoogleAccountManager.ACCOUNT_TYPE, getAuthToken(context));
        setAuthToken(context, null);
    }

    public static interface AuthenticateCallback {
        public boolean shouldCancelAuthentication();
        public void onAuthTokenAvailable(String authToken);
    }

//    public static void tryAuthenticate(Activity activity, AuthenticateCallback callback,
//            int activityRequestCode, Account account) {
//        //noinspection deprecation
//        AccountManager.get(activity).getAuthToken(
//                account,
//                AUTH_TOKEN_TYPE,
//                false,
//                getAccountManagerCallback(callback, account, activity, activity,
//                        activityRequestCode),
//                null);
//    }
//
//    public static void tryAuthenticateWithErrorNotification(Context context,
//            AuthenticateCallback callback, Account account) {
//        //noinspection deprecation
//        AccountManager.get(context).getAuthToken(
//                account,
//                AUTH_TOKEN_TYPE,
//                true,
//                getAccountManagerCallback(callback, account, context, null, 0),
//                null);
//    }

    @SuppressWarnings("unused")
	private static AccountManagerCallback<Bundle> getAccountManagerCallback(
            final AuthenticateCallback callback, final Account account,
            final Context context, final Activity activity, final int activityRequestCode) {
        return new AccountManagerCallback<Bundle>() {
            public void run(AccountManagerFuture<Bundle> future) {
                if (callback != null && callback.shouldCancelAuthentication()) {
                    return;
                }

                try {
                    Bundle bundle = future.getResult();
                    if (activity != null && bundle.containsKey(AccountManager.KEY_INTENT)) {
                        Intent intent = bundle.getParcelable(AccountManager.KEY_INTENT);
                        intent.setFlags(intent.getFlags() & ~Intent.FLAG_ACTIVITY_NEW_TASK);
                        activity.startActivityForResult(intent, activityRequestCode);

                    } else if (bundle.containsKey(AccountManager.KEY_AUTHTOKEN)) {
                        final String token = bundle.getString(AccountManager.KEY_AUTHTOKEN);
                        setAuthToken(context, token);
                        setChosenAccountName(context, account.name);
                        if (callback != null) {
                            callback.onAuthTokenAvailable(token);
                        }
                    }
                } catch (Exception e) {
                    LOGE(TAG, "Authentication error", e);
                }
            }
        };
    }

    public static void signOut(final Context context) {
        invalidateAuthToken(context);
        SharedPreferences sp = PreferenceManager.getDefaultSharedPreferences(context);
        sp.edit().clear().commit();
        // TODO Clear music metadata database.
        //context.getContentResolver().delete(MusicContract.BASE_CONTENT_URI, null, null);
    }

    // TODO Authenticate.
//    public static void start_AuthenticationFlow(final Context context, final Intent finishIntent) {
//        Intent loginFlowIntent = new Intent(context, AccountActivity.class);
//        loginFlowIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
//        loginFlowIntent.putExtra(AccountActivity.EXTRA_FINISH_INTENT, finishIntent);
//        context.startActivity(loginFlowIntent);
//    }
    
    public static void startAuthenticationFlow(final Activity activity, int requestCode) {
    	if (!BuildConfig.DEBUG) {
    		Intent loginFlowIntent = new Intent(activity, LoginActivity.class);
    		activity.startActivityForResult(loginFlowIntent, requestCode);
    	} else {
    		Log.w(TAG, "Debug build, not starting authentication flow.");
    	}
    }
}
