/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.util;

import java.io.File;

import android.content.Context;
import android.os.Environment;
import android.text.TextUtils;
import android.webkit.MimeTypeMap;

import com.ubuntuone.android.music.UbuntuOneMusic;
import com.ubuntuone.api.music.model.U1Song;

/**
 * File utilities related to size, MIME types, etc.
 * 
 * @author Michał Karnicki <mkarnicki@gmail.com>
 */
public class FileUtilities {
	public static final String TAG = FileUtilities.class.getSimpleName();

	private static MimeTypeMap mimeMap = MimeTypeMap.getSingleton();
	
	public static String getMime(String filename) {
		int lastIndex = filename.lastIndexOf('.');
		String mime = null;
		if (lastIndex != -1 && (lastIndex + 1 < filename.length())) {
			String extension = filename.substring(lastIndex + 1).toLowerCase();
			mime = mimeMap.getMimeTypeFromExtension(extension);
		}
		if (mime == null)
			mime = "application/octet-stream";
		return mime;
	}
	
	public static final long B = 1;
	// starting 100B display in KiB
	public static final long kibLimit = 100;
	public static final long KiB = 1024*B;
	// starting 100.000B display in MiB
	public static final long mibLimit = 1000*kibLimit;
	public static final long MiB = 1024*KiB;
	// starting 100.000.000B display in GiB
	public static final long gibLimit = 1000*mibLimit;
	public static final long GiB = 1024*MiB;
	
	
	/**
	 * Formatter.formatShortFileSize is not available on Android 1.5
	 * 
	 * @param ctx
	 * @param bytes
	 * @return
	 */
	public static String getHumanReadableSize(final long bytes) {
		String result = null;
		if (bytes >= gibLimit) {
			result = String.format("%.1f GB", bytes/(float)GiB);
		} else if (bytes >= mibLimit) {
			result = String.format("%.1f MB", bytes/(float)MiB);
		} else if (bytes >= kibLimit) {
			result = String.format("%.1f KB", bytes/(float)KiB);
		} else if (bytes >= 0) {
			result = String.format("%d B", bytes);
		} else {
			result = "?";
		}
		return result;
	}

	static boolean mExternalStorageAvailable = false;
	static boolean mExternalStorageWriteable = false;

	private static void updateExternalStorageState() {
		String state = Environment.getExternalStorageState();
		if (Environment.MEDIA_MOUNTED.equals(state)) {
			mExternalStorageAvailable = mExternalStorageWriteable = true;
		} else if (Environment.MEDIA_MOUNTED_READ_ONLY.equals(state)) {
			mExternalStorageAvailable = true;
			mExternalStorageWriteable = false;
		} else {
			mExternalStorageAvailable = mExternalStorageWriteable = false;
		}
	}
	
	public static File getCacheDirectory() {
		updateExternalStorageState();
		if (mExternalStorageWriteable == true) {
			return new File(Environment.getExternalStorageDirectory().getAbsolutePath() +
					"/Android/data/com.ubuntuone.android.music/cache");
		} else {
			Context context = UbuntuOneMusic.getContext();
			return context.getCacheDir();
		}
	}
	
	public static void clearThumbsCacheDirectory(int thumbSize) {
		File sizedThumbCacheDirectory = getCacheDirectory();
		File[] files = sizedThumbCacheDirectory.listFiles();
		if (files != null) {
			for (File file : files) {
				file.delete();
			}
		}
	}
	
	public static void removeSilently(String path) {
		try {
			if (!TextUtils.isEmpty(path)) {
				File file = new File(path);
				if (file.exists()) {
					if (file.isDirectory()) {
						File[] children = file.listFiles();
						if (children != null) {
							for (File child : children) {
								removeSilently(child.getAbsolutePath());
							}
						}
					} else {
						file.delete();
					}
				}
			}
		} catch (Exception e) {
			// Ignore.
		}
	}
	
	private static final String[] FILE_SYSTEM_UNSAFE = {"/", "\\", "..", ":", "\"", "?", "*", "<", ">"};
	
	public static String sanitize(String filename) {
		if (filename == null || TextUtils.isEmpty(filename.trim())) {
			return "noname";
		}
		for (String s : FILE_SYSTEM_UNSAFE) {
			filename = filename.replace(s, "-");
		}
		return filename;
	}
	
	/**
	 * TODO What about colliding path
	 * /Unknown artist/Unknown album/Unknown track.mp3 ?
	 * 
	 * @param song
	 * @return
	 */
	public static String getSongPath(U1Song song) {
		StringBuilder path = new StringBuilder();
		path.append(File.separator);
		path.append(sanitize(song.getArtist()));
		path.append(File.separator);
		path.append(sanitize(song.getAlbum()));
		path.append(File.separator);
		path.append(sanitize(song.getTitle()));
		path.append(".");
		path.append(sanitize(song.getSuffix()));
		return path.toString();
	}
	
	public static String getSongPath(String artist, String album, String title, String suffix) {
		StringBuilder path = new StringBuilder();
		path.append(File.separator);
		path.append("u1music");
		path.append(File.separator);
		path.append(sanitize(artist));
		path.append(File.separator);
		path.append(sanitize(album));
		path.append(File.separator);
		path.append(sanitize(title));
		path.append(".");
		path.append(sanitize(suffix));
		return path.toString();
	}
	
	public static final File getLogFile() {
		final File extStorageDir =
				Environment.getExternalStorageDirectory();
		final String pkg =
				UbuntuOneMusic.class.getPackage().getName();
		final String dir =
				String.format("%s/Android/data/%s/files/log",
						extStorageDir, pkg);
		final File logDir = new File(dir);
		logDir.mkdirs();
		final File logFile = new File(logDir, "log.txt");
		return logFile;
	}
}
