package com.ubuntuone.android.music.util;

import static com.ubuntuone.android.music.util.StorageUtils.MiB;
import android.content.Context;
import android.content.res.TypedArray;
import android.preference.Preference;
import android.util.AttributeSet;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewParent;
import android.widget.RelativeLayout;
import android.widget.SeekBar;
import android.widget.SeekBar.OnSeekBarChangeListener;
import android.widget.TextView;

import com.ubuntuone.android.music.R;

public class SeekBarPreference extends Preference implements
		OnSeekBarChangeListener {
	private final String TAG = getClass().getName();

	private static final String ANDROID_NS = "http://schemas.android.com/apk/res/android";
	private static final String UBUNTUONE_NS = "http://schemas.one.ubuntu.com/apk/res/android";
	private static final int DEFAULT_VALUE = 50;

	private int mMaxValue = 100;
	private int mMinValue = 0;
	private int mInterval = 1;
	private int mCurrentValue;
	private SeekBar mSeekBar;

	private TextView mStatusText;

	public void setMinValue(int min) {
		mMinValue = min;
		mSeekBar.setMax(mMaxValue - mMinValue);
	}

	public void setMaxValue(int max) {
		mMaxValue = max;
		mSeekBar.setMax(mMaxValue - mMinValue);
	}

	public SeekBarPreference(Context context, AttributeSet attrs) {
		super(context, attrs);
		initPreference(context, attrs);
	}

	public SeekBarPreference(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);
		initPreference(context, attrs);
	}

	private void initPreference(Context context, AttributeSet attrs) {
		setValuesFromXml(attrs);
		mSeekBar = new SeekBar(context, attrs);
		mSeekBar.setMax(mMaxValue - mMinValue);
		mSeekBar.setOnSeekBarChangeListener(this);
	}

	private void setValuesFromXml(AttributeSet attrs) {
		mMaxValue = attrs.getAttributeIntValue(ANDROID_NS, "max", 100);
		mMinValue = attrs.getAttributeIntValue(UBUNTUONE_NS, "min", 0);

		try {
			String newInterval = attrs.getAttributeValue(UBUNTUONE_NS, "interval");
			if (newInterval != null) {
				mInterval = Integer.parseInt(newInterval);
			}
		} catch (Exception e) {
			Log.e(TAG, "Invalid interval value", e);
		}

	}

	@SuppressWarnings("unused")
	private String getAttributeStringValue(AttributeSet attrs,
			String namespace, String name, String defaultValue) {
		String value = attrs.getAttributeValue(namespace, name);
		if (value == null) {
			value = defaultValue;
		}
		return value;
	}

	@Override
	protected View onCreateView(ViewGroup parent) {
		RelativeLayout layout = null;
		try {
			LayoutInflater mInflater = (LayoutInflater) getContext()
					.getSystemService(Context.LAYOUT_INFLATER_SERVICE);

			layout = (RelativeLayout) mInflater.inflate(
					R.layout.seek_bar_preference, parent, false);
		} catch (Exception e) {
			Log.e(TAG, "Error creating seek bar preference", e);
		}
		return layout;
	}

	@Override
	public void onBindView(View view) {
		super.onBindView(view);
		try {
			ViewParent oldContainer = mSeekBar.getParent();
			ViewGroup newContainer = (ViewGroup) view
					.findViewById(R.id.seekBarPrefBarContainer);

			if (oldContainer != newContainer) {
				if (oldContainer != null) {
					((ViewGroup) oldContainer).removeView(mSeekBar);
				}
				newContainer.removeAllViews();
				newContainer.addView(mSeekBar,
						ViewGroup.LayoutParams.MATCH_PARENT,
						ViewGroup.LayoutParams.WRAP_CONTENT);
			}
		} catch (Exception ex) {
			Log.e(TAG, "Error binding view: " + ex.toString());
		}
		updateView(view);
	}

	/**
	 * Update a SeekBarPreference view with our current state
	 * 
	 * @param view
	 */
	protected void updateView(View view) {

		try {
			RelativeLayout layout = (RelativeLayout) view;
			mStatusText = (TextView) layout.findViewById(R.id.seekBarPrefValue);
			mStatusText.setMinimumWidth(30);
			mSeekBar.setProgress(mCurrentValue - mMinValue);

			mStatusText.setText(StorageUtils.getFuzzySize(mCurrentValue * MiB));
		} catch (Exception e) {
			Log.e(TAG, "Error updating seek bar preference", e);
		}

	}

	@Override
	public void onProgressChanged(SeekBar seekBar, int progress,
			boolean fromUser) {
		int newValue = progress + mMinValue;

		if (newValue > mMaxValue)
			newValue = mMaxValue;
		else if (newValue < mMinValue)
			newValue = mMinValue;
		else if (mInterval != 1 && newValue % mInterval != 0)
			newValue = Math.round(((float) newValue) / mInterval) * mInterval;

		mCurrentValue = newValue;
		if (mStatusText != null) {
			mStatusText.setText(StorageUtils.getFuzzySize(newValue * MiB));
		}
		persistInt(newValue);
	}

	@Override
	public void onStartTrackingTouch(SeekBar seekBar) {
	}

	@Override
	public void onStopTrackingTouch(SeekBar seekBar) {
		notifyChanged();
	}

	@Override
	protected Object onGetDefaultValue(TypedArray typedArray, int index) {
		int defaultValue = typedArray.getInt(index, DEFAULT_VALUE);
		return defaultValue;
	}

	@Override
	protected void onSetInitialValue(boolean restoreValue, Object defaultValue) {
		if (restoreValue) {
			mCurrentValue = getPersistedInt(mCurrentValue);
		} else {
			int value = 0;
			try {
				value = (Integer) defaultValue;
			} catch (Exception ex) {
				Log.e(TAG, "Invalid default value: " + defaultValue.toString());
			}
			persistInt(value);
			mCurrentValue = value;
		}
	}
}
