/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2013 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.util;

import static com.ubuntuone.android.music.util.LogUtils.makeLogTag;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Locale;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.SharedPreferences;
import android.os.Build;
import android.os.Environment;
import android.os.StatFs;
import android.preference.PreferenceManager;
import android.util.Log;

import com.ubuntuone.android.music.Constants;
import com.ubuntuone.android.music.Constants.Preference;

/** Utility functions to query and store information about the SDCard "external storage". */
public final class StorageUtils {
	private static final String TAG = makeLogTag(StorageUtils.class.getSimpleName());

	public static final String MUSIC_DIR = "u1music";

	public static final long MiB = 1024*1024;
	public static final long MIN_CACHE_SIZE = 100 * MiB;

	private StorageUtils() {}

	public static void setupMusicDirectory(Context context) {
		SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(context);
		String storage = preferences.getString(Preference.CACHE_LOCATION_KEY, null);
		if (storage == null) {
			final String[] storagePaths = StorageUtils.getAvailableStoragePaths();
			Log.i(TAG, "Available storage paths: " + Arrays.toString(storagePaths));
			if (storagePaths != null) {
				storage = storagePaths[0];
			} else {
				storage = getGenericCacheDirectory();
			}
			PreferenceManager.getDefaultSharedPreferences(context).edit()
					.putString(Constants.Preference.CACHE_LOCATION_KEY, storage).commit();
			
		} else {
			Log.w(TAG, "No storage paths matched on " + Build.MODEL);
		}

		long avail = getAvailableStorage(context, storage);
		long total = getTotalStorage(context, storage);
		Log.i(TAG, String.format("Storage stats: avail %s total %s",
				getFuzzySize(avail), getFuzzySize(total)));
	}

	/**
	 * Returns the directory used to store cached music collection located on the selected storage. 
	 * <br />
	 * Prerequisite: <b>setupMusicDirectory</b> must be called before calling this method,
	 * to initialize the default value of cache location preference.
	 * 
	 * @param context
	 * 
	 * @return Absolute path of the music storage directory.
	 */
	public static String getMusicDirectory(Context context) {
		SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(context);
		String cacheStorage = preferences.getString(Preference.CACHE_LOCATION_KEY, null);
		String cacheDirectory = preferences.getString(Preference.CACHE_DIR_KEY, MUSIC_DIR);
		if (cacheStorage == null) {
			throw new IllegalStateException("storage is null, have you called setupMusicDirectory?");
		}
		File storageDirectory = new File(cacheStorage);
		File musicDirectory = new File(storageDirectory, cacheDirectory);
		if (!musicDirectory.exists()) {
			musicDirectory.mkdirs();
		}
		return musicDirectory.getAbsolutePath();
	}

	/**
	 * @return Absolute path of the default, generic music storage directory.
	 */
	public static String getGenericCacheDirectory() {
		final File storageDirectory = Environment.getExternalStorageDirectory();
		final File genericCache = new File(storageDirectory,
				"Android/data/com.ubuntuone.android.music/files/cache");
		if (!genericCache.isDirectory()) {
			Log.i(TAG, "Creating cache directory.");
			genericCache.delete(); // Doesn't throw IOException
			genericCache.mkdirs();
		}
		return genericCache.getAbsolutePath();
	}
	
	public static String getExternalStoragePath() {
		return Environment.getExternalStorageDirectory().getAbsolutePath();
	}
	
	@SuppressLint("SdCardPath") // Yada yada lint. getExternalStorageDirectory() is NOT enough.
	private static final String[] knownStoragePaths = new String[] {
		"/storage/sdcard0",
		"/mnt/sdcard",
		"/mnt/sdcard2",
		"/mnt/sdcard-ext", // Motorola
		"/mnt/sdcard/external_sd", // Samsung
		"/mnt/ext_card", // Samsung
		"/sdcard/_ExternalSD"
	};
	
	public static String[] getAvailableStoragePaths() {
		ArrayList<String> pathList = new ArrayList<String>();
		for (String path : knownStoragePaths) {
			File storage = new File(path);
			if (storage.exists() && storage.isDirectory()) {
				pathList.add(storage.getAbsolutePath());
			}
		}
		int size = pathList.size();
		return pathList.toArray(new String[size]);
	}
	
	public static int getSelectedStorageIndex(String storage,
			String[] storagePaths) throws InvalidStorage {
		for (int i = 0; i < storagePaths.length; i++) {
			if (storage.equals(storagePaths[i]))
				return i;
		}
		throw new InvalidStorage();
	}
	
	public static long getAvailableStorage(Context context, String storage) {
		StatFs stat = new StatFs(getMusicDirectory(context));
		long blocksAvailable = stat.getAvailableBlocks();
		long blockSizeBytes = stat.getBlockSize();
		return blocksAvailable * blockSizeBytes;
	}
	
	/**
	 * total space - free space - used cache size
	 * 
	 * @param context
	 * @param storage
	 * @return
	 */
	public static long getAvailableCacheStorage(Context context, String storage, long cacheSize) {
		StatFs stat = new StatFs(getMusicDirectory(context));
		long blocksAvailable = stat.getAvailableBlocks();
		long blockSizeBytes = stat.getBlockSize();
		long total = getTotalStorage(context, storage);
		
		long avail = blocksAvailable * blockSizeBytes;
		return total - avail - cacheSize;
	}
	
	public static long getTotalStorage(Context context, String storage) {
		StatFs stat = new StatFs(getMusicDirectory(context));
		long blocksCount = stat.getBlockCount();
		long blockSizeBytes = stat.getBlockSize();
		return blocksCount * blockSizeBytes;
	}

	public static String getFuzzySize(long size) {
		String units = "B";
		String formatString = "%.0f %s";
		double s = size;
		if (s > 1024.0) {
			s /= 1024.0;
			units = "KiB";
		}
		if (s > 1024.0) {
			s /= 1024.0;
			units = "MiB";
			formatString = "%.02f %s";
		}
		if (s > 1024.0) {
			s /= 1024.0;
			units = "GiB";
		}
		return String.format(Locale.US, formatString, s, units);
	}

	public static class InvalidStorage extends Exception {
		private static final long serialVersionUID = -3042684172473773638L;
		public InvalidStorage() { super(); }
		public InvalidStorage(String msg) { super(msg); }
	}

	public static class NoTertiaryStorage extends Exception {
		private static final long serialVersionUID = -5605241960717201218L;
		public NoTertiaryStorage() { super(); }
		public NoTertiaryStorage(String msg) { super(msg); }
	}
}
