/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.util;

import java.io.File;
import java.io.IOException;
import java.lang.ref.WeakReference;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

import android.graphics.Bitmap;
import android.os.AsyncTask;
import android.util.Log;
import android.view.View;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.ImageView;

import com.ubuntuone.android.music.BuildConfig;
import com.ubuntuone.android.music.R;
import com.ubuntuone.android.music.UbuntuOneMusic;
import com.ubuntuone.api.music.U1MusicAPI;
import com.ubuntuone.api.music.client.Failure;
import com.ubuntuone.api.music.request.U1ArtListener;

/**
 * Fetches images from U1 thumbnailing API.
 */
public class U1ImageDownloader
{
	private static final String TAG = U1ImageDownloader.class.getSimpleName();
	
	private static final String ART_DIR = "art";
	private static final String NO_MEDIA = ".nomedia";
	
	private static final int TYPE_ARTIST = 1;
	private static final int TYPE_ALBUM = 2;
	
	protected U1MusicAPI mApi;
	protected Executor mExecutor = Executors.newFixedThreadPool(4);
	
	private static U1ImageDownloader sArtistArtDownloader;
	private static U1ImageDownloader sAlbumArtDownloader;
	
	private ArtDownloader mArtDownloader;
	private File mCacheDir;
	
	public static U1ImageDownloader getArtistArtDownloader() {
		if (sArtistArtDownloader == null) {
			sArtistArtDownloader = new U1ImageDownloader(TYPE_ARTIST);
		}
		return sArtistArtDownloader;
	}
	
	public static U1ImageDownloader getAlbumArtDownloader() {
		if (sAlbumArtDownloader == null) {
			sAlbumArtDownloader = new U1ImageDownloader(TYPE_ALBUM);
		}
		return sAlbumArtDownloader;
	}
	
	private U1ImageDownloader(int type) {
		this.mCacheDir = new File(FileUtilities.getCacheDirectory(), ART_DIR);
		
		setupMusicApi();
		setupCacheDirectory(mCacheDir);
		
		switch (type) {
		case TYPE_ARTIST:
			this.mArtDownloader = new ArtistArtDownloader();
			break;
		case TYPE_ALBUM:
			this.mArtDownloader = new AlbumArtDownloader();
			break;
		}
	}
	
	private void setupCacheDirectory(File cacheDir) {
		if (!cacheDir.exists()) {
			// Create cache directory.
			cacheDir.mkdirs();
			// Make MediaScanner ignore the cache directory.
			File noMedia = new File(cacheDir, NO_MEDIA);
			try {
				noMedia.createNewFile();
			} catch (IOException e) {
				// Ignore.
			}
		}
	}
	
	private void setupMusicApi() {
		try {
			mApi = UbuntuOneMusic.getInstance().getMusicApi();
		} catch (NotAuthenticatedException e) {
			Log.w(TAG, "Not authenticated.");
		}
	}
	
	public void getArt(final String id, final ImageView imageView) {
		if (id == null) {
			imageView.setImageResource(R.drawable.default_art);
			return;
		}
		if (id.equals(imageView.getTag())) {
			return;
		} else {
			// We will be setting a new image. Cancel any pending animation.
			Animation animation = imageView.getAnimation();
			if (animation != null) {
				animation.cancel();
			}
			imageView.setImageDrawable(null);
		}
		
		imageView.setVisibility(View.INVISIBLE);
		imageView.setTag(id);
		File imageFile = getThumbnailFile(id, false);
		
		if (imageFile.exists()) {
			Log.v(TAG, "Using art file for " + id);
			
			new AsyncTask<File, Void, Bitmap>() {
				@Override
				protected void onPreExecute() {
					imageView.setImageResource(R.drawable.default_art);
				}

				@Override
				protected Bitmap doInBackground(File... params) {
					File imageFile = params[0];
					final Bitmap bitmap = BitmapUtils.decodeFile(
							new File(imageFile.getAbsolutePath()), 256);
					return bitmap;
				}

				@Override
				protected void onPostExecute(Bitmap bitmap) {
					if (id.equals(imageView.getTag())) {
						imageView.setImageBitmap(bitmap);
						imageView.setVisibility(View.VISIBLE);
						if (!BuildConfig.DEBUG) {
							Animation animation = AnimationUtils.loadAnimation(
									UbuntuOneMusic.getContext(), android.R.anim.fade_in);
							if (id.equals(imageView.getTag())) {
								imageView.startAnimation(animation);
							}
						}
					}
				}
			}.execute(imageFile);
		} else {
			Log.i(TAG, "Will fetch art for " + id);
			downloadArtAsync(id, imageView);
		}
	}
	
	protected void downloadArtAsync(final String id,
			final ImageView imageView) {
		final WeakReference<ImageView> imageViewRef =
				new WeakReference<ImageView>(imageView);
		
		Runnable downloadRunnable = new Runnable() {
			@Override
			public void run() {
				setupCacheDirectory(mCacheDir);
				downloadArt(id, imageViewRef);
			}
		};
		mExecutor.execute(downloadRunnable);
	}
	
	protected void downloadArt(final String id,
			final WeakReference<ImageView> imageViewRef) {
		
		final File partFile = getThumbnailFile(id, true);
		final File fullFile = getThumbnailFile(id, false);
		
		U1ArtListener apiListener = new U1ArtListener() {
			@Override
			public void onStart() {
				Log.d(TAG, String.format("Starting art request for %s.", id));
			}
			
			@Override
			public void onSuccess(File file) {
				partFile.renameTo(fullFile);
				final ImageView imageView = imageViewRef.get();
				if (imageView != null && id.equals(imageView.getTag())) {
					final Bitmap bitmap = BitmapUtils.decodeFile(
							new File(fullFile.getAbsolutePath()), 256);
					imageView.post(new Runnable() {
						@Override
						public void run() {
							imageView.setImageBitmap(bitmap);
							imageView.setVisibility(View.VISIBLE);
							Animation animation = AnimationUtils.loadAnimation(
									UbuntuOneMusic.getContext(), android.R.anim.fade_in);
							if (id.equals(imageView.getTag())) {
								imageView.startAnimation(animation);
							}
						}
					});
				}
			}
			
			@Override
			public void onFailure(Failure failure) {
				Log.w(TAG, String.format("Art request failed. %s",
						failure.getMessage()));
			}
			
			@Override
			public void onFinish() {
				Log.d(TAG, String.format("Finished art request for %s.", id));
			}
		};
		
		mArtDownloader.getArt(id, partFile.getAbsolutePath(), apiListener);
	}
	
	protected File getThumbnailFile(String key, boolean isPartial) {
		File cacheDir = new File(FileUtilities.getCacheDirectory(), ART_DIR);
		String filename = null;
		if (isPartial) {
			filename = String.format("%s.jpg.part", key);
		} else {
			filename = String.format("%s.jpg", key);
		}
		return new File(cacheDir, FileUtilities.sanitize(filename));
	}
	
	public boolean isThumbnailCached(String key) {
		return getThumbnailFile(key, false).exists();
	}
	
	private interface ArtDownloader {
		public void getArt(String id, String filePath, U1ArtListener callback);
	}
	
	private class ArtistArtDownloader implements ArtDownloader {
		@Override
		public void getArt(String id, String filePath, U1ArtListener callback) {
			mApi.getArtistArt(id, filePath, callback);
		}
	}
	
	private class AlbumArtDownloader implements ArtDownloader {
		@Override
		public void getArt(String id, String filePath, U1ArtListener callback) {
			mApi.getAlbumArt(id, filePath, callback);
		}
	}
}
