package com.ubuntuone.android.music.provider;

import static com.ubuntuone.android.music.provider.MusicProviderUtils.QUEUE_URI;
import android.database.Cursor;
import android.net.Uri;

import com.ubuntuone.android.music.MockData;
import com.ubuntuone.android.music.provider.MusicContract.Albums;
import com.ubuntuone.android.music.provider.MusicContract.Artists;
import com.ubuntuone.android.music.provider.MusicContract.PlaylistSongs;
import com.ubuntuone.android.music.provider.MusicContract.Playlists;
import com.ubuntuone.android.music.provider.MusicContract.Songs;
import com.ubuntuone.android.music.util.HashUtils;

public class MusicProviderUtilsTest extends MusicProviderMockDataTestBase {
	
	public MusicProviderUtilsTest() {
		super(MusicProvider.class, MusicContract.CONTENT_AUTHORITY);
	}
	
	public void testGetArtistSongCount() {
		String artistId = HashUtils.md5("artist1-artist_name");
		Uri artistUri = Artists.buildArtistUri(artistId);
		int songCount = MusicProviderUtils.getArtistSongCount(mContext, artistUri);
		assertEquals("Wrong artist song count.", 2, songCount);
	}
	
	public void testGetAlbumSongCount() {
		String albumId = HashUtils.md5("album1-album_titleartist1-artist_name");
		Uri albumUri = Albums.buildAlbumUri(albumId);
		int songCount = MusicProviderUtils.getAlbumSongCount(mContext, albumUri);
		assertEquals("Wrong album song count.", 2, songCount);
	}
	
	public void testGetSelectedSongCount() {
		int songCount = MusicProviderUtils.getSongCount(mContext, null);
		assertEquals("Wrong total song count.", MockData.SONGS_COUNT, songCount);
	}
	
	public void testGetSongCount() {
		int songCount = MusicProviderUtils.getSongCount(mContext);
		assertEquals("Wrong total song count.", MockData.SONGS_COUNT, songCount);
	}
	
	public void testGetStarredSongCount() {
		int songCount = MusicProviderUtils.getStarredSongCount(mContext);
		assertEquals("Wrong starred song count.", MockData.STARRED_SONGS_COUNT, songCount);
	}
	
	public void testGetSongStarred() {
		Uri songUri = Songs.buildSongUri("song2-song_id");
		
		int starred = MusicProviderUtils.getSongStarred(mContext, songUri);
		assertEquals("Wrong song starred state.", 0, starred);
	}
	
	public void testSetSongStarred() {
		Uri songUri = Songs.buildSongUri("song2-song_id");
		String[] projection = new String[] { Songs.STARRED };
		
		Cursor cursor = mResolver.query(songUri, projection, null, null, null);
		cursor.moveToFirst();
		assertEquals("Wrong song starred state.",
				0, cursor.getInt(cursor.getColumnIndex(Songs.STARRED)));
		cursor.close();
		
		MusicProviderUtils.setSongStarred(mContext, songUri, true);
		
		cursor = mResolver.query(songUri, projection, null, null, null);
		cursor.moveToFirst();
		assertEquals("Wrong song starred state.",
				1, cursor.getInt(cursor.getColumnIndex(Songs.STARRED)));
		cursor.close();
		
		MusicProviderUtils.setSongStarred(mContext, songUri, false);
		
		cursor = mResolver.query(songUri, projection, null, null, null);
		cursor.moveToFirst();
		assertEquals("Wrong song starred state.",
				0, cursor.getInt(cursor.getColumnIndex(Songs.STARRED)));
		cursor.close();
	}
	
	public void testToggleSongStarred() {
		Uri songUri = Songs.buildSongUri("song2-song_id");
		String[] projection = new String[] { Songs.STARRED };
		
		Cursor cursor = mResolver.query(songUri, projection, null, null, null);
		cursor.moveToFirst();
		assertEquals("Wrong song starred state.",
				0, cursor.getInt(cursor.getColumnIndex(Songs.STARRED)));
		cursor.close();
		
		MusicProviderUtils.toggleSongStarred(mContext, songUri);
		
		cursor = mResolver.query(songUri, projection, null, null, null);
		cursor.moveToFirst();
		assertEquals("Wrong song starred state.",
				1, cursor.getInt(cursor.getColumnIndex(Songs.STARRED)));
		cursor.close();
		
		MusicProviderUtils.toggleSongStarred(mContext, songUri);
		
		cursor = mResolver.query(songUri, projection, null, null, null);
		cursor.moveToFirst();
		assertEquals("Wrong song starred state.",
				0, cursor.getInt(cursor.getColumnIndex(Songs.STARRED)));
		cursor.close();
	}
	
	public void testGetCachedSongCount() {
		int songCount = MusicProviderUtils.getCachedSongCount(mContext);
		assertEquals("Wrong starred song count.", MockData.CACHED_SONGS_COUNT, songCount);
	}
	
	public void testGetPlaylistSongCount() {
		Uri playlistUri = Playlists.buildPlaylistUri("playlist1-playlist_id");
		int songCount = MusicProviderUtils.getPlaylistSongCount(mContext, playlistUri);
		assertEquals("Wrong playlist song count.", 2, songCount);
	}
	
	public void testGetPlayQueueSongCount() {
		Uri playlistUri = MusicProviderUtils.QUEUE_URI;
		int songCount = MusicProviderUtils.getPlaylistSongCount(mContext, playlistUri);
		assertEquals("Wrong play queue song count.", 1, songCount);
	}
	
	public void testClearPlaylist() {
		Uri playlistUri = Playlists.buildPlaylistSongsUri("playlist1-playlist_id");
		int songCount = MusicProviderUtils.getPlaylistSongCount(mContext, playlistUri);
		assertTrue("Playlist should not be empty.", songCount != 0);
		
		MusicProviderUtils.clearPlaylist(mContext, playlistUri);
		
		songCount = MusicProviderUtils.getPlaylistSongCount(mContext, playlistUri);
		assertEquals("Playlist not empty.", 0, songCount);
	}
	
	public void testClearPlayQueue() {
		Uri playlistUri = MusicProviderUtils.QUEUE_URI;
		int songCount = MusicProviderUtils.getPlaylistSongCount(mContext, playlistUri);
		assertTrue("Play queue should not be empty.", songCount != 0);
		
		MusicProviderUtils.clearPlayQueue(getMockContext());
		
		songCount = MusicProviderUtils.getPlaylistSongCount(mContext, playlistUri);
		assertTrue("Play queue not empty.", songCount == 0);
	}
	
	public void testDeletePlaylist() {
		Uri playlistUri = Playlists.buildPlaylistSongsUri("playlist2-playlist_id");
		int songCount = MusicProviderUtils.getPlaylistSongCount(mContext, playlistUri);
		assertTrue("Playlist should not be empty.", songCount > 0);
		
		MusicProviderUtils.deletePlaylist(mContext, playlistUri);
		
		songCount = MusicProviderUtils.getPlaylistSongCount(mContext, playlistUri);
		assertEquals("No songs should have been returned.", 0, songCount);
		
		Cursor cursor = mResolver.query(playlistUri, new String[] { Playlists._ID },
				null, null, null);
		int rowsReturned = cursor.getCount();
		cursor.close();
		assertEquals("Playlist should not exists.", 0, rowsReturned);
	}
	
	public void testDeletePlaylistDoesNotDeletePlayQueue() {
		Uri playlistUri = MusicProviderUtils.QUEUE_URI;
		int songCount = MusicProviderUtils.getPlayQueueSongCount(mContext);
		assertTrue("Play queue should not be empty.", songCount > 0);
		
		MusicProviderUtils.deletePlaylist(mContext, playlistUri);
		
		int newSongCount = MusicProviderUtils.getPlaylistSongCount(mContext, playlistUri);
		assertEquals("No songs should have been deleted.", songCount, newSongCount);
		
		Cursor cursor = mResolver.query(playlistUri, new String[] { Playlists._ID },
				null, null, null);
		int rowsReturned = cursor.getCount();
		cursor.close();
		assertEquals("Play queue should still exists.", 1, rowsReturned);
	}
	
	public void testEnqueueSongInPlaylist() {
		Uri playlistUri = Playlists.buildPlaylistUri("playlist2-playlist_id");
		int songCount = MusicProviderUtils.getPlaylistSongCount(mContext, playlistUri);
		assertEquals("Wrong song count of playlist.", 1, songCount);
		
		final String newSongId = "song1-song_id";
		Uri songUri = Songs.buildSongUri("song1-song_id");
		MusicProviderUtils.enqueueSong(mContext, songUri, playlistUri);
		
		int newSongCount = MusicProviderUtils.getPlaylistSongCount(mContext, playlistUri);
		assertEquals("Wrong new song count of playlist.", songCount + 1, newSongCount);
		
		Uri playlistSongsUri = Playlists.buildPlaylistSongsUri("playlist2-playlist_id");
		Cursor cursor = null;
		try {
			String[] projection = new String[] { Songs.SONG_ID };
			cursor = mResolver.query(playlistSongsUri, projection,
					null, null, PlaylistSongs.DEFAULT_SORT);
			if (cursor != null && cursor.isBeforeFirst()) {
				cursor.moveToLast();
				String songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
				assertEquals("Wrong queued song id.", newSongId, songId);
			}
		} finally {
			if (cursor != null) cursor.close();
		}
	}
	
	public void testClearAndEnqueueSong() {
		final String newSongId = "song1-song_id";
		Uri songUri = Songs.buildSongUri("song1-song_id");
		
		MusicProviderUtils.clearAndEnqueueSong(mContext, songUri);
		
		int songCount = MusicProviderUtils.getPlayQueueSongCount(mContext);
		assertEquals("Wrong new song count of play queue.", 1, songCount);
		
		Cursor cursor = null;
		try {
			String[] projection = new String[] { Songs.SONG_ID };
			cursor = mResolver.query(QUEUE_URI, projection,
					null, null, PlaylistSongs.DEFAULT_SORT);
			if (cursor != null && cursor.isBeforeFirst()) {
				cursor.moveToFirst();
				String songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
				assertEquals("Wrong queued song id.", newSongId, songId);
			}
		} finally {
			if (cursor != null) cursor.close();
		}
	}

	public void testEnqueueAlbumInPlaylist() {
		Uri playlistUri = Playlists.buildPlaylistUri("playlist2-playlist_id");
		int songCount = MusicProviderUtils.getPlaylistSongCount(mContext, playlistUri);
		assertEquals("Wrong song count of playlist.", 1, songCount);
		
		final String newAlbumId = HashUtils.md5("album1-album_titleartist1-artist_name");
		Uri albumUri = Albums.buildAlbumUri(newAlbumId);
		int albumSongCount = MusicProviderUtils.getAlbumSongCount(mContext, albumUri);
		MusicProviderUtils.enqueueAlbum(mContext, albumUri, playlistUri);
		
		int newSongCount = MusicProviderUtils.getPlaylistSongCount(mContext, playlistUri);
		assertEquals("Wrong new song count of playlist.", songCount + albumSongCount, newSongCount);
		
		Uri playlistSongsUri = Playlists.buildPlaylistSongsUri("playlist2-playlist_id");
		Cursor cursor = null;
		try {
			String[] projection = new String[] { Songs.SONG_ID };
			cursor = mResolver.query(playlistSongsUri, projection,
					null, null, PlaylistSongs.DEFAULT_SORT);
			if (cursor != null && cursor.isBeforeFirst()) {
				cursor.moveToFirst();
				
				cursor.moveToNext();
				String songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
				assertEquals("Wrong queued album song id.", "song1-song_id", songId);
				
				cursor.moveToNext();
				songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
				assertEquals("Wrong queued album song id.", "song2-song_id", songId);
			}
		} finally {
			if (cursor != null) cursor.close();
		}
	}

	public void testClearAndEnqueueAlbum() {
		Uri playlistUri = MusicProviderUtils.QUEUE_URI;
		int songCount = MusicProviderUtils.getPlayQueueSongCount(mContext);
		assertEquals("Wrong song count of play queue.", 1, songCount);
		
		final String newAlbumId = HashUtils.md5("album1-album_titleartist1-artist_name");
		Uri albumUri = Albums.buildAlbumUri(newAlbumId);
		int albumSongCount = MusicProviderUtils.getAlbumSongCount(mContext, albumUri);
		MusicProviderUtils.clearAndEnqueueAlbum(mContext, albumUri);
		
		int newSongCount = MusicProviderUtils.getPlayQueueSongCount(mContext);
		assertEquals("Wrong new song count of play queue.", albumSongCount, newSongCount);
		
		Cursor cursor = null;
		try {
			String[] projection = new String[] { Songs.SONG_ID };
			cursor = mResolver.query(playlistUri, projection,
					null, null, PlaylistSongs.DEFAULT_SORT);
			if (cursor != null && cursor.isBeforeFirst()) {
				cursor.moveToNext();
				String songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
				assertEquals("Wrong queued album song id.", "song1-song_id", songId);
				
				cursor.moveToNext();
				songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
				assertEquals("Wrong queued album song id.", "song2-song_id", songId);
			}
		} finally {
			if (cursor != null) cursor.close();
		}
	}
	
	public void testEnqueueArtistInPlaylist() {
		Uri playlistUri = Playlists.buildPlaylistUri("playlist2-playlist_id");
		int songCount = MusicProviderUtils.getPlaylistSongCount(mContext, playlistUri);
		assertEquals("Wrong song count of playlist.", 1, songCount);
		
		String newArtistId = HashUtils.md5("artist1-artist_name");
		Uri artistUri = Artists.buildArtistUri(newArtistId);
		int artistSongCount = MusicProviderUtils.getArtistSongCount(mContext, artistUri);
		assertTrue("Artist song count should be non zero.", artistSongCount > 0);
		MusicProviderUtils.enqueueArtist(mContext, artistUri, playlistUri);
			
		int newSongCount = MusicProviderUtils.getPlaylistSongCount(mContext, playlistUri);
		assertEquals("Wrong new song count of playlist.", songCount + artistSongCount, newSongCount);
		
		Uri playlistSongsUri = Playlists.buildPlaylistSongsUri("playlist2-playlist_id");
		Cursor cursor = null;
		try {
			String[] projection = new String[] { Songs.SONG_ID };
			cursor = mResolver.query(playlistSongsUri, projection,
					null, null, PlaylistSongs.DEFAULT_SORT);
			if (cursor != null && cursor.isBeforeFirst()) {
				cursor.moveToFirst();
				
				cursor.moveToNext();
				String songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
				assertEquals("Wrong queued album song id.", "song1-song_id", songId);
				
				cursor.moveToNext();
				songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
				assertEquals("Wrong queued album song id.", "song2-song_id", songId);
			}
		} finally {
			if (cursor != null) cursor.close();
		}
	}
	
	public void testClearAndEnqueueArtist() {
		Uri playlistUri = MusicProviderUtils.QUEUE_URI;
		int songCount = MusicProviderUtils.getPlayQueueSongCount(mContext);
		assertEquals("Wrong song count of play queue.", 1, songCount);
		
		String newArtistId = HashUtils.md5("artist1-artist_name");
		Uri artistUri = Artists.buildArtistUri(newArtistId);
		int artistSongCount = MusicProviderUtils.getArtistSongCount(mContext, artistUri);
		assertTrue("Artist song count should be non zero.", artistSongCount > 0);
		MusicProviderUtils.clearAndEnqueueArtist(mContext, artistUri);
			
		int newSongCount = MusicProviderUtils.getPlayQueueSongCount(mContext);
		assertEquals("Wrong new song count of play queue.", artistSongCount, newSongCount);
		
		Cursor cursor = null;
		try {
			String[] projection = new String[] { Songs.SONG_ID };
			cursor = mResolver.query(playlistUri, projection,
					null, null, PlaylistSongs.DEFAULT_SORT);
			if (cursor != null && cursor.isBeforeFirst()) {
				cursor.moveToNext();
				String songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
				assertEquals("Wrong queued album song id.", "song1-song_id", songId);
				
				cursor.moveToNext();
				songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
				assertEquals("Wrong queued album song id.", "song2-song_id", songId);
			}
		} finally {
			if (cursor != null) cursor.close();
		}
	}
}
