package com.ubuntuone.android.music.ui;
import android.app.ActionBar;
import android.app.Activity;
import android.app.Instrumentation;
import android.app.Instrumentation.ActivityMonitor;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.net.Uri;
import android.support.v4.view.ViewPager;
import android.test.ActivityInstrumentationTestCase2;
import android.view.KeyEvent;
import android.widget.ListView;

import com.ubuntuone.android.music.MockData;
import com.ubuntuone.android.music.R;
import com.ubuntuone.android.music.provider.MusicContract.Songs;
import com.ubuntuone.android.music.provider.MusicDatabase;
import com.ubuntuone.android.music.provider.MusicProviderUtils;
import com.ubuntuone.android.test.UIUtils;

public class SongsPageTest extends ActivityInstrumentationTestCase2<HomeActivity>
{
	private Instrumentation mInstrumentation;
	private ActivityMonitor mPlayerActivityMonitor;
	private ActivityMonitor mPlaylistsActivityMonitor;

	private SQLiteDatabase mDatabase;
	private HomeActivity mActivity;
	private ActionBar mActionBar;
	private ListView mList;

	public SongsPageTest() {
		super(HomeActivity.class);
	}

	@Override
	protected void setUp() throws Exception {
		super.setUp();
		
		mInstrumentation = getInstrumentation();
		
		mDatabase = new MusicDatabase(getActivity()).getWritableDatabase();
		MockData.setUp(mDatabase);
		
		mPlayerActivityMonitor = new ActivityMonitor(
				PlayerActivity.class.getName(), null, false);
		mInstrumentation.addMonitor(mPlayerActivityMonitor);
		
		mPlaylistsActivityMonitor = new ActivityMonitor(
				PlaylistsActivity.class.getName(), null, false);
		mInstrumentation.addMonitor(mPlaylistsActivityMonitor);

		mActivity = getActivity();
		mActivity.onProgress(R.id.song_loader_id); // Simulate finished sync.
		mActionBar = mActivity.getActionBar();
		
		mInstrumentation.waitForIdleSync();
	}

	@Override
	protected void tearDown() throws Exception {
		mInstrumentation.removeMonitor(mPlayerActivityMonitor);
		mInstrumentation.removeMonitor(mPlaylistsActivityMonitor);
		
		if (!mActivity.isFinishing()) mActivity.finish();
		mActionBar = null;
		mActivity = null;
		
		MockData.tearDown(mDatabase);
		mDatabase.close();
		mDatabase = null;
		
		super.tearDown();
	}

	private void openSongsTab() {
		UIUtils.openTab(mActionBar, 2);
		ViewPager pager = (ViewPager) mActivity.findViewById(R.id.pager);
		mList = (ListView) pager.findViewWithTag("song_list");
	}

	private void skipListHeader() {
		sendKeys(KeyEvent.KEYCODE_DPAD_DOWN);
		sendKeys(KeyEvent.KEYCODE_DPAD_DOWN);
	}
	
	public void testPlayQueueExists() {
		int songCount = MusicProviderUtils.getPlayQueueSongCount(mActivity);
		assertTrue(MusicProviderUtils.QUEUE_NAME + " does not exist.", songCount != -1);
	}
	
	public void testClearQueueAndShuffleAllOnShuffleAllButtonClick() {
		mActivity.runOnUiThread(new Runnable() {
			public void run() {
				openSongsTab();
				mList.requestFocus();
			}
		});

		mInstrumentation.waitForIdleSync();
		mInstrumentation.sendKeyDownUpSync(KeyEvent.KEYCODE_DPAD_DOWN);
		
		// --- Click Shuffle all button ---
		mInstrumentation.sendKeyDownUpSync(KeyEvent.KEYCODE_DPAD_CENTER);
		
		assertTrue("Shuffle all songs task not launched.", mActivity._shuffleAllSongsTaskLaunched);

		// {@link PlayerActivity} should have launched.
		Activity activity = mPlayerActivityMonitor.waitForActivityWithTimeout(5 * 1000);
		assertTrue("PlayerActivity was not started",
				mInstrumentation.checkMonitorHit(mPlayerActivityMonitor, 1));
		activity.finish();
		
		mInstrumentation.waitForIdleSync();
	}

	public void testClearQueueAndPlaySongOnOnListItemClick() {
		mActivity.runOnUiThread(new Runnable() {
			public void run() {
				openSongsTab();
				mList.requestFocus();
			}
		});

		mInstrumentation.waitForIdleSync();
		skipListHeader();

		// --- Click on a song ---
		mInstrumentation.sendKeyDownUpSync(KeyEvent.KEYCODE_DPAD_CENTER);
		
		// Play queue should contain one song.
		int songCount = MusicProviderUtils.getPlayQueueSongCount(mActivity);
		assertEquals("Wrong play queue song count.", 1, songCount);

		// {@link PlayerActivity} should have launched.
		Activity activity = mPlayerActivityMonitor.waitForActivityWithTimeout(5 * 1000);
		assertTrue("PlayerActivity was not started",
				mInstrumentation.checkMonitorHit(mPlayerActivityMonitor, 1));
		activity.finish();
		
		mInstrumentation.waitForIdleSync();
	}
	
	public void testOnPlayContextItemSelected() {
		mActivity.runOnUiThread(new Runnable() {
			public void run() {
				openSongsTab();
				mList.requestFocus();
			}
		});

		mInstrumentation.waitForIdleSync();
		skipListHeader();

		// --- Open song context menu ---
		UIUtils.longClickDpadCenter(mInstrumentation);
		
		// --- Select Play ---
		mInstrumentation.sendKeyDownUpSync(KeyEvent.KEYCODE_DPAD_CENTER);
		
		// Play queue should contain one song.
		int songCount = MusicProviderUtils.getPlayQueueSongCount(mActivity);
		assertEquals("Wrong play queue song count.", 1, songCount);

		// {@link PlayerActivity} should have launched.
		Activity activity = mPlayerActivityMonitor.waitForActivityWithTimeout(5 * 1000);
		assertTrue("PlayerActivity was not started",
				mInstrumentation.checkMonitorHit(mPlayerActivityMonitor, 1));
		activity.finish();
		
		mInstrumentation.waitForIdleSync();
	}
	
	public void testOnStarContextItemSelected() {
		mActivity.runOnUiThread(new Runnable() {
			public void run() {
				openSongsTab();
				mList.requestFocus();
			}
		});
		
		mInstrumentation.waitForIdleSync();
		skipListHeader();
		
		final int position = mList.getFirstVisiblePosition();
		Cursor cursor = (Cursor) mList.getItemAtPosition(position);
		
		String songId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ID));
		Uri songUri = Songs.buildSongUri(songId);
		int isStarred = MusicProviderUtils.getSongStarred(mActivity, songUri);
		
		// --- Open song context menu ---
		UIUtils.longClickDpadCenter(mInstrumentation);
		
		// --- Select Star ---
		mInstrumentation.sendKeyDownUpSync(KeyEvent.KEYCODE_DPAD_DOWN);
		mInstrumentation.sendKeyDownUpSync(KeyEvent.KEYCODE_DPAD_CENTER);
		
		int newIsStarred = MusicProviderUtils.getSongStarred(mActivity, songUri);
		assertEquals("Song star status not toggled.", 1 - isStarred, newIsStarred);
	}
	
//	public void testAddToPlaylistSongContextMenuOption() {
//		mActivity.runOnUiThread(new Runnable() {
//			public void run() {
//				openSongsTab();
//				mList.requestFocus();
//			}
//		});
//
//		mInstrumentation.waitForIdleSync();
//		skipListHeader();
//
//		// --- Long press on a song ---
//		UIUtils.longClickDpadCenter(mInstrumentation);
//		
//		// --- Select the 'Add to playlist' menu option ---
//		UIUtils.sleep(500);
//		sendKeys(KeyEvent.KEYCODE_DPAD_DOWN);
//		UIUtils.sleep(500);
//		sendKeys(KeyEvent.KEYCODE_DPAD_DOWN);
//		UIUtils.sleep(500);
//		sendKeys(KeyEvent.KEYCODE_DPAD_CENTER);
//		
//		// Play queue should contain one song.
//		int songCount = MusicProviderUtils.getPlayQueueSongCount(mActivity);
//		assertEquals("Wrong play queue song count.", 1, songCount);
//
//		// {@link PlaylistsActivity} should have launched.
//		Activity activity = mPlaylistsActivityMonitor.waitForActivityWithTimeout(5 * 1000);
//		assertNotNull("Activity was not started", activity);
//		
//		// activity.getList().requestFocus() needed?
//		// Select first playlist.
//		UIUtils.sleep(500);
//		sendKeys(KeyEvent.KEYCODE_DPAD_DOWN);
//		UIUtils.sleep(500);
//		sendKeys(KeyEvent.KEYCODE_DPAD_CENTER);
//		
//		// First playlist should now have 1 more element.
//		// TODO
//	
//		activity.finish();
//	}
}
